#!/usr/bin/env python
# coding: utf8
# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)

from Imports_Basic import *
from OpenCV import *
from PlotGallery import *
from HDT.HarpDataTree import *

#==========================================================================================================================
#============================================= MINING OF BIT CONSUMPTION ==================================================
#==========================================================================================================================


#====================================
def get_Bits_SideParam(POC, Mode="Inter", SideParam="Coeffs"):
#====================================
    #Chose sideparam from:
    #SideParams_Coeff = "Coeffs",
    #SideParams_Other = "SkipFlag", "SkipMergeIndex", "PredMode", "PartSize", "IntraIPCM", "PredInfoAllPUs", 

    Canvas = getCanvas(POC, np.float, NumCh=1)
    Canvas = Canvas * np.NAN
    for CTU in POC["CTUs"]:
        for CU in CTU["CUs_FINAL"]:
            if CU["Mode"] == Mode:    
                Bits = CU["Bits"][SideParam]
#                         if Mode == "Inter": #DEBUG
#                             show_Object(CU["Bits"])
                if not math.isnan(Bits):
                #if Bits != np.NAN: #NAN means: not set, so not used in bitstream
                    Idcs = get_Idcs_Y(CU)
                    CU_Area = CU["Size"][0] * CU["Size"][1]
                    Canvas[Idcs] = Bits / np.float(CU_Area)
    return Canvas

# ======================================
def mine_Consumed_Bits(POC, ConsumerNames, PrintDesc):
# ======================================
    ModeNames = ("Intra", "Inter" , "Merge", "Skip")
    
    ConsumedBits = {}
    ConsumedBits["Sum"] = 0
    ConsumedBits["Sum_SliceBits"] = POC["Bits_POC"]
    for Mode in ModeNames:
        printLARGE("Mining " + PrintDesc + " " + Mode, 2)
        ConsumedBits[Mode] = {}
        ConsumedBits[Mode]["Sum"] = 0
        ConsumedBits[Mode]["Map_Total"] = getCanvas(POC, np.float, NumCh=1)
        
        for ConsumerName in ConsumerNames: 
            print "Mining ", ConsumerName
            Map = get_Bits_SideParam(POC, Mode, ConsumerName)
            ConsumedBits[Mode][ConsumerName] = {}
            ConsumedBits[Mode][ConsumerName]["Map"] = Map
            #Sum = np.nan_to_num(Map)
            Sum = np.nansum(Map)
            if not np.isfinite(Sum): #workaround: Numpy <= 1.8 returns Nan if all values are NaN :(
                Sum = 0
            ConsumedBits[Mode][ConsumerName]["Sum"] = Sum
            ConsumedBits[Mode]["Sum"] += Sum
            ConsumedBits[Mode]["Map_Total"] += np.nan_to_num(Map)
            
        # ==========
        # SHARES OF EACH CONSUMER
        # ==========
        ConsumedBits[Mode]["ConsumerNames"]  = ConsumerNames
        ConsumedBits[Mode]["ConsumerBits"]   = []
        ConsumedBits[Mode]["ConsumerShares"] = []
        for ConsumerName in ConsumerNames:
            ConsumedBits[Mode]["ConsumerBits"].append(ConsumedBits[Mode][ConsumerName]["Sum"])
            try:
                ConsumedBits[Mode]["ConsumerShares"].append(ConsumedBits[Mode][ConsumerName]["Sum"] / ConsumedBits[Mode]["Sum"] * 100)
            except:                                        
                ConsumedBits[Mode]["ConsumerShares"].append(None)
        #ConsumedBits[Mode]["Map_Total"][ConsumedBits[Mode]["Map_Total"]==0]=np.nan #workaround: restore NaNs for pretty plotting
        BackCheck = np.nansum(ConsumedBits[Mode]["Map_Total"])
        
        # ==========
        # STATISTICS CONSUMERS
        # ==========
        print "ConsumerNames: ", ConsumedBits[Mode]["ConsumerNames"]
        print "ConsumerBits: ",  ConsumedBits[Mode]["ConsumerBits"]
        print "ConsumerShares in %%: ", ConsumedBits[Mode]["ConsumerShares"]
        print 'ConsumedBits[%s]["Sum"]: ' % Mode, ConsumedBits[Mode]["Sum"]
        print 'ConsumedBits[%s]["Sum over MapTotal"] (BACKCHECK): ' % Mode, BackCheck
        assert BackCheck == ConsumedBits[Mode]["Sum"]
        ConsumedBits["Sum"] += ConsumedBits[Mode]["Sum"]   

    ConsumedBits["Sum_OtherSliceInfo"] = ConsumedBits["Sum_SliceBits"] - ConsumedBits["Sum"] 
    #ConsumedBits["Sum_OtherSliceInfo"]
        
    #show_Object(ConsumedBits)
     
    # ==========
    # SHARES OF EACH MODE
    # ==========
    ConsumedBits["ModeNames"]  = ModeNames
    ConsumedBits["ModeShares"] = []
    ConsumedBits["ModeBits"]   = []
    for Mode in ModeNames:
        ConsumedBits["ModeBits"].append(ConsumedBits[Mode]["Sum"])
        ConsumedBits["ModeShares"].append(ConsumedBits[Mode]["Sum"] / ConsumedBits["Sum"] * 100)
        
    # ==========
    # STATISTICS MODES
    # ==========
    printLARGE("Statistics for " + PrintDesc, 2)
    print "ModeNames: ", ConsumedBits["ModeNames"]
    print "ModeBits: ", ConsumedBits["ModeBits"]
    print "ModeShares in %%: ", ConsumedBits["ModeShares"]
    print 'ConsumedBits["Sum_SliceBits"]: ', ConsumedBits["Sum_SliceBits"]
    print 'ConsumedBits["Sum"]: ', ConsumedBits["Sum"]
    print 'ConsumedBits["Sum_OtherSliceInfo"]: ', ConsumedBits["Sum_OtherSliceInfo"]
    print 'Percentage of untracked bits: ', ConsumedBits["Sum_OtherSliceInfo"] / ConsumedBits["Sum_SliceBits"] * 100    
    return  ConsumedBits

# ======================================
def mine_Differences_Consumed_Bits(Epic, Anchor, ConsumerNames):
# ======================================
    #NOTE: we assume Epic is better, so we subtract "Epic-Anchor" in order to get negative values

    Diff_Epic = deepcopy(Epic) #deepcopy: otherwise lists can not be subtracted!!
    ModeNames = ("Intra", "Inter" , "Merge", "Skip")
    
    for Mode in ModeNames:
        for ConsumerName in ConsumerNames:
            Diff_Epic[Mode][ConsumerName]["Map"] = Epic[Mode][ConsumerName]["Map"] - Anchor[Mode][ConsumerName]["Map"]         
        Diff_Epic[Mode]["ConsumerBits"]          = list(np.asarray(Epic[Mode]["ConsumerBits"]) - np.asarray(Anchor[Mode]["ConsumerBits"]))
        Diff_Epic[Mode]["Map_Total"]             = Epic[Mode]["Map_Total"] - Anchor[Mode]["Map_Total"] 

    return Diff_Epic

#==========================================================================================================================
#==================================================== PLOTTING ============================================================
#==========================================================================================================================

#==========================================
def plot_CoeffSinfo_BarPlots(POCIdcs, df_coeff, df_sinfo): 
#==========================================

    
    bbox_x = 1.02


    printLARGE("plot_coeff_sinfo_bars")
    print "POCIdcs:", POCIdcs

    fig, axarr = plt.subplots(3, 1, figsize=(A4_x *0.7,A4_y/2)) 
    my_colors1 = ['r', 'b', 'g', 'y']
    
    # ============================
    # BARPLOT COEFFS (hatched)
    # ============================
    ax = axarr[0]
    ax.set_title("Coefficient Bits")
    df_coeff.plot(kind='bar', color=my_colors1, ax=ax, rot=0)


    # hatching
    for patch in ax.patches:
        patch.set_hatch('\\')
        patch.set_edgecolor("white")
    
    handles, labels = ax.get_legend_handles_labels()
    handles = []
    for labidx, label in enumerate(labels):
        patch = matplotlib.patches.Patch(color=my_colors1[labidx], label=label)
        patch.set_hatch('\\')
        patch.set_edgecolor("white")
        handles.append(patch)
#     for patch in patches:
#         patch.set_hatch('\\')
#         patch.set_edgecolor("white")
    ax.legend(handles, labels, loc='upper right', fontsize=10, bbox_to_anchor=(bbox_x, 1.1))
        
    # ============================
    # BARPLOT SINFO
    # ============================        
    ax = axarr[1]
    ax.set_title("Sideinfo Bits")
    df_sinfo.plot(kind='bar', color=my_colors1, ax=ax, rot=0)
    patches, labels = ax.get_legend_handles_labels()
    ax.legend(patches, labels, loc='upper right', fontsize=10, bbox_to_anchor=(bbox_x, 1.1))   

#     near_patch = Patch(facecolor=[168/256,175/256,175/256], 
#                    label='Nearshore', hatch ='o')
#     red_patch = mpatches.Patch(color='red', label='The red data')
#     plt.legend(handles=[red_patch])

    
    # ============================
    # STACKPLOT
    # ============================
    ax = axarr[2]
    ax.set_title("Coefficient vs Sideinfo Bits, Stacked")
    
    # xticks
    ticks =  []
    ticklabels = []
    for POCIdx in POCIdcs:
        ticks.append(POCIdx) 
        ticklabels.append("POC%d" % POCIdx)
    
    IntraCoeffs = np.asarray(df_coeff.loc[:,'Intra'].values, dtype=np.float)
    InterCoeffs = np.asarray(df_coeff.loc[:,'Inter'].values, dtype=np.float)
    SkipCoeffs  = np.asarray(df_coeff.loc[:,'Skip'].values, dtype=np.float)
    MergeCoeffs = np.asarray(df_coeff.loc[:,'Merge'].values, dtype=np.float)
    IntraOther = np.asarray(df_sinfo.loc[:,'Intra'].values, dtype=np.float)
    InterOther = np.asarray(df_sinfo.loc[:,'Inter'].values, dtype=np.float)
    SkipOther  = np.asarray(df_sinfo.loc[:,'Skip'].values, dtype=np.float)
    MergeOther = np.asarray(df_sinfo.loc[:,'Merge'].values, dtype=np.float) 
    
    # Debug sum plot
    List = IntraCoeffs, InterCoeffs, MergeCoeffs, SkipCoeffs, IntraOther, InterOther, MergeOther, SkipOther
    Sum = 0
    for BitPerPOC in List:
        Sum += BitPerPOC
    print "POCIdcs: ", POCIdcs
    print "Overall bits per POC: ", Sum
    #DEBUG PLOT
    #figtest, axarrtest = plt.subplots(1, 1, figsize=(A4_x,A4_y/2)) 
    #axarrtest.plot(POCIdcs, Sum)
    #plt.show()
    
    

    stack_coll = ax.stackplot(POCIdcs, *List, colors=(my_colors1+my_colors1))
    ax.grid(True)
        
    plt.xticks(ticks, ticklabels)
    # hatching
    for i in range(0,4):
        stack_coll[i].set_hatch('\\')
        stack_coll[i].set_edgecolor("white")
#     for i in range(4,8):
#         stack_coll[i].set_hatch('o')
#         stack_coll[i].set_edgecolor("white")

    #legend for stackplot (tricky... see proxy_rects)
    LegendLabels = "Intra-C", "Inter-C", "Merge-C", "Skip-C", "Intra-S", "Inter-S", "Merge-S", "Skip-S"
    #proxy_rects = [matplotlib.patches.Rectangle((0, 0), 1, 1, fc=pc.get_facecolor()[0]) for pc in stack_coll]
    my_colors1.extend(my_colors1)
    handles = []
    for labidx, label in enumerate(LegendLabels):
        patch = matplotlib.patches.Patch(color=my_colors1[labidx], label=label)
        if labidx <= 3:
            patch.set_hatch('\\')
            patch.set_edgecolor("white")
        handles.append(patch)    

#     patch1 = matplotlib.patches.Patch(edgecolor="k", facecolor="w", label="Coefficents")    
#     patch1.set_hatch('\\')
#     patch2 = matplotlib.patches.Patch(edgecolor="k", facecolor="w",  label="Sideinfo")     
#     #patch2.set_hatch('o')
#     handles.extend((patch1, patch2))     
#     LegendLabels = "Coefficents", "Sideinfo"
    
    ax.legend(handles, LegendLabels, ncol=1, fontsize=8, loc='upper right', bbox_to_anchor=(bbox_x, 1.1)) #, title = "Coefficients vs Sideinfo")    
    plt.tight_layout()    
    return fig, axarr

# =====================================
def plot_Consumed_Bits_OVERVIEW(POC, ConsumedBits, ConsumerNames):
# ======================================          
    printLARGE("POC%d: Plotting Consumed_Bits_OVERVIEW" % POC["Idx"], 1)

    # ==========
    # PIE CHART MODE-LEVEL
    # ==========  
    print "Plotting pie chart" 
    Gallery  = PlotGallery(3,2, DimX=A4_x, DimY=A4_y*1.0) 
    ax = Gallery.axarr[0, 1]
    colors = scaleColors( (IntraColor, InterColor, MergeColor, SkipColor, GRAY) )
    Data       = ConsumedBits["ModeBits"]
    DataLabels = ConsumedBits["ModeNames"]
    Data.append(POC["Bits_POC"] -  np.nansum(np.asarray(ConsumedBits["ModeBits"])))
    DataLabels += ("Other slice info",)            
    
    ax.pie(Data, labels=DataLabels, colors=colors, 
             autopct='%1.1f%%', shadow=False, startangle=0, radius=1.05, pctdistance=0.85)  #autopct='%1.1f%%',
    ax.set_aspect(1.0) #lower: stretch sideways
    #ax.set_title("POC bit consumption")        
    #ax.axis('equal') # prevent ellipse
    #ax.axis([0, 0.8, 0, 0.8])

    # ==========
    # 5x MODE VIS
    # ==========      
    print "Plotting 5x mode vis"        
    O =  (1,0), (1,1), (2, 0), (2,1), #(3,0), (3,1)  #(0,0), (0,1),  #USE AS: *O[ib+1]
    Modes_Vis = getCanvas(POC)
    for CTU in POC["CTUs"]:
        for CU in CTU["CUs_FINAL"]:
            draw_CU(CU, Modes_Vis)  
    Gallery.subplot(0, 0, Modes_Vis, "All modes")   
    
    ModeNames = ("Intra", "Inter" , "Merge", "Skip")
    for idx, Mode in enumerate(ModeNames):     
        Modes_Vis = getCanvas(POC)
        for CTU in POC["CTUs"]:
            for CU in CTU["CUs_FINAL"]:
                if CU["Mode"] == Mode:
                    draw_CU(CU, Modes_Vis)  
        Gallery.subplot(O[idx][0], O[idx][1], Modes_Vis, "Modes " + Mode )       
    
    plt.tight_layout()
    return Gallery, Gallery.fig, Gallery.axarr    

# ======================================
def plot_Consumed_Bits_DETAIL(Mode, POC, ConsumedBits, ConsumerNames, diffMode = False, anc_POC = None):
# ======================================

    printLARGE("Plotting Consumed_Bits_DETAIL: Mode " + Mode, 1)
    
    Gallery  = PlotGallery(3,3, DimX=A4_y, DimY=A4_x) 

    Modes_Vis = getCanvas(POC)
    for CTU in POC["CTUs"]:
        for CU in CTU["CUs_FINAL"]:
            if CU["Mode"] == Mode:
                draw_CU(CU, Modes_Vis)  
    Gallery.subplot(0, 0, Modes_Vis, "a) Modes " + Mode, )    
    
    if diffMode == True:
        print "DIFFERENCE MODE ACTIVE"
        Prefix = "Differences: "
        
        Modes_Vis_ANC = getCanvas(anc_POC)
        for CTU in anc_POC["CTUs"]:
            for CU in CTU["CUs_FINAL"]:
                if CU["Mode"] == Mode:
                    draw_CU(CU, Modes_Vis_ANC)  
        Gallery.subplot(0, 1, Modes_Vis_ANC, "b) Modes " + Mode, )          
        
    else: Prefix = ""        

    # ==========
    def get_cmap(Map, diffMode):
    # ==========
        # special 0-centered colormap for the "diff" case:
        if diffMode:
            orig_cmap = matplotlib.cm.jet
            midpoint, isNegative = get_colormap_midpoint(Map) #just check if we can use a "0-centered color map"
            cmap = shiftedColorMap(orig_cmap, start=0, midpoint=midpoint, stop=1.0, name='shifted')
        else:
            cmap = getCustomCMap_RedToGreen()
        return cmap                    
        
    #O =  (1,0), (1,1), (2, 0), (2,1), (3,0), (3,1)  #(0,0), (0,1),  #USE AS: *O[idx]
    O =  (1,0), (1,1), (1,2),(2, 0), (2,1), (2,2)
    enum = "c) ", "d) ", "e) ", "f) ", "g) ", "h) "
    for ib, ConsumerName in enumerate(ConsumerNames):
        Map = ConsumedBits[Mode][ConsumerName]["Map"]

        cmap = get_cmap(Map, diffMode)
        #cmap = "jet"

        if Mode == "Skip" and ConsumerName == "Coeffs": #special case
            Map = get_Bits_SideParam(POC, "Skip", "SkipMergeIndex")
            Gallery.subplot(*O[ib], Matrix=Map, Label=enum[ib] + Prefix+"SkipMergeIndex", cmap=cmap)               
        else:
            Gallery.subplot(*O[ib], Matrix=Map, Label=enum[ib] + Prefix+ConsumerName, cmap=cmap)  
    
    
    #adding total sum over all consumer maps
    if 0:
        Map = ConsumedBits[Mode]["Map_Total"]
        cmap = get_cmap(Map, diffMode)
        Gallery.subplot(1, 0, Map, Prefix + "All", cmap)                      


    # ==========
    # PIE CHART CONSUMER-LEVEL (OFF!)
    # ==========                   
    if 0:
        ax = Gallery.axarr[0, 1]
        ax.pie(ConsumedBits[Mode]["ConsumerShares"], labels=ConsumedBits[Mode]["ConsumerNames"],
                autopct='%1.1f%%', shadow=False, startangle=0)
        ax.axis('equal') # prevent ellipse
    
    # ==========
    # BAR PLOT CONSUMER-LEVEL
    # ==========               
    if 0:
        ax = Gallery.axarr[0, 1]
        ax.axis('on') 
        GroupNames = ("Bitstream consumption", ) #will show as x-axis ticks
        BarNames   = ConsumedBits[Mode]["ConsumerNames"]
        df=pd.DataFrame([ConsumedBits[Mode]["ConsumerBits"],], columns=BarNames, index=GroupNames)
        ax2 = df.plot(kind='bar', ax=ax)
        #ax.set_xlabel("Bitstream consumption")
        ax.set_ylabel("Bits")
        #ax.set_position([0.1,0.1, 0.9, 0.9])
        #ax.set_xticklabels( ["Bitstream consumption",], rotation=0 )
        ax.set_xticklabels( ["\n\n",], rotation=0 )
        #ax.get_legend().get_title().set_fontsize('3')
        ax.set_title(Prefix + "Bit Consumption " + "Modes " + Mode , size='large', weight="bold")
        
        ax.legend(loc='center left', bbox_to_anchor=(0.7, 0.5), fancybox=True, shadow=True, ncol=1)
        plt.setp(ax.get_legend().get_texts(), fontsize='10')
        vals = ConsumedBits[Mode]["ConsumerBits"]
        ax.set_ylim([np.min(vals), np.max(vals) + np.max(vals)  / 100*70])
        #ax.set_ymargin(0.9)
        #ax.autoscale(False)

        #bbox_to_anchor=(1.1, 1.05)
        #Gallery.fig.canvas.draw()
    
    plt.tight_layout()
    return Gallery, Gallery.fig, Gallery.axarr    

    
    