#!/usr/bin/env python
# coding: utf8
# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)

from Imports_Basic import *
from OpenCV import *
from PlotGallery import *
#from Editra.src.extern.pygments.lexers._phpbuiltins import idx
from Pandas import *
import math

#==========================================================================================================================
#=================================================== INDEXING =============================================================
#==========================================================================================================================

#==========================================
def subset(List, Key, Val):
#========================================== 
    for Obj in List:
        if Obj[Key] == Val:
            yield Obj
 
#==========================================            
def get_Idcs(AnyObject):
#==========================================
    Pos, Size = AnyObject["Pos"], AnyObject["Size"]
    x1, x2 = Pos[0], Pos[0] + Size[0]
    y1, y2 = Pos[1], Pos[1] + Size[1]  
    idcs = np.index_exp[y1:y2, x1:x2, :]
    return idcs

#==========================================            
def get_Idcs_Y(AnyObject):
#==========================================
    Pos, Size = AnyObject["Pos"], AnyObject["Size"]
    x1, x2 = Pos[0], Pos[0] + Size[0]
    y1, y2 = Pos[1], Pos[1] + Size[1]  
#     print x1, x2
#     print y1, y2
    idcs = np.index_exp[y1:y2, x1:x2]
    return idcs

#==========================================            
def get_Idcs_UV(AnyObject): #subsampling
#==========================================
    Pos, Size = AnyObject["Pos"], AnyObject["Size"]
    x1, x2 = Pos[0], Pos[0] + Size[0]
    y1, y2 = Pos[1], Pos[1] + Size[1]  
    idcs = np.index_exp[y1/2:y2/2, x1/2:x2/2]
    return idcs

#DEPRECATED
def get_index_args(Object):
#==========================================
    Pos, Size = Object["Pos"], Object["Size"]
    x1, x2 = Pos[0], Pos[0] + Size[0]
    y1, y2 = Pos[1], Pos[1] + Size[1]  
    return x1, x2, y1, y2

#==========================================================================================================================
#=================================================== HELPERS ==============================================================
#==========================================================================================================================

#==========================================            
def get_POC_Idcs(POCs):
#==========================================
    POC_Idcs = []
    for POC in POCs:
        POC_Idcs.append(POC["Idx"])
    return POC_Idcs


#==========================================            
def unpickle_POCs(RunDir, POCIdcs):
#==========================================
    printLARGE("Unpickling POCs")
    POCs = {}
    POCs["POCIdcs"] = POCIdcs
    for POCIdx in POCIdcs:
        HDT_FN = RunDir + "/POC_%05d.hdt" % POCIdx
        POC = load_CPickle_FN(HDT_FN, silent=False)         
        POCs[POCIdx] = POC        
    return POCs

#==========================================            
def NEW_unpickle_POCs(RunDir, POCIdcs):
#==========================================
    printLARGE("Unpickling POCs")
    POCs = []
    for POCIdx in POCIdcs:
        HDT_FN = RunDir + "/POC_%05d.hdt" % POCIdx
        POC = load_CPickle_FN(HDT_FN, silent=False)         
        POCs.append(POC)        
    return POCs

#==========================================            
def load_POC(RunDir, POCIdx): #load single POC
#==========================================
    #printLARGE("Loading POC %d" % POCIdx)
    HDT_FN = RunDir + "/POC_%05d.hdt" % POCIdx
    POC = load_CPickle_FN(HDT_FN, silent=False)              
    return POC

#==========================================================================================================================
#====================================== DATAMINE BITS AND BUFFERS =========================================================
#==========================================================================================================================

#=====================
def get_Abs_MV(Pos, Size, RelMV):
#=====================
    Start = np.asarray(Pos) + ( np.asarray(Size) / 2.0) #centering in PU
    End   = Start + np.asarray(RelMV) / 4.0
    return (Start,End)

#==========================================
def get_PU_RGB_Pixels(PIC, PU):
#==========================================
    idcsY  = get_Idcs_Y(PU) 
    idcsUV = get_Idcs_UV(PU) 
    Y,U,V = PIC["Y"][idcsY], PIC["U"][idcsUV], PIC["V"][idcsUV]
    U = cv2.resize(U, None, fx=2.0, fy=2.0, interpolation=cv2.INTER_LINEAR)
    V = cv2.resize(V, None, fx=2.0, fy=2.0, interpolation=cv2.INTER_LINEAR)
    YUV = np.dstack((Y, U, V))
    YUV = YUV.astype(np.uint8)
    RGB = cv2.cvtColor(YUV, cv2.COLOR_YUV2RGB)

    return RGB      

#==========================================
def get_POC_YUV_Pixels(POC, Buf = "YuvOrg"):
#==========================================
    Y,U,V = POC[Buf]["Y"],  POC[Buf]["U"],  POC[Buf]["V"]
    U = cv2.resize(U, None, fx=2.0, fy=2.0, interpolation=cv2.INTER_LINEAR)
    V = cv2.resize(V, None, fx=2.0, fy=2.0, interpolation=cv2.INTER_LINEAR)
    YUV = np.dstack((Y, U, V))
    YUV = YUV.astype(np.uint8)
    return YUV

#==========================================
def get_POC_RGB_Pixels(POC, Buf = "YuvOrg"):
#==========================================
    YUV = get_POC_YUV_Pixels(POC, Buf)
    RGB = cv2.cvtColor(YUV, cv2.COLOR_YUV2RGB)
    return RGB              

#==========================================
def get_CU_RecY(POC, CU):
#==========================================
    RecY = POC["YuvRec"]["Y"]
    
    Pos, Size = CU["Pos"], CU["Size"]
    x1, x2 = Pos[0], Pos[0] + Size[0]
    y1, y2 = Pos[1], Pos[1] + Size[1]  
    
    idcs = np.index_exp[y1:y2, x1:x2]  # nice fella!
    CU_RecY = RecY[idcs]
    return CU_RecY


#==========================================
def get_Bits(POC, POSTorFINAL="POST", BitsName="Coeffs", Mode = "Inter"):
#==========================================
    Canvas = getCanvas(POC, np.float, NumCh=1)
    Canvas = Canvas * np.NAN
    
    for CTU in POC["CTUs"]:
        for CU in CTU["CUs_FINAL"]:
            if CU["Mode"] == Mode:    
                Bits = CU["Bits"][BitsName]
                Idcs = get_Idcs_Y(CU)
                CU_Area = CU["Size"][0] * CU["Size"][1]
                Canvas[Idcs] = Bits / np.float(CU_Area)
                
#                 Skip = False
#                 #focus ONLY on fully-affine CUs
#                 for PU in CU["PUs"]:
#                     if PU["Motion"]["Type"] == "TRANSL":
#                         Skip = True
#                 if Skip:
#                     continue
#                 print "minVal, maxVal, minLoc, maxLoc:", cv2.minMaxLoc(M)   
        
    return Canvas

# DEPRECATED, USE DATAMINING "CONSUMEDBITS.PY"
# STILL USED BY GWRF5_BARPLOTS.PY
#==========================================
def get_DF_Coeff_SInfo(POCs, addGWRFBits=False):
#==========================================  

    # for Bar Plots
    #==========================================
    def get_Bits_Sum(POC, POSTorFINAL="POST", Mode = "Inter"):
    #==========================================  
        
        SideParams_Coeff = "Coeffs",
        SideParams_Other = "SkipFlag", "SkipMergeIndex", "PredMode", "PartSize", "IntraIPCM", "PredInfoAllPUs", 
        
        def get_Bits_SideParam(SideParam):  
            sum = 0.0
            for CTU in POC["CTUs"]:
                for CU in CTU["CUs_FINAL"]:
                    if CU["Mode"] == Mode:    
                        Bits = CU["Bits"][SideParam]
                        if not math.isnan(Bits):
                        #if Bits != np.NAN: #NAN means: not set, so not used in bitstream
                            sum+= Bits
                            assert not math.isnan(sum)
            #print "returning ", sum                
            return sum                
            
        Sum_Coeff = 0.0
        for SideParam in SideParams_Coeff:
            Sum_Coeff += get_Bits_SideParam(SideParam)
            assert not math.isnan(Sum_Coeff)
            
        Sum_Other = 0.0
        for SideParam in SideParams_Other:
            Sum_Other += get_Bits_SideParam(SideParam)
            assert not math.isnan(Sum_Other)
            
        return Sum_Coeff, Sum_Other

    # Dataframe init
    Modes = "Intra", "Inter", "Merge", "Skip", 
    
    POC_Idcs = get_POC_Idcs(POCs)
    print POC_Idcs
    
    GroupNames = ["POC%d" % POCIdx for POCIdx in POC_Idcs]
    BarNames   = list(Modes)
    df_coeff = pd.DataFrame(columns=BarNames, index=GroupNames)
    df_sinfo = pd.DataFrame(columns=BarNames, index=GroupNames)
    
    POCBits = []
    for POC in POCs:
        
        POCBits.append(POC["Bits_POC"])

        Sum = 0
        for ModeIdx, Mode in enumerate(Modes):
            Sum_Coeff, Sum_Other = get_Bits_Sum(POC, "POST", Mode)
            Sum += Sum_Coeff + Sum_Other
            #df.at["POC%d" % POCIdx, Mode] = Bits_Sum
            #df.at["POC1", Mode] = 20
            #df.at["POC2", Mode] = np.NAN
            #print Bits_Sum

#             hack = 0
#             if Mode != "Intra" and addGWRFBits:
#                 hack = int(193/3) #equally spreading for illustration purposes
                
            df_coeff.at["POC%d" % POC["Idx"], Mode] = Sum_Coeff 
            df_sinfo.at["POC%d" % POC["Idx"], Mode] = Sum_Other #+ hack
            
        #print "POCBits:   ", POCBits[-1]
        #print "Summed Bits: ", Sum  
        
    
        
    return df_coeff, df_sinfo, POCBits

#==========================================================================================================================
#==================================================== DRAWING =============================================================
#==========================================================================================================================

#==========================================
def getCanvas(POC, type=np.uint8, NumCh=3):
#==========================================
    #DimX, DimY = POC["Size_inCTUs"][0]*64, POC["Size_inCTUs"][1]*64
    DimX, DimY = POC["Size"][0], POC["Size"][1]
    if NumCh == 1:
        return np.zeros( (DimY, DimX), type)
    else:
        return np.zeros((DimY, DimX, NumCh), type)

# Usage for multiple objects: create canvas, then call with Canvas = MyCanvas
#==========================================
def draw_Object(POC, AnyObject, Color = GREEN, Canvas = None, BGR_Alpha = 0.3, withBorder = True):
#==========================================
    if Canvas is None:
        Canvas = np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64,3),np.uint8)
        
    if AnyObject == None:
        print "Warning draw_Object: Object == None"
        return Canvas

    idcs = get_Idcs(AnyObject)
    Background = np.copy(Canvas[idcs]).astype(np.float32)
    Foreground = np.copy(Canvas[idcs]).astype(np.float32)
    Foreground = Color
    AlphaScalar = [BGR_Alpha, BGR_Alpha, BGR_Alpha]
    
    Canvas[idcs] = (Background*AlphaScalar + Foreground*(1.0 - np.asarray(AlphaScalar))).astype(np.uint8) 
    
    #WONT WORK! COPY IS CREATED
    #Block = Canvas[idcs]
    #Block = (Background*AlphaScalar + Foreground*(1.0 - np.asarray(AlphaScalar))).astype(np.uint8) 
  
    if withBorder:
        cv2.rectangle(Canvas[idcs], (0,0), (AnyObject["Size"][0]-1, AnyObject["Size"][1]-1), WHITE, 1)   
    
    #Canvas = Canvas.astype(np.uint8) 
    return Canvas

# Usage for multiple objects: create canvas, then call with Canvas = MyCanvas
#==========================================
def draw_CU(CU, Canvas):
#==========================================
    #Note: since MVs need 

    idcs = get_Idcs(CU)
    if CU["Mode"] == "Skip":  #skip: no PUs
        Canvas[idcs] = SkipColor
    if CU["Mode"] == "Intra":  
        Canvas[idcs] = IntraColor
    else:
        for PU in CU["PUs"]:
            draw_PU(PU, Canvas)              

    cv2.rectangle(Canvas[idcs], (0,0), (CU["Size"][0]-1, CU["Size"][1]-1), BLACK, 1)  
    return Canvas

#==========================================
def draw_PU(PU, Canvas, Color=None, Lines=WHITE):
#==========================================    
    idcs = get_Idcs(PU) 
    
    #note: skip has no PUs
    if Color == None:
        if PU["Mode"] == "Intra":
            Color = IntraColor
        elif PU["Mode"] == "Inter":
            #try:
                if "Type" in PU.keys(): #only special HARP next branches  have a motion "type"
                    if PU["Motion"]["Type"] == "TRANSL":
                        Color = InterColor
                    elif PU["Motion"]["Type"] == "AFFINE":
                        Color = DARKORANGE
                else:
                    Color = InterColor
            #except:
            #    show_Dict(PU)
        elif PU["Mode"] == "Merge":
            Color = MergeColor
        elif PU["Mode"] == "Skip":
            Color = SkipColor
        else:
            print "Warning: Unknown PU mode: ", PU["Mode"]
            Color = WHITE
    Canvas[idcs] = Color
    cv2.rectangle(Canvas[idcs], (0,0), (PU["Size"][0]-1, PU["Size"][1]-1), Lines, 1)      
#==========================================
def draw_PU_MV(PU, Canvas, Color=YELLOW):
#==========================================
    #behold: the most ugliest code ever...

    if PU["Mode"] != "Intra":  #REMOVE & DEBUG
        if "Type" in PU.keys(): #only special HARP next branches  have a motion "type"
            if PU["Motion"]["Type"] == "TRANSL":
                Start = np.asarray(PU["Pos"]) + ( np.asarray(PU["Size"]) / 2.0) #centering in PU
                End   = Start + np.asarray(PU["Motion"]["MV"]) / 4.0
                MV = (Start,End)    
                cv2.line(Canvas, tuple(MV[0].astype(int)), tuple(MV[1].astype(int)), Color, thickness=1, lineType=cv2.CV_AA, shift=0) 
        else:
            Start = np.asarray(PU["Pos"]) + ( np.asarray(PU["Size"]) / 2.0) #centering in PU
            End   = Start + np.asarray(PU["Motion"]["MV"]) / 4.0
            MV = (Start,End)    
            cv2.line(Canvas, tuple(MV[0].astype(int)), tuple(MV[1].astype(int)), Color, thickness=1, lineType=cv2.CV_AA, shift=0) 