#!/usr/bin/env python
# coding: utf8
# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)

from Imports_Basic import *
from OpenCV import *
from PlotGallery import *
from HDT.HarpDataTree import *
from Sequence import *
#from Motion.Warp import *

#==========================================
def draw_CU_Modes(POC):
#==========================================
    # VISUALIZE CU MODES
    VisModes = getCanvas(POC)
    for CTU in POC["CTUs"]:
        for CU in CTU["CUs_FINAL"]:
            draw_CU(CU, VisModes)  
    return VisModes  

#==========================================
def draw_Ref_Indices(POC, RefIdcsColors):
#==========================================           
    # VISUALIZE REF INDICES
    VisRefIndices = getCanvas(POC)
    for CTU in POC["CTUs"]:
        for CU in CTU["CUs_FINAL"]:  
            for PU in CU["PUs"]:
                if PU["Mode"] != "Intra":
                    RefIdx = PU["Motion"]["RefIdx"]
                    assert(RefIdx <= 3)
                    Color = RefIdcsColors[RefIdx]
#                     if RefIdx == len(POC["List0_Refs"]) -1: # //last one always yellow
#                         Color = RefIdcsColors[3]
                    #if (RefIdx == AvailableRefs-1) //last one always yellow
                    draw_PU(PU, VisRefIndices, Color, Lines=GRAY) 
    return VisRefIndices

#==========================================
def draw_MV_Vectors(POC, Canvas, Color=RED):
#==========================================        
    # AND ADD MOTION FIELD TO BOTH    
    Canvas = Canvas.copy()
    for CTU in POC["CTUs"]:
        for CU in CTU["CUs_FINAL"]:  
            for PU in CU["PUs"]:
                if PU["Mode"] != "Intra":
                    draw_PU_MV(PU, Canvas, Color=Color)
    return Canvas

#==========================================
def POST_drawMode_NewMC(POC, doPlot=True):
#==========================================
    Canvas1 = getCanvas(POC)
    
    Canvas2 = getCanvas(POC)
    for CTU in POC["CTUs"]:
        for CU in CTU["CUs_POST"]:
            if CU["Mode"] == "Inter": 
                if "PUs" in CU:
                    for PU in CU["PUs"]:
                        Idcs = get_Idcs_Y(PU)
                        Canvas2[Idcs] = BLUE 
    
    try:
        for CTU in POC["CTUs"]:
            for CU in CTU["CUs_POST"]:
                if CU["Mode"] == Mode:    
                    
                    if CU["Mode"] == "Inter": 
                        if "PUs" in CU:
                            for PU in CU["PUs"]:
                                #show_Object(CU)   
                                if "NewMC" in PU["Motion"]:
                                    if PU["Motion"]["NewMC"] == 1:
                                        Idcs = get_Idcs_Y(PU)
                                        Canvas1[Idcs] = GREEN 
                                        print "NewMC == 1!"
                                    else:
                                        Idcs = get_Idcs_Y(PU)
                                        Canvas1[Idcs] = RED 
                                    
                                    #print "NewMC found!"
    except:
        show_Object(CU) 
        raise                         
                
    if doPlot:
        Plot  = PlotGallery(2,2)       
        Plot.set_suptitle("POST-CUs")  
        Plot.subplot(0, 0, Canvas1,  " NewMC on/off ")
        Plot.subplot(1, 0, Canvas2, " Inter PUs ")
        Plot.save(HARP_TmpDir + "/POST2 SpecialVis POC%05d.png" % POC["Idx"], format = "png", dpi=100)
        Plot.show()
        close_All_Figures()    
        
    #return Canvas, Canvas2

#old name: POST_ICIP2016_InterAndIntraBits(POC, BN = ""):
#==========================================
def plot_coeff_sinfo_maps(POC, BN=""): 
#==========================================

    Intra_PredInfo = get_Bits(POC, "POST", "PredInfoAllPUs", "Intra")
    Intra_Coeffs   = get_Bits(POC, "POST", "Coeffs", "Intra")
    Intra_PartSize = get_Bits(POC, "POST", "PartSize", "Intra")
    Inter_PredInfo = get_Bits(POC, "POST", "PredInfoAllPUs", "Inter")
    Inter_Coeffs   = get_Bits(POC, "POST", "Coeffs", "Inter")
    Inter_PartSize = get_Bits(POC, "POST", "PartSize", "Inter")
    #Intra_PredMode = get_Bits(POC, "POST", "PredMode", "Intra"
    
    RedGreenCMap = getCustomCMap_RedToGreen()
    Plot  = PlotGallery(2,3)       
    Plot.set_suptitle("POST-CUs")
    #Plot.subplot(0, 0, Intra_Partitions, "POST (=FINAL) CU Partition")     
    Plot.subplot(0, 0, Intra_Coeffs, "Intra Coeff Bits", RedGreenCMap)   
    Plot.subplot(0, 1, Intra_PredInfo,  "Intra PredInfo Bits", RedGreenCMap)
    Plot.subplot(0, 2, Intra_PartSize, "Intra PartSize Bits", RedGreenCMap)  
    Plot.subplot(1, 0, Inter_Coeffs, "Inter Coeff Bits", RedGreenCMap)   
    Plot.subplot(1, 1, Inter_PredInfo,  "Inter PredInfo Bits", RedGreenCMap)
    Plot.subplot(1, 2, Inter_PartSize, "Inter PartSize Bits", RedGreenCMap)  
  
    
    #Plot.subplot(1, 1, PredMode, "CU PredMode Bits", RedGreenCMap)  
    #Plot.subplot(1, 2, Y_Ref0, "CU PredMode Bits", RedGreenCMap)  
    
    FN = KitTmpDir + "/" + BN + "POST CUs POC%05d.png" % POC["Idx"]
    Plot.save(FN, format = "png", dpi=100)
    #Plot.show()
    close_All_Figures()
    return FN  



#==========================================
def plot_RefIdcs_MVField_Buffers(POC):
#==========================================

# DELETE
#     # VISUALIZE CU PARTITIONS
#     Modes = getCanvas(POC)
#     for CTU in POC["CTUs"]:
#         for CU in CTU["CUs_FINAL"]:
#             draw_CU(CU, Modes)    
#                             
#     # VISUALIZE REF INDICES
#     RefIndices = getCanvas(POC)
#     for CTU in POC["CTUs"]:
#         for CU in CTU["CUs_FINAL"]:  
#             for PU in CU["PUs"]:
#                 if PU["Mode"] != "Intra":
#                     RefIdx = PU["Motion"]["RefIdx"]
#                     assert(RefIdx <= 3)
#                     Color = RefIdcsColors[RefIdx]
#                     if RefIdx == len(POC["List0_Refs"]) -1: # //last one always yellow
#                         Color = RefIdcsColors[3]
#                     #if (RefIdx == AvailableRefs-1) //last one always yellow
#                     draw_PU(PU, RefIndices, Color, Lines=GRAY) 
#     
#     # AND ADD MOTION FIELD TO BOTH    
#     for CTU in POC["CTUs"]:
#         for CU in CTU["CUs_FINAL"]:  
#             for PU in CU["PUs"]:
#                 if PU["Mode"] != "Intra":
#                     draw_PU_MV(PU, Modes, Color=RED)
#                     draw_PU_MV(PU, RefIndices, Color=RED)

    VisModes = draw_CU_Modes(POC)
    VisRefIndices = draw_Ref_Indices(POC, RefIdcsColors)
    VisModes = draw_MV_Vectors(POC, VisModes)
    VisRefIndices = draw_MV_Vectors(POC, VisRefIndices)

    # =================
    # FINAL BUFFERS
    # =================
    Y_Pred = get_FINAL_Buffer(POC, "Y_Pred")
    Y_Resi = get_FINAL_Buffer(POC, "Y_Resi")
    Y_Reco = get_FINAL_Buffer(POC, "Y_Reco")
    Y_Orig = get_FINAL_Buffer(POC, "Y_Orig")
    LostInSpace = Y_Reco - Y_Orig
    
    # =================
    # GET REF PIC
    # =================    
    Y_Cur  = POC["YuvOrg"]["Y"]
    if len(POC["List0_Refs"]) > 0: # for IDR frames
        Y_Ref0 = POC["List0_Refs"][0]["YuvOrg"]["Y"]
    else:
        Y_Ref0 = dc(Y_Cur)
        Y_Ref0[:,:] = np.NaN
    
                  
    Gallery  = PlotGallery(2,3)       
    Gallery.set_suptitle("FINAL-CUs")  
    Gallery.subplot(0, 0, Y_Cur,  "Cur frame (Luminance)")
    Gallery.subplot(1, 0, Y_Ref0, "Ref frame 0 (Luminance)")      
    Gallery.subplot(0, 1, VisModes, "Final CU Partition")
    Gallery.subplot(1, 1, VisRefIndices, "Final Ref-Indices")
    Gallery.subplot(0, 2, Y_Resi,      "Y-Resi")
    Gallery.subplot(1, 2, LostInSpace, "Y-Reco minus Y-Orig")
    Gallery.set_Limits(0, 2, -16.0, +16.0)  
    Gallery.set_Limits(1, 2, -16.0, +16.0)  
     
    #FN = KitTmpDir + "/" + prefix +  "FINAL CUs POC%05d_"% POC["Idx"]  + desc + "_" + suffix + ".png"
    #Plot.save( FN, format = "png")
    #copy_to_SubDir(FN, "FINAL CUs")
    #Plot.show()
    #close_All_Figures()
    plt.tight_layout()
    return Gallery, Gallery.fig, Gallery.axarr   

#get average "PredInfoBits" per PU
# PredInfoBits:
# ==============
# MergeFlag
# MergeIndex
# InterDirection (only for B-Frames)
# RefFrameIdx
# MVd
# MVPIdx
#========================================== 
def get_PUMapBits(POC, PUMotionType):
#==========================================
    print "get_PUMapBits()"
    DimX, DimY = POC["Size"]
    Map = np.zeros((DimY,DimX), np.float32)
    Map = Map * np.NAN

    Sum_PUBits = 0

    #from FINAL: from all Inter CUs : from all TRANSL motion types: for all PUs: get PredInfo

    #get_PUMapBits
    for CTU in POC["CTUs"]:
        for CU in CTU["CUs_FINAL"]:
            if CU["Mode"] == "Intra":
                continue
            
            for PU in CU["PUs"]:
                if PU["Motion"]["Type"] == PUMotionType: #eg: AFFINE, or: TRANSL

                    # get Pred-Info bits
                    PUBits = PU["Motion"]["Bits_PUPredInfo"]
                    Idcs = get_Idcs_Y(PU)
                    PU_Area = PU["Size"][0] * PU["Size"][1]
                    AvgPerSample = PUBits / np.float(PU_Area)
                    
                    # save to Matrix
                    Map[Idcs]   = AvgPerSample
                    Sum_PUBits += PUBits
            
    #print "minVal, maxVal, minLoc, maxLoc:", cv2.minMaxLoc(M)
    Res = dict(
        POCIdx = POC["Idx"],
        Map = Map,
        Sum_PUBits = Sum_PUBits,
        )
    return Res
     
# collect CU buffers, return POC-sized float buffer
#==========================================
def get_FINAL_Buffer(POC, BufferName = "Y_Resi", StageName="CUs_FINAL"):  
#==========================================   
    Buffer = getCanvas(POC, np.float, NumCh=1)
    for CTU in POC["CTUs"]:
        #for CU in CTU["CUs_FINAL"]:
        for CU in CTU[StageName]:   

            
            idcs = get_Idcs_Y(CU) 
            #try: 
            Buffer[idcs] = CU[BufferName]
            #except:
            #    SHOW((POC, CTU))
            
            #if POC["Idx"] == 5 and CTU["Idx"] == 18:
            #    print CU[BufferName]
            #    #SHOW((POC, CTU))
            
            # SPECIAL HACK TO GET INTRA RESIDUAL APPROXIMATION:
            if CU["Mode"] == "Intra": 
                Buffer[idcs] = CU["Y_Orig"].astype(np.float) - CU["Y_Pred"] # CONVENTION: ALWAYS ORIG-PRED
            
            
            
            if BufferName == "Y_Resi" and  np.abs(CU[BufferName]).sum() == 0:
                Buffer[idcs] = np.NaN  
                #assert 0 # bug was fixed. Was it?
    return Buffer
        
#UNUSED
#=========================================
def get_PUs_FIN_inside_CU_RDO(CTU, CU_RDO):
#==========================================
    
    Found_PUs = []
    for CU_FIN in CTU["CUs_FINAL"]:
        for PU_FIN in CU_FIN["PUs"]:
            
            #checking y range
            idcs_PU_FIN = get_Idcs(PU_FIN)[0] 
            idcs_CU_RDO = get_Idcs(CU_RDO)[0]
            sequ_PU_FIN = np.arange(idcs_PU_FIN.start, idcs_PU_FIN.stop, idcs_PU_FIN.step)
            sequ_CU_RDO = np.arange(idcs_CU_RDO.start, idcs_CU_RDO.stop, idcs_CU_RDO.step)
            isInsideY = len(np.intersect1d(sequ_PU_FIN, sequ_CU_RDO)) > 0
            
            #checking x range
            idcs_PU_FIN = get_Idcs(PU_FIN)[1] 
            idcs_CU_RDO = get_Idcs(CU_RDO)[1]
            sequ_PU_FIN = np.arange(idcs_PU_FIN.start, idcs_PU_FIN.stop, idcs_PU_FIN.step)
            sequ_CU_RDO = np.arange(idcs_CU_RDO.start, idcs_CU_RDO.stop, idcs_CU_RDO.step)
            isInsideX = len(np.intersect1d(sequ_PU_FIN, sequ_CU_RDO)) > 0
            
            # if PU_FIN is inside CU_RDO 
            if isInsideY and isInsideX:
                Found_PUs.append(PU_FIN)
                
    return Found_PUs

#UNUSED
#==========================================
def calculate_Area_of_FoundPUs(Found_PUs):
#==========================================    
    MeanArea = np.float64(0.0)
    for Found_PU in Found_PUs:
#         assert(CU["Size"] == (32,32))
#         Canvas = draw_Object(POC, CU, BLUE)
#         Canvas = draw_Object(POC, Found_PU, GREEN, Canvas)
#         saveBGR(Canvas, TmpDir + "/Canvas_CHECK%05d_PUAbsPartIdx%03d.png" % (cnt_CHECKS, Found_PU["AbsPartIdx"]))
        PU_Area = Found_PU["Size"][0] * Found_PU["Size"][1]
        MeanArea += PU_Area
        a = 0
    MeanArea /= len(Found_PUs)
    return MeanArea
    #print "Mean PU Area in CHECK CU %d: %0.2f" % (cnt_CHECKS, MeanArea)
        
#==========================================================================================================================
#==================================================== PLOTTING ============================================================
#==========================================================================================================================




#==========================================================================================================================
#==================================================== DEFUNC! ============================================================
#==========================================================================================================================

if 0: 
    #-------------------------------------
    # PLOT RDO RESI YUVS
    #-------------------------------------
    Plot_ResiYuvs = PlotGallery(4,4)
    Plot_ResiYuvs.set_suptitle("RDO CU BUFFERS: INTER-RESIDUALS " + "\n(for different PartSizes, at different depths, with the best MV and ref frame found, clipped at %d, %d)" % (Min, Max))
    for tmp in ResiYuvs["Depths"]:
        # each depth becomes a row
        row = tmp["Depth"]
        Plot_ResiYuvs.subplot(row, 0, tmp["ResiYuv_2Nx2N"], "ResiYuv_2Nx2N")
        Plot_ResiYuvs.subplot(row, 1, tmp["ResiYuv_Nx2N"], "ResiYuv_Nx2N")
        Plot_ResiYuvs.subplot(row, 2, tmp["ResiYuv_2NxN"], "ResiYuv_2NxN")
        Plot_ResiYuvs.subplot(row, 3, tmp["ResiYuv_NxN"], "ResiYuv_NxN")
      
        for col in range(0,4): #cols
            Plot_ResiYuvs.set_Limits(row, col, Min, Max)

    Plot_ResiYuvs.set_Labels_atCol(0, ("Depth0 (64x64)", "Depth1 (32x32)", "Depth2 (16x16)", "Depth3 (8x8)"))
    Plot_ResiYuvs.save(TmpDir + "/HDT_POC_%05d___RDO_InterResiduals.png" % POC["Idx"], format = "png", dpi=100)
    close_All_Figures()
     
    #-------------------------------------
    # PLOT RDO PRED YUVS
    #-------------------------------------
    Plot_PredYuvs = PlotGallery(4,4)
    Plot_PredYuvs.set_suptitle(PredYuvs["Description"])
    for tmp in PredYuvs["Depths"]:
        # each depth becomes a row
        row = tmp["Depth"]
        Plot_PredYuvs.subplot(row, 0, tmp["PredYuv_2Nx2N"], "PredYuv_2Nx2N")
        Plot_PredYuvs.subplot(row, 1, tmp["PredYuv_Nx2N"], "PredYuv_Nx2N")
        Plot_PredYuvs.subplot(row, 2, tmp["PredYuv_2NxN"], "PredYuv_2NxN")
        Plot_PredYuvs.subplot(row, 3, tmp["PredYuv_NxN"], "PredYuv_NxN")
    
        for col in range(0,4): #cols
            pass

    Plot_PredYuvs.set_Labels_atCol(0, ("Depth0 (64x64)", "Depth1 (32x32)", "Depth2 (16x16)", "Depth3 (8x8)"))
    Plot_PredYuvs.save(TmpDir + "/HDT_POC_%05d___RDO_InterPredictions.png" % POC["Idx"], format = "png", dpi=100)
    close_All_Figures()

if 0:
    #-------------------------------------
    # PLOT FINAL CSHOWS
    #-------------------------------------
    if POC.has_key("SHOW_PredictionUnits"): #may be switched off
        
        #============================================================ 
        printLARGE("FINAL_PredictionUnits")
        #============================================================
        
        Plot_CShows = PlotGallery(2,2)
        Plot_CShows.SelPos = None #hack, prevent messup in case of area selection
        Plot_CShows.set_suptitle("CShow Visualizations \n(see tmp/SinglePics)")
        Plot_CShows.subplot(0, 0, POC["SHOW_PredictionUnits"], "SHOW_PredictionUnits")
        Plot_CShows.subplot(0, 1, POC["SHOW_RefIndices"],      "SHOW_RefIndices")
        Plot_CShows.subplot(1, 0, POC["SHOW_Predictions"],     "SHOW_Predictions")
        Plot_CShows.subplot(1, 1, POC["SHOW_Residuals"],       "SHOW_Residuals")
        Plot_CShows.save(TmpDir + "/HDT_POC_%05d___FINAL_CShows.png" % POC["Idx"], format = "png", dpi=300)
        close_All_Figures()  
    
    del POC # Help to free memory in time: HD consumes RAM! Oh so much precious RAM! :)
    print "Done"  

