#!/usr/bin/env python
# coding: utf8
# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)

from Imports_Basic import *
from OpenCV import *
from PlotGallery import *
from Plot import * 
from HDT.HarpDataTree import *


#================================
def plot_RDO_Resi_Buffers(POC):
#================================
    ResiYuvs = collect_RDO_Buffers_ResiYuvs(POC) 
    
    #-------------------------------------
    # PLOT RDO RESI YUVS
    #-------------------------------------
    Min, Max =  -16, 16 #shown value range for Y differences
    Gallery = PlotGallery(2,3,DimX=A3q_x, DimY=A3q_y)
    Gallery.set_suptitle("RDO CU BUFFERS: INTER-RESIDUALS " + "\n(for different PartSizes, at different depths, with the best MV and ref frame found, clipped at %d, %d)" % (Min, Max))
    #for tmp in ResiYuvs["Depths"]:
    for row, Depth in enumerate((1,3)):
        # each depth becomes a row
        Gallery.subplot(row, 0, ResiYuvs["Depths"][Depth]["ResiYuv_2Nx2N"], "ResiYuv-2Nx2N", cmap="jet")
        Gallery.subplot(row, 1, ResiYuvs["Depths"][Depth]["ResiYuv_2Nx2N"], "ResiYuv-Nx2N", cmap="jet")
        Gallery.subplot(row, 2, ResiYuvs["Depths"][Depth]["ResiYuv_2Nx2N"], "ResiYuv-2NxN", cmap="jet")
      
        for col in range(0,3): #cols
            Gallery.set_Limits(row, col, Min, Max)

    Gallery.set_Labels_atCol(0, ("Depth1 (32x32)", "Depth3 (8x8)"))
    
    plt.tight_layout()
    return Gallery, Gallery.fig, Gallery.axarr 

#================================
def plot_RDO_Pred_Buffers(POC):
#================================
    PredYuvs = collect_RDO_Buffers_PredYuvs(POC)

    #-------------------------------------
    # PLOT RDO PRED YUVS
    #-------------------------------------
    Gallery = PlotGallery(4,4,DimX=A3q_x, DimY=A3q_y*1.1)
    Gallery.set_suptitle(PredYuvs["Description"])
    for tmp in PredYuvs["Depths"]:
        # each depth becomes a row
        row = tmp["Depth"]
        Gallery.subplot(row, 0, tmp["PredYuv_2Nx2N"], "PredYuv-2Nx2N")
        Gallery.subplot(row, 1, tmp["PredYuv_Nx2N"], "PredYuv-Nx2N")
        Gallery.subplot(row, 2, tmp["PredYuv_2NxN"], "PredYuv-2NxN")
        Gallery.subplot(row, 3, tmp["PredYuv_NxN"], "PredYuv-NxN")
    
        for col in range(0,4): #cols
            pass

    Gallery.set_Labels_atCol(0, ("Depth0 (64x64)", "Depth1 (32x32)", "Depth2 (16x16)", "Depth3 (8x8)"))
    plt.tight_layout()
    return Gallery, Gallery.fig, Gallery.axarr 

#================================
def plot_RDO_Motion_Estimation(POC):
#================================
    depths = (1,3)
    depths = (0,1,2,3)
    dimx, dimy = A4_y, A4_x*1.2 

    printLARGE("plot_RDO_Motion_Estimation, POC%d" % POC["Idx"], 1)
    
    PartSize = "SIZE_2Nx2N"
    #VisLabels = ("Costs_MvBits", "Costs_Sideinfo", "Costs_HadSAD", "Costs_Final")
    
    VisLabels = ("Costs_MvBits", "Costs_Sideinfo", "Costs_HadSAD") #, "RefIdx") 
   
    #=====================
    def rdo_datamine_MVs_RefIdcs(Depth, PartSize = "SIZE_2Nx2N"):
    #=====================
        MVs = []
        RefIdcs = []
        
        for CTU in POC["CTUs"]:
            for CU in subset(CTU["CUs_RDO"], "Depth", Depth):
                for CHECK in subset(CU["CHECKs"], "PartSize", PartSize):
                    for PU in CHECK["PUs"]:
                        
                        # GET THE INDEX WITH BEST COSTS
                        BestIdx   = 0
                        BestCosts = PU["CHECKs"][0]["Costs_Final"]
                        for idx, PU_CHECK in enumerate(PU["CHECKs"]):
                            if PU_CHECK["Costs_Final"] < BestCosts:
                                BestCosts = PU_CHECK["Costs_Final"]
                                BestIdx = idx
                        
                        RefIdx = PU["Motion"]["RefIdx"]
                        RefIdcs.append(RefIdx)

                        Start = np.asarray(PU["Pos"]) + ( np.asarray(PU["Size"]) / 2.0) #centering in PU
                        End   = Start + np.asarray(PU["Motion"]["MV"]) / 4.0
                        MV = (Start,End)
                        MVs.append(MV)
                        
        return MVs, RefIdcs
    
     
    #=====================
    def datamine_depth(Depth):
    #=====================
        MVs = []
        Vises = []
        Dims = POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64
        #print *Dims
        VisMV =  np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64,3),np.uint8)      
        for Label in VisLabels:
            Vises.append(np.zeros(Dims,np.float))
        
        for CTU in POC["CTUs"]:
            for CU in subset(CTU["CUs_RDO"], "Depth", Depth):
                for CHECK in subset(CU["CHECKs"], "PartSize", PartSize):
                    for PU in CHECK["PUs"]:
                        
                        #SHOW((CTU, CHECK))
                        
                        # GET THE INDEX WITH BEST COSTS
                        BestIdx   = 0
                        BestCosts = PU["CHECKs"][0]["Costs_Final"]
                        for idx, PU_CHECK in enumerate(PU["CHECKs"]):
                            if PU_CHECK["Costs_Final"] < BestCosts:
                                BestCosts = PU_CHECK["Costs_Final"]
                                BestIdx = idx
                        
                        PU_BestCheck = PU["CHECKs"][BestIdx]
                        RefIdx = PU_BestCheck["RefIdx"]
                        PU_Area = PU["Size"][0] * PU["Size"][1]
                        idcs= get_Idcs_Y(PU)
                       
                        MV = get_Abs_MV(PU["Pos"], PU["Size"], PU_BestCheck["cMVTemp"]) #cMVPred
                        MVs.append(MV)
                        
                        for VisIdx, Label in enumerate(VisLabels):
                            if Label != "RefIdx":
                                Vises[VisIdx][idcs] = PU_BestCheck[Label] / PU_Area
                            else: Vises[VisIdx][idcs] = PU_BestCheck[Label]
                        
                        # on rare occasions, this fires, since RDO has a more complicated cost weighting
                        MV_2 = get_Abs_MV(PU["Pos"], PU["Size"], PU["Motion"]["MV"])
                        if (RefIdx != PU["Motion"]["RefIdx"]):
                            print "Warning: non matching cost decisions, see RDO"
                            #SHOW((CTU, CU, CHECK, PU))
                        if not isEqual(MV, MV_2): 
                            print "Warning: non matching cost decisions, see RDO"
                     
        #assert len(MVs) != 0                           
        for MV in MVs:
            cv2.line(VisMV, tuple(MV[0].astype(int)), tuple(MV[1].astype(int)), YELLOW, thickness=2, lineType=8, shift=0) 
        #plt.imshow(VisMV)        
        #plt.show()
            
        return Vises, VisMV
    
    
    #Vises, VisMV = datamine_depth(1)
    
    rows = len(VisLabels) + 1
    cols = 4
    
    
    
    Gallery = PlotGallery(rows, cols, dimx, dimy)
    RedGreenCMap = getCustomCMap_RedToGreen()
    for col in range(cols):
        depth = depths[col]
        Vises, VisMV = datamine_depth(depth)
        Gallery.subplot(0, col, VisMV, "Motion Vectors")
        
        
        TmpVisLabels = VisLabels #escapeUnderscores(VisLabels)
        print "VisLabels: ", VisLabels
        print "TmpVisLabels: ", TmpVisLabels
        for idx, Vis in enumerate(Vises):
            
            Label = TmpVisLabels[idx]
            if Label == "RefIdx":
                cmap = "jet"
            else: cmap = "jet" #RedGreenCMap
            
            Gallery.subplot(idx+1, col, Vis, Label, cmap)
        
    
    plt.tight_layout()
    return Gallery, Gallery.fig, Gallery.axarr  
    
                          
                     


    
#================================
def visualize_Processed_Depths(POC):
#================================    
    printLARGE("calcFeatures__CU_CHECK_PartSizeInter (POC%d, Depth%d)" % (POC["Idx"], -1))
    Canvas = np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64,3),np.uint8)
    NumCTUs = POC["Size_inCTUs"][0] * POC["Size_inCTUs"][1]

    for Depth in np.arange(0,4):
        for CTU in POC["CTUs"]:
            for CU in subset(CTU["CUs_RDO"], "Depth", Depth):
                for CHECK in subset(CU["CHECKs"], "PartSize", "SIZE_2Nx2N"):
    
                    draw_Object(POC, CU, Color=GREEN, Canvas=Canvas, BGR_Alpha=0.8)
                    
    #saveBGR(Canvas, TmpDir + "/visualize_Processed_Depths.png")        
    
    return Canvas  

#==========================================
def collect_RDO_Buffers_PredYuvs(POC):
#==========================================   
    printLARGE("collect_RDO_Buffers_PredYuvs()")
    PredYuvs = {}
    PredYuvs["Depths"] = []
    PredYuvs["Description"] = "Y PREDICTION\nfor different INTER PartSize-Checks, at different depths, with the best MV and ref frame found" 
    #collect_RDO_Buffers_PredYuvs
    for Depth in range(0,4):
        print "Processing depth %d" % Depth
        tmp = {}
            
        tmp["Depth"] = Depth
        tmp["PredYuv_2Nx2N"] = np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float)
        tmp["PredYuv_Nx2N"]  = np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float)
        tmp["PredYuv_2NxN"]  = np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float)
        tmp["PredYuv_NxN"]   = np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float)
        
        
        for CTU in POC["CTUs"]:
            for CU in subset(CTU["CUs_RDO"], "Depth", Depth):
              
                for CHECK in subset(CU["CHECKs"], "PartSize", "SIZE_2Nx2N"):
                    idcs = get_Idcs_Y(CU)   
                    tmp["PredYuv_2Nx2N"][idcs] = CHECK["PredYuv"]
                         
                for CHECK in subset(CU["CHECKs"], "PartSize", "SIZE_Nx2N"):
                    idcs = get_Idcs_Y(CU)   
                    tmp["PredYuv_Nx2N"][idcs] = CHECK["PredYuv"]
                  
                for CHECK in subset(CU["CHECKs"], "PartSize", "SIZE_2NxN"):
                    idcs = get_Idcs_Y(CU)            
                    tmp["PredYuv_2NxN"][idcs] = CHECK["PredYuv"]
                  
                for CHECK in subset(CU["CHECKs"], "PartSize", "SIZE_NxN"):
                    idcs = get_Idcs_Y(CU)   
                    tmp["PredYuv_NxN"][idcs] = CHECK["PredYuv"]
                        
        PredYuvs["Depths"].append(tmp)
                        
    return PredYuvs
            
#==========================================
def collect_RDO_Buffers_ResiYuvs(POC):
#==========================================   
    printLARGE("collect_RDO_Buffers_ResiYuvs()")
    ResiYuvs = {}
    ResiYuvs["Depths"] = []
    ResiYuvs["Description"] = "Y RESIDUAL\nfor different INTER PartSize-Checks, at different depths, with the best MV and ref frame found" 
    for Depth in range(0,4):
        print "Processing depth %d" % Depth
        tmp = {}
            
        tmp["Depth"] = Depth
        tmp["ResiYuv_2Nx2N"] = np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float)
        tmp["ResiYuv_Nx2N"]  = np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float)
        tmp["ResiYuv_2NxN"]  = np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float)
        tmp["ResiYuv_NxN"]   = np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float)
        
        for CTU in POC["CTUs"]:
            for CU in subset(CTU["CUs_RDO"], "Depth", Depth):
              
                for CHECK in subset(CU["CHECKs"], "PartSize", "SIZE_2Nx2N"):
                    idcs = get_Idcs_Y(CU)   
                    tmp["ResiYuv_2Nx2N"][idcs] = CHECK["ResiYuv"]
                         
                for CHECK in subset(CU["CHECKs"], "PartSize", "SIZE_Nx2N"):
                    idcs = get_Idcs_Y(CU)   
                    tmp["ResiYuv_Nx2N"][idcs] = CHECK["ResiYuv"]
                  
                for CHECK in subset(CU["CHECKs"], "PartSize", "SIZE_2NxN"):
                    idcs = get_Idcs_Y(CU)            
                    tmp["ResiYuv_2NxN"][idcs] = CHECK["ResiYuv"]
                  
                for CHECK in subset(CU["CHECKs"], "PartSize", "SIZE_NxN"):
                    idcs = get_Idcs_Y(CU)   
                    tmp["ResiYuv_NxN"][idcs] = CHECK["ResiYuv"]
                        
        ResiYuvs["Depths"].append(tmp)
                        
    return ResiYuvs

#==========================================================================================================================
#==================================================== PLOTTING ============================================================
#==========================================================================================================================           

#================================
def calcFeatures__CU_CHECK_PartSizeInter(POC, Depth, RefIdx, VarName):
#================================    
    printLARGE("calcFeatures__CU_CHECK_PartSizeInter %s (POC%d, Depth%d)" % (VarName, POC["Idx"], Depth))
    NumCTUs = POC["Size_inCTUs"][0] * POC["Size_inCTUs"][1]
    
    for CTU in POC["CTUs"]:
        for CU in subset(CTU["CUs_RDO"], "Depth", Depth):
            for CHECK in subset(CU["CHECKs"], "PartSize", "SIZE_2Nx2N"):

                
                # DEACTIVATED, TO LIFT DEPENDENCE ON GT
                #idcs = get_Idcs(CU)
                #Patch = StaticArea_Mask[idcs]
                #Patch = Patch[:,:,0]
                
                #StaticArea_Mask = GT["StaticArea_Mask"]
#                 Pos, Size = CU["Pos"], CU["Size"]
#                 x1, x2 = Pos[0], Pos[0] + Size[0]
#                 y1, y2 = Pos[1], Pos[1] + Size[1]  
#                 idcs = np.index_exp[y1:y2, x1:x2, :]  # nice fella!
#                 Patch = StaticArea_Mask[y1:y2, x1:x2, 0]
                
                #print CTU["Idx"]
                #saveRGB(StaticArea_Mask*255, TmpDir + "/StaticArea_Mask.png")
                #saveY(Patch*255, TmpDir + "/Test.png")
                #saveY(get_CU_RecY(POC, CU), TmpDir + "/TestY.png")
                
                #HACK
                Patch = np.ones(CU["Size"], np.uint8) #careful, size is first x, then y, not relevant for CU
                
                CU_Area = float((CU["Size"][0] * CU["Size"][1]))
                MaskMean = np.sum(np.sum(Patch)) / CU_Area

                if MaskMean == 1.0: # fully inside map area
                    Label = "Map"
                elif MaskMean == 0.0: #fully inside static area
                    Label = "StaticArea"
                else: 
                    Label = "Mixture" # some mixture
                    
                LA_CU_Content = Label
                FE_Distortion          = CHECK["TotalDistortion"] / CU_Area
                FE_DistortionDiff_2NxN = (CHECK["TotalDistortion"] - subset(CU["CHECKs"], "PartSize", "SIZE_2NxN").next()["TotalDistortion"]) / CU_Area
                FE_DistortionDiff_Nx2N = (CHECK["TotalDistortion"] - subset(CU["CHECKs"], "PartSize", "SIZE_Nx2N").next()["TotalDistortion"]) / CU_Area
                FE_DistortionDistance  = np.sqrt( FE_DistortionDiff_Nx2N**2 + FE_DistortionDiff_2NxN**2 )
                
                CHECK["Features"] = dict(
                    LA_CU_Content = LA_CU_Content,
                    FE_Distortion = FE_Distortion,
                    FE_DistortionDiff_2NxN = FE_DistortionDiff_2NxN,
                    FE_DistortionDiff_Nx2N = FE_DistortionDiff_2NxN,
                    FE_DistortionDistance = FE_DistortionDistance, 
                )
                    
    Vis = None
    doVis = True            
    if doVis:
        Vis3_Label, Vis3 = "PU Block Vis (use for sanity check)", np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64, 3),np.uint8)
        Vis1_Label, Vis1 = "CU GT Class (Map, StatA)", np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float)
        Vis2_Label, Vis2 = "TotalCost (=value from HM, per CU)", np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float)
        Vis4_Label, Vis4 = "FE_Distortion (=mean distortion per pixel)", np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float)
        Vis5_Label, Vis5 = "ResiYuv (=m_ppcResiYuvTemp after all PUs are processed)", np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float)
        Vis6_Label, Vis6 = "Processed_Depths (=dark shades signal less processing)", visualize_Processed_Depths(POC) 
        Vis7_Label, Vis7 = "Costs_HadSAD", np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float)
        
        Vis8Tup_Label = ("Costs_MvBits", "Costs_Sideinfo", "Costs_HadSAD", "Costs_Final")
        Vis8Tup_Affine = []
        Vis8Tup_Transl = []
        for Label in Vis8Tup_Label:
            Vis8Tup_Affine.append(np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float))
            Vis8Tup_Transl.append(np.zeros((POC["Size_inCTUs"][1]*64,POC["Size_inCTUs"][0]*64),np.float))
        
        #-------------------------------------
        # IDENTIFY "PREDICTABLE" PIXELS
        #-------------------------------------
        # DEACTIVATED, TO LIFT DEPENDENCE ON GT
        #PredictableArea_Mask = get_PredictableArea_Mask(GT["POCs"][POC["Idx"]]["H_gt"], GT["StaticArea_Mask"])
        #Extended_Mask = np.zeros((PredictableArea_Mask.shape[:2]),np.float32)
        #np.copyto(Extended_Mask, PredictableArea_Mask)
        
        PartSize = "SIZE_2Nx2N"
        
        MVs = []
        MVs_Rel = np.empty(shape=[0, 2])
        
        #calcFeatures__CU_CHECK_PartSizeInter: VIS-processing
        for CTU in POC["CTUs"]:
            for CU in subset(CTU["CUs_RDO"], "Depth", Depth):
                for CHECK in subset(CU["CHECKs"], "PartSize", PartSize):
                    #print CTU["Idx"]
                    
                    #-------------------------------------
                    # VISUALIZE PUS, SAVE MVs FOR LATER
                    #-------------------------------------
                    for PU in CHECK["PUs"]:
                        #Vis3[:,:,:] = BLACK
                        PU_Area = PU["Size"][0] * PU["Size"][1]
                        idcs= get_Idcs_Y(PU)
                        Vis3[idcs] = BLUE
                        cv2.rectangle(Vis3, (PU["Pos"][0],PU["Pos"][1]), (PU["Pos"][0]+PU["Size"][0], PU["Pos"][1]+PU["Size"][1]), WHITE, 2)
                        
                        #SAVE MVs FOR LATER (UNUSED)
                        Start = np.asarray(PU["Pos"]) + ( np.asarray(PU["Size"]) / 2.0) #centering in PU
                        End   = Start + np.asarray(PU["Motion"]["MV"]) / 4.0
                        MV = (Start,End)
                        MVs.append(MV)
                        
                        MVs.append(MV)
                        MVx, MVy = PU["Motion"]["MV"]
                        MVs_Rel = np.append(MVs_Rel, [[MVx, MVy]], axis=0)
                        #print "PU[Motion][MV]", PU["Motion"]["MV"]
                        #print "MV", MV
                        
                        #cv2.line(Vis3, tuple(MV[0].astype(int)), tuple(MV[1].astype(int)), GREEN, thickness=2, lineType=8, shift=0) 
                        
                        #-------------------------------------
                        # VISUALIZE SIDE INFO COSTS FOR AFFINE CHECKS (FOR ONE SPECIFIC REFIDX)
                        #------------------------------------- 
                        for CHECK_MotionType in subset(PU["CHECKs"], "Type", "AFFINE"):
                            if CHECK_MotionType["RefIdx"]==RefIdx:
                                for idx, Label in enumerate(Vis8Tup_Label):
                                    Vis8Tup_Affine[idx][idcs] = CHECK_MotionType[Label] / PU_Area
                                    
                        #-------------------------------------
                        # VISUALIZE SIDE INFO COSTS FOR AFFINE CHECKS (REFIDX=0)
                        #------------------------------------- 
                        for CHECK_MotionType in subset(PU["CHECKs"], "Type", "TRANSL"):
                            if CHECK_MotionType["RefIdx"]==RefIdx:
                                for idx, Label in enumerate(Vis8Tup_Label):
                                    Vis8Tup_Transl[idx][idcs] = CHECK_MotionType[Label] / PU_Area
                                
                                
                        #minimal_Plot_Example(Vis3)


                        
                    idcs = get_Idcs_Y(CU)
 
                    #-------------------------------------
                    # VISUALIZE RESI YUV
                    #-------------------------------------    
                    if 1:
                        
                    # uncomment this if you want to focus on "promising areas" (=where GWR / ABMC might kick in)
                    #if (Extended_Mask[idcs]).sum() == CU["Size"][0] * CU["Size"][0]:              
                        Vis5[idcs] = CHECK["ResiYuv"]
                    
                    #-------------------------------------
                    # VISUALIZE LABELS
                    #-------------------------------------
                    
                    if CHECK["Features"]["LA_CU_Content"] == "Map":
                        Vis1[idcs] = 0
                    elif CHECK["Features"]["LA_CU_Content"] == "StaticArea":
                        Vis1[idcs] = 1
                    
                    #-------------------------------------
                    # VISUALIZE FEATURES
                    #-------------------------------------
                    #Vis2[idcs] = CHECK["Features"]["FE_DistortionDistance"]
                    
                    Vis2[idcs] = CHECK["TotalCost"]
                    Vis4[idcs] = CHECK["Features"]["FE_Distortion"]
#                     ColorVal = np.clip(FE_DistortionDistance*50, 0, 255)
#                     draw_Object(POC, CU, [0, ColorVal, 0], Vis2)

        #-------------------------------------
        # MASK ResiYuv
        #-------------------------------------
#         PredictableArea_Mask = get_PredictableArea_Mask(GT["POCs"][POC["Idx"]]["H_gt"], GT["StaticArea_Mask"])
#         #ShapeY, ShapeX = PredictableArea_Mask.shape[:2]
#         Vis5 = Vis5[:ShapeY, :ShapeX]
#         Vis5 = Vis5 * PredictableArea_Mask #masking

        #-------------------------------------
        # VISUALIZE MVs 
        #-------------------------------------
        for MV in MVs:
            cv2.line(Vis3, tuple(MV[0].astype(int)), tuple(MV[1].astype(int)), GREEN, thickness=2, lineType=8, shift=0) 
         
         
        #-------------------------------------
        # PLOT PAGE A 
        #-------------------------------------                      
        minimal_Multiplot_Example((3,2), (Vis2, Vis5, Vis3, Vis4, Vis6), 
                                  block=False,
                                  LabelTuple=(Vis2_Label, Vis5_Label, Vis3_Label, Vis4_Label, Vis6_Label))   
        getCurrentFigure().suptitle("CUs_RDO, %s only, at Depth%s\n" \
                                    % (PartSize, Depth), fontsize=20)
        FN_Figure = HARP_TmpDir + "/PAGE_A_CUs_RDO_POC%04d_Depth%01d___%s.png" % (POC["Idx"], Depth, VarName)     
        #plt.savefig(FN_Figure) 
        savePlot_PNG(FN_Figure)  
        
        #-------------------------------------
        # PLOT PAGE B
        #-------------------------------------                      
        # minimal_Multiplot_Example((4,2), tuple(Vis8Tup_Affine + Vis8Tup_Transl), 
        minimal_Multiplot_Example((2,2), Vis8Tup_Transl, # tuple(Vis8Tup_Affine + Vis8Tup_Transl), 
                                  block=False,
                                  LabelTuple= tuple(Vis8Tup_Label + Vis8Tup_Label))   
        getCurrentFigure().suptitle("CUs_RDO, %s only, at Depth%s\n" \
                                    "Required side info (using RefFrame%d, normalized (=per pixel))\n  " \
                                    "TOP: Affine checks\n" \
                                    "BOTTOM: Transl checks\n" \
                                    % (PartSize, Depth, RefIdx), fontsize=20)
        plt.subplots_adjust(top=0.90) # make space for multiline suptitle
        FN_Figure = HARP_TmpDir + "/PAGE_B_CUs_RDO_POC%04d_Depth%01d___%s.png" % (POC["Idx"], Depth, VarName)     
        #plt.savefig(FN_Figure) 
        savePlot_PNG(FN_Figure)  
            
            
        #-------------------------------------
        # PLOT MV HISTOGRAMS
        #------------------------------------- 
        if(0):
            import pylab as P
            fig = P.figure()
            ax = fig.add_subplot(111)        
            n, bins, patches = ax.hist(MVs_Rel, 80, normed=1, histtype='bar',
                                      color=['crimson', 'burlywood'],
                                      label=['MVx', 'MVy'])
            
            VarMVx, VarMVy  = np.var(MVs_Rel,axis = 0)
            StdMVx, StdMVy  = np.std(MVs_Rel,axis = 0)
            
            ax.text(0.02, 0.9, "VarMVx=%0.2f | VarMVy=%0.2f \nStdMVx=%0.2f | StdMVy=%0.2f" % (VarMVx, VarMVy, StdMVx, StdMVy),
                    horizontalalignment='left',
                    verticalalignment='center',
                    transform = ax.transAxes)
            
            P.legend()
            P.show()                        
 
                    
                
    
    #return POC            
    #save_CPickle_FN(POC, TmpDir + "/Depth0_CUs.hdt") 