#!/usr/bin/env python
# coding: utf8
# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)

from Imports_Basic import *

import threading 
import Queue 
import multiprocessing




import multiprocessing
import time

#==========================================================================================================================
#=============================================== NEW: MULTIPLE PROCESSES! ===================================================
#==========================================================================================================================
# very simple!
# own stdout (can be redirected individually)
# queuing concept is very interesting!  

# |||||||||||||||||||||||||||||||||||||||||||||||||
class Consumer(multiprocessing.Process):
# |||||||||||||||||||||||||||||||||||||||||||||||||
    
    #========================
    def __init__(self, task_queue, result_queue):
    #========================
        multiprocessing.Process.__init__(self)
        self.task_queue = task_queue
        self.result_queue = result_queue

    #========================
    def run(self):
    #========================
        proc_name = self.name
        while True:
            next_task = self.task_queue.get()
            if next_task is None:
                # Poison pill means shutdown
                #print '%s: Exiting' % proc_name
                self.task_queue.task_done()
                break
            print 'Starting: %s: %s' % (proc_name, next_task)
            answer = next_task()
            self.task_queue.task_done()
            self.result_queue.put(answer)
        return



# |||||||||||||||||||||||||||||||||||||||||||||||||
class Task(object):
# |||||||||||||||||||||||||||||||||||||||||||||||||
    #========================
    def __init__(self, a, b):
    #========================
        self.a = a
        self.b = b
    
    #========================
    def __call__(self):
    #========================
        time.sleep(0.1) # pretend to take some time to do the work
        return '%s * %s = %s' % (self.a, self.b, self.a * self.b)
    
    #========================
    def __str__(self):
    #========================
        return '%s * %s' % (self.a, self.b)
    
#==========================================
def get_Queues():
#==========================================
    # Establish communication queues
    tasks = multiprocessing.JoinableQueue()
    results = multiprocessing.Queue()
    
    # Start consumers
    num_consumers = multiprocessing.cpu_count() * 2
    print 'Creating %d consumers' % num_consumers
    consumers = [ Consumer(tasks, results)
                  for i in xrange(num_consumers) ]
    for w in consumers:
        w.start()
        
    return tasks, results, num_consumers
    
#==========================================
def test_multiprocessing():
#==========================================
    tasks, results, num_consumers = get_Queues()
    
    # Enqueue jobs
    num_jobs = 10
    for i in xrange(num_jobs):
        tasks.put(Task(i, i))
    
    # Add a poison pill for each consumer
    for i in xrange(num_consumers):
        tasks.put(None)

    # Wait for all of the tasks to finish
    
    # Start printing results
    while num_jobs:
        result = results.get()
        print 'Result:', result
        num_jobs -= 1
        
    #tasks.join()

#==========================================================================================================================
#===============================================  SIMPLE MULTITHREADING ===================================================
#==========================================================================================================================

#==========================================
def swissKnife(myfunc, args):
#==========================================
    import threading
    print "Detaching simple thread"
    MyThread = threading.Thread(target=myfunc, args=args)
    MyThread.daemon = False
    MyThread.start()

# GENERAL APPROACH:
# Choose position inside a FOR loop where many calls take place: each will get a thread!
# The return value of the funtion musst be placed in a numpy array / list with index
# Remember to state "ThreadList = []" (see above)
# Note that try/except will not help in catching thread exceptions globally

#==========================================
def threading_EXAMPLE(PNG_FNs):
#==========================================

    class CAnyClass:
        def process(self, POC, idx):
            pass

    AnyClass = CAnyClass()

    ThreadList = []
    for idx, PNG in enumerate(PNG_FNs):
        print "Processing ", PNG
        POC = cv2.imread(PNG)
        
        SingleThread = True
        if SingleThread:
            AnyClass.process(POC, idx)
            
        else:
            MyThread = threading.Thread(target=AnyClass.process, args=(POC, idx))
            MyThread.daemon = False
            MyThread.start()
            ThreadList.append(MyThread)
            
            # SMPLE BLOCK MECHANISM
            Num = len(ThreadList)
            if Num >= 8:
                for Thread in ThreadList:
                    Thread.join()
                del ThreadList[:]
                print "\nSynchronizing %s threads..." % Num,   
        
        del POC
    print "Done"


#==========================================================================================================================
#=============================================  COMPLICATED MULTITHREADING ================================================
#==========================================================================================================================

# HOW A MATHEMATIKER WORKS ================
# The MATHEMATIKER waits for a JOB in the mailbox 
# The JOB contains what he/she should do, along with the right TOOLSET (a blackbox)
# The TOOLSET offers prepare() and apply() functions, which need to know about the MATHEMATIKER's PASSPORT ID
# The PASSPORT is used to create unique (output) directories while the JOB is done
# The RESULT is returned by Job.Toolset.apply() and written by MATHEMATIKER to a mutex-locked file 
#==========================================

#==========================================
def wakeupMathematiker(NumMathematiker="Auto"): 
#==========================================
    NumMathematiker = multiprocessing.cpu_count() if NumMathematiker == "Auto" else NumMathematiker
    MathematikerList = [Mathematiker() for i in range(NumMathematiker)]
    for ID, aMathematiker in enumerate(MathematikerList): 
        aMathematiker.setDaemon(True)
        aMathematiker.ID = ID
        aMathematiker.start() # heyho single-threaded Mathematiker, start thinking!
            
    print "Mathematiker: wokeup %d Mathematiker, now waiting for jobs" % NumMathematiker
    return MathematikerList #not needed anymore

#====================================================================================
class Mathematiker(threading.Thread): 
#====================================================================================
    
    ThreadLock = threading.Lock() 
    Briefkasten = Queue.Queue() 
     
#     #==========================================
#     def __init__(self):
#     #==========================================
        
    #==========================================
    def run(self): 
    #==========================================
      
        #-------------------------------------
        # WAKING UP
        #-------------------------------------    
        time.sleep(0.1)
        Mathematiker.ThreadLock.acquire() 
        print "Mathematiker %d woke up in his office, waits for work" % self.ID
        Mathematiker.ThreadLock.release() 
    
        while True: 
            #try:            
                #-------------------------------------
                # GET A JOB, DUDE
                #-------------------------------------    
                Toolbox = Mathematiker.Briefkasten.get()  
                print "Mathematiker took job from Mailbox, jobs left: %d" % Mathematiker.Briefkasten.qsize()
                Toolbox.ID = self.ID #Mathematiker ID
                
                #DISABLED
                #Toolbox.prepare(self.ID) # do some initial setup with the information so far
                
                #-------------------------------------
                # MATHEMATIKER! THINK! APPLY TOOLSET ! HEUREKA!
                #-------------------------------------    
                Toolbox.run()
     
                #-------------------------------------
                # FINISH UP
                #------------------------------------- 
                print str(Mathematiker.Briefkasten.qsize()) + " jobs still waiting in mailbox ..."
                Mathematiker.ThreadLock.acquire() #thread safe file IO
#                 ResultsFile = open(Toolset.ResultsFile, "a")
#                 ResultsFile.write(Result)
#                 ResultsFile.close()
                Mathematiker.ThreadLock.release() 
                Mathematiker.Briefkasten.task_done() 

# USE THIS CODE IF MATHEMATIKER CRASHES AND NEEDS REHAB
#             except Exception, Str:
#                 print "EXCEPTION------------------------"
#                 print "Unexpected ERROR:", sys.exc_info()[0]
#                 traceback.print_tb(sys.exc_info()[2])
#                 print Str  
#                 print "---------------------------------"
#                 Mathematiker.Briefkasten.task_done()  #prevent thread lock in case of exception
#                 raise
