#!/usr/bin/env python
# coding: utf8
# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)

from Imports_Basic import *

#==========================================
# TYPE CASTS
#==========================================
# Img = np.uint8(Img)
# Integer data types: int8 int16 int32 int64 uint8 uint16 uint32 uint64 bool
# Float data types: float16  float32  float64   
# DECLARE: np.zeros((512,512,3), np.uint8)  
# TEST: M.dtype == np.uint8 ?

# Important note: 1D arrays are always seen as column vectors (though are printed in a row)
# Hence: you need hstack to stack them, not vstack!

#==========================================
def get_Numpy_Version():
#==========================================
    return np.version.short_version

#==========================================================================================================================
#=============================================== INSTANTIATION ============================================================
#==========================================================================================================================

#==========================================
def example_Instantiate():
#==========================================
    # ROW VECTOR / ARRAY
    RowVec = np.zeros((100,), np.float32)
    
    # MATRICES
    Matrix_1Ch = np.zeros((2,3), np.float32)
    Matrix_3Ch = np.zeros((100,100,3), np.uint8)
    
    Eye  = np.eye(2, dtype=int)
    Diag = np.diagflat([[1,2,2,3]])
    


#==========================================
def example_Create_Empty_Matrix():
#==========================================
    #trick: use list and convert
    
    #creates shape (0,) #empty col vector!!
    empty = np.asarray([], dtype=np.float) #
    np.hstack((empty, np.zeros((10), np.uint8))) #hstack:because 1D is seen as a column vector

#==========================================
def get_Sample_GT_H():
#==========================================   
    #use print M, then copy from console, don't forget the commas
    GT_H = [[  9.81032776e-01,  -1.93441791e-01,   7.15840630e+01], 
            [  1.93367426e-01,   9.80987643e-01,  -9.29572510e+01], 
            [ -4.92444221e-09,  -1.81900066e-07,   1.00000000e+00]]
    GT_H = np.array(GT_H)  #wow! simple as that!!
    print "get_Sample_GT_H: \n", GT_H
    return GT_H

#==========================================
def get_Eye_Homography():
#==========================================  
    return np.eye(3, dtype=np.float32)

#==========================================
def emptyCopy(M):
#==========================================
    copy = M.copy() #preserve dims and type
    copy.fill(0) #inplace
    return copy

#==========================================================================================================================
#================================================== INDEXING ==============================================================
#==========================================================================================================================

#==========================================
def get_Last(Array):
#==========================================
    return Array[-1]
    
#==========================================
def reverse_Array(Array): #flip array
#==========================================
    #untested    
    return Array[::-1] #interesting! Like L[1:10:-1], the -1 turns the count direction!

#==========================================
def get_Index(M, val):
#==========================================
    if type(M) == np.ndarray:
        return np.where(M == val)[0]
    elif type(M) == list:
        return M.index(val)
    
    

#==========================================================================================================================
#=============================================== MATH =====================================================================
#==========================================================================================================================

#==========================================
def get_Vector_Length(Vec):
#==========================================
    return np.linalg.norm(Vec, ord=2)
    

#==========================================
def example_Interpolate():
#==========================================
    from scipy import interpolate
    x = np.arange(0, 10)
    y = np.exp(-x/3.0)
    f = interpolate.interp1d(x, y)
    
    xnew = np.arange(0,9, 0.1)
    ynew = f(xnew)   # use interpolation function returned by `interp1d`
    plt.plot(x, y, 'o', xnew, ynew, '-')
    plt.show()

#==========================================
def isEqual(Matrix1, Matrix2): #compare two matrices elementwise
#==========================================
    return np.array_equal(Matrix1, Matrix2)

#==========================================
def isAlmostEqual(float_a, float_b):  # Ohnoez: rounding and precision issues when comparing floats
#==========================================
    return np.isclose(float_a, float_b, rtol=1e-05, atol=1e-08, equal_nan=False)

#test:
#     array = np.random.random(10)
#     print array, find_nearest(array, 0.4)
#==========================================
def find_nearest(List,value):
#==========================================
    idx = (np.abs(List-value)).argmin()
    return List[idx], idx

#==========================================
def example_StdDev():
#==========================================
    mean, sigma = 0, 0.1 # mean and standard deviation
    s = np.random.normal(mean, sigma, 1000000)
    StdDev = np.std(s)
    print "standard_deviation: ", StdDev   

#==========================================
def getMinMax(Img):
#==========================================
    return np.amax(Img), np.amax(Img) #over all dimensions

#==========================================
def getMax(Img):
#==========================================
    return np.amax(Img) #over all dimensions

#==========================================
def getMin(Img):
#==========================================
    return np.amin(Img) #over all dimensions

#==========================================
def getAbs_elementwise(M):
#==========================================
    return np.absolute(M)

#==========================================
def get_Matrix_SAD(M1, M2):
#==========================================
    M1c = M1.astype(np.float)
    M2c = M2.astype(np.float)
    SAD = getAbs_elementwise(M1c-M2c).sum()
    return SAD

#==========================================
def get_Matrix_Sum(Matrix_2D):
#==========================================
    return np.float(Matrix_2D.sum()) #simple as that!

#==========================================
def getAverageOLD(Img):
#==========================================
    return np.average(Img) #over all dimensions

#==========================================
def getAverage2(Matrix_2D):
#==========================================
    NumSamples = (Matrix_2D.shape[0] * Matrix_2D.shape[1])
    return np.float(Matrix_2D.sum() / NumSamples) #simple as that!

#==========================================
def getMean1(Img): #average
#==========================================
    return np.average(Img) #over all z dimensions (tested)

#==========================================
def getMean2(M): #average
#==========================================
    return np.mean(M) #over complete 2D matrix (tested)

#==========================================
def inv(Matrix):
#==========================================
    return np.linalg.inv(Matrix)

#==========================================
def get_absolute_Maximum(M):
#==========================================
    return np.max(np.max(np.absolute(M)))

#==========================================================================================================================
#=================================================== CASTING / SHAPE / TYPE ===============================================
#==========================================================================================================================

#==========================================
def cast_2_uint8(M):
#==========================================
    return M.astype(np.uint8)

#==========================================
def cast_2_float(M):
#==========================================
    return M.astype(np.float)

#==========================================
def flattenMatrix(Matrix):
#==========================================
    return Matrix.flatten() #all elemtents in 1D array

#==========================================
def clone_Type_and_Shape(OtherMatrix):
#==========================================
    return np.zeros_like(OtherMatrix) #also: np.ones_like()

#==========================================
def dstack_Channels(Tuple):
#==========================================
    return np.dstack(Tuple)

#==========================================
def to3Ch(Ch0):
#==========================================
    return np.dstack((Ch0, Ch0, Ch0))

#==========================================
def assert_Matrices_Equal(M1, M2):
#==========================================
    return np.array_equal(M1, M2)

#==========================================
def assert_Type(M, dtype = np.uint8):
#==========================================
    assert(M.dtype == dtype)
    
#==========================================
def assert_NumChannels(M, NumCh = 3):
#==========================================
    assert(len(M.shape) == NumCh)

#==========================================
def scale_To_UInt8(M):
#==========================================
    # NOTE: untested, no offset for handling negative values
    minVal, maxVal, minLoc, maxLoc = cv2.minMaxLoc(M)
    Dst = cv2.convertScaleAbs(M, alpha = 255/maxVal)
    return Dst
    #saveY(np.uint8(M), TmpDir + "/test.png")
    #minimal_Plot_Example(M)

#==========================================================================================================================
#======================================================= VARIOUS ==========================================================
#==========================================================================================================================

#==========================================
def zeropad_array(Arr, NumLeft, NumRight):
#==========================================
    return np.lib.pad(Arr, (NumLeft,NumRight), 'constant', constant_values=(0, 0))


#==========================================
def getNumBlocks(DimX, DimY, Blocksize):
#==========================================  
    NumMBsX = int(m.ceil(DimX / float(Blocksize)))
    NumMBsY = int(m.ceil(DimY / float(Blocksize)))
    return (NumMBsX, NumMBsY)

#==========================================
def get_XYZ_Random_Values():
#==========================================
    N=1000
    x = np.random.randn(N)
    y = np.random.randn(N)
    z = np.random.randint(0,2)
    return x,y,z

#==========================================
def logical_AND(a,b):
#==========================================
    # make sure you got binary arrays!
    return np.logical_and(a, b)

#==========================================
def invert_Boolean(Array):
#==========================================
    return ~Array
    # OLD:
    # return np.logical_not(Array)
