#!/usr/bin/env python
# coding: utf8
# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)

from Imports_Basic import *
from General import *

#first import: dry run (indexer needs this)
try:
    import cv2     
    print "\nTesting OpenCV system installation, found version " + cv2.__version__                
except ImportError:
    print "\nOpenCV: no system installation found"
    pass


# ALTERNATIVE: import via imp
#import imp
#mypath = res("~/lib/OPENCV_2.4.9/lib/python2.6/site-packages/")
#fp, pathname, description = imp.find_module('cv2', [mypath])
#cv2 = imp.load_module('cv2', fp, pathname, description)

# check out later:
# Test = cv2.decomposeHomographyMat(GT_H, get_Eye_Homography())
# Malis, E. and Vargas, M. Deeper understanding of the homography decomposition for vision-based control, Research Report 6303, INRIA (2007)
# http://www.lfd.uci.edu/~gohlke/code/transformations.py.html

#==========================================
# INTERPOLATION
#==========================================
# cv2.INTER_NEAREST
# cv2.INTER_LINEAR
# cv2.INTER_CUBIC
# cv2.INTER_LANCZOS4

#==========================================
# COLOR SPACE CONVERSIONS
#==========================================
if 0:
    YUV = cv2.cvtColor(Img,cv2.COLOR_BGR2YUV)
    COLOR_BGR2GRAY
    COLOR_BGR2YUV
    COLOR_BGR2RGB
    COLOR_GRAY2BGR
    COLOR_GRAY2RGB
    COLOR_RGB2GRAY
    COLOR_RGB2YUV
    COLOR_RGB2YUV_I420
    COLOR_YUV2BGR
    COLOR_YUV2BGRA_I420
    COLOR_YUV2RGB


#==========================================
def get_Color_Conversions():
#==========================================
    flags = [i for i in dir(cv2) if i.startswith('COLOR_')]
    return flags

#==========================================
def print_Color_Conversions():
#==========================================
    List = get_Color_Conversions()
    for entry in List:
        if "YUV" in entry or "RGB" in entry or "BGR" in entry or "GRAY" in entry:
            if not "BAYER" in entry and not "RGBA" in entry and not "HLS" in entry and not "LAB" in entry:
                print entry   
    

#==========================================
def OpenCV_Swiss_Knife():
#==========================================
    img = np.zeros((512,512,3), np.uint8)
    img = (128,128,128)
    cv2.line(img,(0,0),(511,511),(255,0,0),5)
    cv2.imwrite("testimage.jpg", img)
    cv2.imshow('image',img)
    cv2.waitKey(0)
    cv2.destroyAllWindows()

#========================================== 
def load_INI_Test():
#========================================== 
    if not __builtins__.has_key("INI"):
        FN = HARP_ProjectDir + "/HARP_Settings.ini"
        assertFileExists(FN)
        print "Loading INI: ", FN
        from External import configobj
        __builtins__.update(INI = configobj.ConfigObj(FN))

#==========================================================================================================================
#============================================= IMPORT OF SPECIFIC VERSION  ================================================
#==========================================================================================================================

#==========================================
def import_OpenCV2():
#==========================================
    import imp
    global cv2

    
    def res(Dir): 
        return os.path.expandvars(os.path.expanduser(Dir))
    
    Locations = (#res("~/lib/OPENCV_3.0.0/lib/python2.7/dist-packages/"), #ubuntu OpenCV 3 experimental
                 #res("~/lib/OPENCV_2.4.9/lib/python2.7/dist-packages/"), #ubuntu custom
                 #res("~/lib/OPENCV_2.4.9/lib/python2.7/site-packages/"),  #suse custom
                 #res("~/lib/OPENCV_2.4.9/lib/python2.6/site-packages/"),  #CentOS custom
                 res("/usr/lib64/python2.7/site-packages/"),  #suse system
                 res("/usr/lib/python2.7/site-packages/"),  #suse 32bit system
                 res("/usr/lib/python2.7/dist-packages/") )   #ubuntu system

    for Location in Locations:
        if os.path.isdir(Location): #we found a candidate
            fp, pathname, description = imp.find_module('cv2', [Location])
            cv2 = imp.load_module('cv2', fp, pathname, description)
            print "Importing OpenCV, version=%s, import from: %s" % (cv2.__version__, Location) 
            return 
        
    #last try: get system installation (will find Pythonxy under Windows)
    res = imp.find_module('cv2')
    if res != None:
        fp, pathname, description = res
        cv2 = imp.load_module('cv2', fp, pathname, description)
        print "Importing OpenCV, version=%s, import from: %s" % (cv2.__version__, Location) 
    else:
      raise Exception("OpenCV not found. Sorry.")

import_OpenCV2() # we default to Version 2.4 (srlsy?)
import cv2   

#==========================================
def import_OpenCV3():
#==========================================
    import imp
    global cv2
    Locations = (res("~/lib/OPENCV_3.0.0/lib/python2.7/dist-packages/"),
                res("~/lib/OPENCV_3.0.0/lib/python2.7/site-packages/") )
    Locations = (res("~/lib/OPENCV_3.0.0/lib/python2.7/dist-packages/"),
                res("~/lib/OPENCV_3.0.0/lib/python2.7/site-packages/") )
    for Location in Locations:
        if os.path.isdir(Location): #we found a candidate
            fp, pathname, description = imp.find_module('cv2', [Location])
            cv2 = imp.load_module('cv2', fp, pathname, description)
            print "Importing OpenCV, version=%s, import from: %s" % (cv2.__version__, Location) 
            return

    raise Exception("OpenCV not found. Sorry.")
   

#==========================================================================================================================
#============================================= COLOR SPACES, COLORS =======================================================
#==========================================================================================================================

#==========================================
def Y_2_BGR(Img):
#==========================================
    return cv2.cvtColor(Img, cv2.COLOR_GRAY2BGR)

#==========================================
def switchRGB(Img):
#==========================================
    return cv2.cvtColor(Img, cv2.COLOR_BGR2RGB)

#==========================================
def invertImage(Img):
#==========================================
    return (255 - Img)


# #==========================================
# def RGBtoBGR(Img): #color space conversion
# #==========================================
#     return cv2.cvtColor(Img, cv2.COLOR_RGB2BGR)
# 
# #==========================================
# def BGRtoRGB(Img): #color space conversion
# #==========================================
#     return cv2.cvtColor(Img, cv2.COLOR_BGR2RGB)


#==========================================================================================================================
#============================================= OPERATING ON IMAGES ========================================================
#==========================================================================================================================

#==========================================
def crop_Img_Borders_FN(FN):
#==========================================
    Img = readImg(FN)
    Img = crop_Img_Borders(Img)
    saveImg(Img, FN)
    
#==========================================
def crop_Img_Borders(Img):
#==========================================
    print "Cropping image... ",
    DimY, DimX = Img.shape[:2]
    
    #top to bottom
    for rowidx in range(0, DimY):
        if np.average(Img[rowidx, :, :]) != 255.0:
            break;
    t_y = rowidx
    
    for rowidx in range(DimY-1,-1,-1):
        if np.average(Img[rowidx, :, :]) != 255.0:
            break;      
    b_y = rowidx
    
    #left to right
    for colidx in range(0, DimX):
        if np.average(Img[:, colidx, :]) != 255.0:
            break;
    l_x = colidx
    
    for colidx in range(DimX-1,-1,-1):
        if np.average(Img[:, colidx, :]) != 255.0:
            break;    
    r_x = colidx        

    print "done"
    return Img[t_y:b_y, l_x:r_x,:]

# ============================
def stack_2x3(Imgs, Descs, withStrings = True):
# ============================
    
    Imgs = copy.deepcopy(Imgs)
    
    if withStrings:
        for Img, Desc in zip(Imgs, Descs):
            drawString(Img, Desc, Box=True)

    vs = getSpacer("Vertical", Imgs[0])
    Top    = np.hstack((Imgs[0], vs, Imgs[1], vs, Imgs[2]))
    Bottom = np.hstack((Imgs[3], vs, Imgs[4], vs, Imgs[5]))
    vs_ = getSpacer("Horizontal", Top)
    Final = np.vstack((Top, vs_, Bottom))
    return Final

# ============================
def stack_Vertically(ImgList):
# ============================
    Cols = ImgList[0].shape[1]
    Final = np.zeros((1,Cols,3), np.uint8)
    vs_ = getSpacer("Horizontal", Final)
    for Img in ImgList:
        Final = np.vstack((Final, vs_, Img ))
            
    return crop_Img_Borders(Final)

# ============================
def stack_Vertically_FNs(ImgFNList):
# ============================
    ImgList = [readImg(ImgFN) for ImgFN in ImgFNList]
    Final = stack_Vertically(ImgList)
    return Final
    

#==========================================
def resizeToWidth(Img, Width, Interp=cv2.INTER_LINEAR):
#==========================================
    (rows, cols) = Img.shape[:2]
    RatioX = float(cols)/Width;
    Height = int(rows/RatioX);
    return cv2.resize(Img, (Width, Height), interpolation=Interp)

#==========================================
def getSpacer(Str, Img, Dim = 20):
#==========================================
    h, w = Img.shape[:2]
    if Str == "Vertical": #vertical line
        VertSpacer = np.ones((h, Dim, 3), np.uint8)*255
        return VertSpacer
    elif Str == "Horizontal": #horizontal line
        HoriSpacer = np.ones((Dim,w, 3), np.uint8)*255
        return HoriSpacer

#==========================================
def composeImages(Img1, Img2, Mask, Transparency): 
#==========================================
    #Img1: Map, Img2: StaticAreas
    Shape = Mask.shape
    composed = np.empty(Shape, dtype = 'uint8')
    idcs = np.index_exp[:, :, :3]  # nice fella!
    composed[idcs] = (Img1[idcs]*Mask[idcs] + (Img2[idcs]*(1- Mask[idcs])*(1.0-Transparency) + Img1[idcs]*(1- Mask[idcs])*Transparency)   )
    return composed

#==========================================
def getLumDiff(Ref, Cur):
#==========================================
    if len(Ref.shape) == 3: #we got several channels
        Y_Ref = Ref[:,:,0]
        Y_Cur = Cur[:,:,0]
    else:
        Y_Ref = Ref
        Y_Cur = Cur
    DiffImg = cv2.addWeighted(np.float32(Y_Cur), 0.5, np.float32(Y_Ref), -0.5, 127)
    DiffImg = DiffImg.astype(np.uint8)
    
    return DiffImg
   
#==========================================================================================================================
#==================================================== SAVE / READ =========================================================
#==========================================================================================================================

    
#==========================================
def readImg(FN, Y_only = False):  #load OpenCV Image as BGR
#==========================================
    assertFileExists(FN)
    Img = cv2.imread(FN,0) if  Y_only else cv2.imread(FN) # attention: channel order is BGR!
    return Img
#==========================================
def saveImg(Img, FN):  #assuming BGR image
#==========================================
    saveBGR(Img, FN)    
    
#==========================================
def saveY(Img, FN):  #save 1-channel PNG
#==========================================    
    cv2.imwrite(FN, Img[:,:]) if len(Img.shape) == 2 else cv2.imwrite(FN, Img[:,:,0])

#==========================================
def readY(FN):  #save 1-channel PNG
#==========================================    
    assertFileExists(FN)
    Img = cv2.imread(FN,0)
    return Img
    
#==========================================
def saveYUV(Img, FN):  #save YUV as RGB
#==========================================    
    cv2.imwrite(FN, cv2.cvtColor(Img, cv2.COLOR_YUV2RGB)) # switch red and blue channels 
    
#==========================================
def saveBGR(Img, FN):  #save OpenCV Image
#==========================================
    #Img = invertImage(Img) if Invert else Img
    cv2.imwrite(FN, Img)
    
#==========================================
def saveRGB(Img, FN):  #save numpy matrix as file (WILL CROSSSWITCH OPENCV CHANNELS!)
#==========================================
    Img = cv2.cvtColor(Img, cv2.COLOR_RGB2BGR) # switch red and blue channels 
    #Img = invertImage(Img) if Invert else Img
    cv2.imwrite(FN, Img)    


#==========================================================================================================================
#=================================================== WRITE TEXT ===========================================================
#==========================================================================================================================

#CHANGED: now first s (string), then coords
#==========================================
def drawString(dst, str, (x, y) = (20, 40), Color=(255, 255, 255), FontScale=2.0, Box = False):
#==========================================
    FontFace, Thickness = cv2.FONT_HERSHEY_PLAIN, 2
    if Box:
        xExtray, yExtra = 5, 10
        TextSize = cv2.getTextSize(str, FontFace, FontScale, Thickness)[0]
        cv2.rectangle(dst, (x-xExtray, y+yExtra), (x+TextSize[0]+xExtray, y-TextSize[1]-yExtra), color=(0, 0, 0), thickness=cv2.FILLED)
        
    cv2.putText(dst, str, (x+1, y+1), FontFace, FontScale, (0, 0, 0), thickness = Thickness, lineType=cv2.CV_AA)
    cv2.putText(dst, str, (x, y), FontFace, FontScale, Color, thickness = Thickness, lineType=cv2.CV_AA)
    

#OLD:
#==========================================
def writeText(Img, (x, y), Text, Size=1.0, Color=(255, 255, 255)):
#==========================================
    cv2.putText(Img, Text, (x+1, y+1), cv2.FONT_HERSHEY_PLAIN, Size, Color, thickness = 2, lineType=cv2.CV_AA)
    cv2.putText(Img, Text, (x, y), cv2.FONT_HERSHEY_PLAIN, 1.0, (255, 255, 255), lineType=cv2.CV_AA)

#==========================================
def write_Str_FN(Str, FN):
#==========================================
    Img = readImg(FN)
    drawString(Img, Str, Box=True)
    saveImg(Img, FN)


#==========================================================================================================================
#================================================== VARIOUS ===============================================================
#==========================================================================================================================
   

#==========================================
def printOpenCVFunctions():
#==========================================
    from inspect import getmembers, isfunction
    
    functions_list = [o for o in getmembers(cv2) if isfunction(o[1])] #returns empty list (why?)
    members = getmembers(cv2)
    print members
        
    return members

#==========================================
def printOpenCVOptions():
#==========================================
    flags = [i for i in dir(cv2) if i.startswith('COLOR_')]
    print flags

#REFACTOR!
#==========================================
if 0: #thresholding experiment
#==========================================
        
    import matplotlib.pyplot as plt
    
    from skimage import data
    from skimage.filter import threshold_otsu, threshold_adaptive
    #from skimage.restoration import denoise_tv_chambolle, denoise_bilateral
    
    #print_Color_Conversions()
            
    #sys.exit()
    
    #image = data.page()
    FN = "/home/dom/Desktop/Jetson_TK1/DA/04_ENCODER/Zeichnungen/IMG_20160508_092939.jpg"
    FN = "/home/dom/Desktop/Jetson_TK1/DA/03_MOTION/Zeichnungen/test.jpg"
    OutFN = getDir(FN) + "/AdaptiveThresholding.jpg"
    
    image = readImg(FN)
    YUV = cv2.cvtColor(image,cv2.COLOR_BGR2YUV)
    Y = YUV[:,:,0]
    
    #Y = denoise_tv_chambolle(Y, weight=0.2, multichannel=True)
    #        Y = cv2.fastNlMeansDenoisingColored(Y,None,10,10,7,21)
    
      #Y = denoise_tv_chambolle(Y, weight=0.2, multichannel=True)
    test = np.dstack((Y, Y, Y))
    Y = cv2.fastNlMeansDenoising(Y,None,10,7,21)
    #Y = test[:,:,0]
    cv2.imwrite(OutFN, Y)
    
    global_thresh = threshold_otsu(Y)
    binary_global = Y > global_thresh
    
    block_size = 40
    binary_adaptive = threshold_adaptive(Y, block_size, offset=10)
    
    fig, axes = plt.subplots(nrows=3, figsize=(7, 8))
    ax0, ax1, ax2 = axes
    plt.gray()
    
    ax0.imshow(Y)
    ax0.set_title('Image')
    
    ax1.imshow(binary_global)
    ax1.set_title('Global thresholding')
    
    ax2.imshow(binary_adaptive)
    ax2.set_title('Adaptive thresholding')
    
    for ax in axes:
        ax.axis('off')
        
    from skimage import img_as_ubyte
    
    img = img_as_ubyte(binary_adaptive)
    cv2.imwrite(OutFN, img)
    
    plt.show()



#==========================================================================================================================
#================================================== IMAGE MATRIX ==========================================================
#==========================================================================================================================

#==========================================
class CImageMatrix():
#==========================================

    #==========================================
    def __init__(self, MatrixRows, MatrixCols, DimX, DimY):
    #==========================================
        self.DimX = DimX
        self.DimY = DimY
        self.MatrixRows = MatrixRows
        self.MatrixCols = MatrixCols
        
        self.Final = np.zeros((MatrixRows*DimY,MatrixCols*DimX,3), np.uint8)
    
    #==========================================
    def fillPosition(self, Image, Row, Col, text = None, CustomDimY = -1, CustomDimX = -1):
    #==========================================
        if CustomDimY == -1 or CustomDimX == -1:
            self.CustomDimY = self.DimY;
            self.CustomDimX = self.DimX;

        ImgClone = Image.copy()
        if len(ImgClone.shape) != 3: #if no channel information given, i.e. grayscale
            ImgClone = cv2.cvtColor(ImgClone, cv2.COLOR_GRAY2RGB)
        drawText(ImgClone, (10,20), text)
        #drawText(self.Final, (50,50), text)

        d = 2
        self.Final[Row*self.DimY:Row*self.DimY+self.DimY, Col*self.DimX:Col*self.DimX+self.DimX] = ImgClone
        
    #==========================================
    def getFinal(self):
    #==========================================
#        return self.Final
        Resized = cv2.resize(self.Final, (0, 0), 
                          fx=1920.0/(self.DimX*self.MatrixCols*1.2), 
                          fy=1080.0/(self.DimY*self.MatrixRows*1.2), interpolation=cv2.INTER_LINEAR)
            
        return Resized
    
#==========================================
def test_CImageMatrix():
#==========================================

    #ImageMatrix example
    ImgMatrix = CImageMatrix(2, 2, 1280, 720)
    test = np.zeros((720,1280,3), np.uint8)
    ImgMatrix.fillPosition(test, 0, 0, "Hello World")
    cv2.imshow("Test", ImgMatrix.getFinal())
    cv2.waitKey()


    
#==========================================
if __name__ == '__main__':
#==========================================
    #sys.exit(0)
    members = printOpenCVFunctions()
    for member in members:
        logText(str(member), "./OpenCV_members.txt")
    #print "\n".join(members)
    #writeStringToFN(str(members), "./OpenCV_members.txt")
    


    
