#!/usr/bin/env python
# coding: utf8
# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)

from Imports_Basic import *
from Plot import *
#from Latex import *

import thread
import pandas as pd
from threading import Thread, Lock


# ACHTUNG: rot=0 hilft bestens bei Barplots!!! Man muss nicht mit TickLabels spielen

# ==============================
# BARPLOT SWISS KNIFE
# ==============================
# Bar plots are best setup with PANDAS!! -> simply create a "list of lists"
#==========================================
def example_Bar_Plot_withPandas(): #BarPlot
#==========================================
    
    #from Pandas import *
    #import pandas as pd
    
    # ==========================================
    # STEP1: declare GroupNames and BarNames
    # ==========================================
    # GroupNames   == PANDA Index == PANDA row == name of the groups
    # BarNames     == PANDA Columns == what is shown in legend
    GroupNames = ("Group1", "Group2", "Group3", "Group4") #will show as x-axis ticks
    BarNames   = ['Legend1','Legend2','Legend3','Legend4','Legend5'] #will show in legend

    # ==========================================
    # STEP2: create empty DF
    # ==========================================        
    #ListOfLists  == each individual list represents a "bar group" 
    #ListOfLists=np.array([[3,6,8,9,6],[2,3,4,5,6],[4,5,6,7,8],[3,6,5,8,6]]) # vecs get stacked vertically
    df=pd.DataFrame(columns=BarNames, index=GroupNames)
    df.at["Group1","Legend4"] = 10
    #df.at["Group2","Legend4"] = 20
    df.loc["Group3":,:] = 20
    df.loc[:,:"Legend2"] = 30

    # ==========================================
    # STEP3: plot
    # ==========================================
    fig, axarr = plt.subplots(1, 1, figsize=(A4_x,A4_y/2))            
    df=pd.DataFrame(np.array(BarGroups), columns=BarNames, index=BarGroupNames)
    ax = df.plot(kind='bar', ax=axarr)
    plt.setp(plt.xticks()[1], rotation=0)
    #ax.axis["bottom"].major_ticklabels.set_rotation(30) 
    plt.tight_layout(pad=0.8, w_pad=2.0, h_pad=2.0)
    
    # ==========================================
    # STEP4: save
    # ==========================================
    FN = TmpDir + "/example_Bar_Plot_withPandas.jpg"
    plt.savefig(FN)
    save_DF_to_Excel(df, removeSuffix(FN) + ".xlsx", TabName="MyTab", float_format="%0.4f")
    copy_to_PlotExamples(FN)   
    #plt.show()
    
    
#==========================================================================================================================
#==================================================== FILTERING ===========================================================
#==========================================================================================================================

#==========================================    
def filter_Only_Cols(df, ColNames):
#==========================================
    return df.loc[:, ColNames]

#==========================================    
def filter_Only_Rows(df, RowNames):
#==========================================
    return df.loc[RowNames, :]

#==========================================================================================================================
#==================================================== INDEXING ============================================================
#==========================================================================================================================

# ==============================
# >> IX << (= BY COLUMN)
# ==============================
# df.ix[:,('Nummer', "Nummer")]
# UNCLEAR WHAT IX ACTUALLY DOES!! CLARIFY!!

#==========================================    
def iterate_Over_Rows(df):
#==========================================
# STRANGE. Did not find a way to set values in these series. Do they also set orig df?
#     for i in df.index:
#         row = df.ix[i] #returns a series
#         show_Object(row)
#         #row.ix[0, "PSNR"] = 0
#         #row.at[0, "PSNR"] = 0
#         print "Row%d, Dtype %s:\n" % (i, row.dtype), row
        
    for idx, row in df.iterrows():
        print "Index: ", idx
        print row['PSNR']   # PSNR: column name   
        print df.at[idx, "PSNR"]
        row['PSNR'] = 0 # will NOT set orig df!
        df.at[idx, "PSNR"] = 0 # WILL set orig df!
        
    print df

#==========================================    
def example_Indexing():
#==========================================

#     loc:  select rows and colums
#     iloc: select rows by their index numbers (integer only)
#     ix: You can get data from dataframe without to be this in the index
#     at:  very fast loc
#     iat: very fast iloc

    df = pd.DataFrame({'A':['a', 'b', 'c'], 'B':[50, 60, 70]}, index=[100, 200, 300])
    print df
#           A   B
#     100  a  50
#     200  b  60
#     300  c  70   
    
    print df.loc[100]
#     A     a
#     B    50
    print df.iloc[0]
#     A     a
#     B    50s
    print df.ix[100, 'A']
#     a

    # ===========================
    # SLICING (always: row-col)
    # ===========================
    myslice = df.loc[:,'B'].values #First rows then columns
    print myslice.values                             #note: entries remain abstract "objects"!
    print np.asarray(myslice.values, dtype=np.float) #note: perfect float array / matrix

#==========================================    
def get_single_Value(df, row=400, col='A'):
#==========================================
    #return df['A'][400]    #first x, then y -> STRONGLY DISCOURAGED!!
    return df.ix[row, col] #first y, then x -> NOTE THE IX-FUNCTION!

#export complete column to Numpy array
#==========================================    
def get_column_as_Numpy(df, ColName):
#==========================================
    return np.array(df.loc[:,ColName])

#==========================================  
def get_Value(df, row, col):
#==========================================  
    return df.at[row,col]

#==========================================  
def set_Value(df, Value, row, col):
#==========================================  
    df.at[row,col] = Value

#==========================================  
def example_index_by_LABELS(): # Label == named indexs
#==========================================  
    # .LOC() = SELECT BY ROW AND COLUMN
    # ATTENTION: first row, then column
    # NOTE: Null-Slices are automatically added: p.loc['a'] is equivalent to to p.loc['a', :, :])
    
    df = pd.DataFrame(np.random.randn(6,4), index=list('abcdef'), columns=list('ABCD'))
    
    # ===========================
    # SLICING (always: row-col)
    # ===========================
    print df.loc[['a','b','d'],:] #note the additional brackets!!
    print df.loc[['a','b','d']]   # same effect
    
#        A         B         C         D
#     a  0.132003 -0.827317 -0.076467 -1.187678
#     b  1.130127 -1.436737 -1.413681  1.607920
#     d  0.974466 -2.006747 -0.410001 -0.078638

    # ===========================
    # SLICING (always: row-col)
    # ===========================
    myslice = df.loc['d':,'A':'C']
    print myslice.values                             #note: entries remain abstract "objects"!
    print np.asarray(myslice.values, dtype=np.float) #note: perfect float array / matrix
    
    # ===========================
    # SLICING (collected examples)
    # ===========================
    if 0:
        DF.loc[:, ("Intra", "Inter", "Merge")] #First rows then columns
        DF.loc[:4,:]  # NOTE: first pandas "inconsistency" : slice limits ("4") are included
        DF.loc[:4,:"NumFr"]  #NOTE: second pandas "inconsistency" : no col indices allowed (wh00t?)
    
#         A         B         C
#     d  0.974466 -2.006747 -0.410001
#     e  0.545952 -1.219217 -1.226825
#     f -1.281247 -0.727707 -0.121306

    # LOGICAL ACCESS
    cols = df.loc['a']>0  # "trues" for all columns in 'a' which have an entry > 0
    print df.loc[:,cols]   # equivalent to slice df.loc[:,'A']
    
    # SET / GET VALUES EXPLICITELY:
    print df.get_value('a','A') # deprecated
    print df.at['a','A']
    print df.loc['a','A']
    
#==========================================  
def example_index_by_POSITION():
#==========================================  
    df = pd.DataFrame(np.random.randn(6,4), index=list('abcdef'), columns=list('ABCD'))    
    print df.iloc[:3] # first three rows    

#==========================================================================================================================
#=============================================== DATAFRAME CREATION =======================================================
#==========================================================================================================================

# 3 BASIC INIT OPTIONS!
#==========================================    
def example_Initialize_Dataframe():
#==========================================

    percent = available = rss = vms = shared = data = text = lib = 1.1
    val2 = 20
    
    RowName1 = "Row1"
    RowName2 = "Row2"
    
    #VERSION 1: SIMPLEST WAY
    Indices = ("Group1", "Group2", "Group3", "Group4")
    Columns  = ['POC0','POC1','POC2','POC3','POC4']
    ListOfLists=np.array([[3,6,8,9,6],[2,3,4,5,6],[4,5,6,7,8],[3,6,5,8,6]]) 
    df_filled = pd.DataFrame(ListOfLists, columns=Columns, index=Indices)
    df_empty  = pd.DataFrame(             columns=Columns, index=Indices)
    df_filled.at["Group2","POC4"] = 20
    # result:
    #     POC0    POC1    POC2    POC3    POC4
    # Group1    3    6    8    9    6
    # Group2    2    3    4    5    6
    # Group3    4    5    6    7    8
    # Group4    3    6    5    8    6

    #VERSION 2: ONELINER
    df1 = pd.DataFrame({'percent':[percent,val2],'available':[available,val2]})
    
    #VERSION 3: "MULTIPLE-LINE ONLINER" :)
    df2 = pd.DataFrame({'percent': [percent,val2],
                    'available': [available,val2],
                    'rss': [rss,val2],
                    'vms': [vms,val2],
                    'shared': [shared,val2],
                    'data': [data,val2],
                    'text': [text,val2],
                    'lib': [1.4, None]}, index=(RowName1, RowName2)) #note: use None, otherwise values get duplicated!
               
    #VERSION 4: ELEGANT NUMPY IMPORT! (add 1D array as new row) 
    Col_Labels = ["percent", "available", "rss", "vms", "shared", "data", "text", "lib"]
    df3 = pd.DataFrame(columns = Col_Labels)    
    data   = np.array([[percent,available,rss,vms,shared,data,text,lib]])  # NOTE THE DOUBLE BRACKETS!
    df_row = pd.DataFrame(data, index=(RowName1,), columns=Col_Labels)
    df3 = df3.append(df_row)
    
    #VERSION 5: ELEGANT LIST IMPORT (each list gives a column, use transpose to stack rows)
    col1 = ['Bob','Jessica','Mary','John','Mel']
    col2 = [968, 155, 77, 578, 973]
    col3 = [0.1, 0.2, 0.3, 0.4, 0.5]
    BabyDataSet = zip(col1, col2, col3) #Zip: v-stacks the columns!
    df4 = pd.DataFrame(data = BabyDataSet, columns=['Names', 'Births', 'Whatever'])
    df4 = df4.transpose() # optional
    # Output:
    # Names    &  Bob &  Jessica &  Mary &  John &  Mel \\
    # Births   &  968 &      155 &    77 &   578 &  973 \\
    # Whatever &  0.1 &      0.2 &   0.3 &   0.4 &  0.5 \\
    
    save_DF_to_Excel(df1, "TEST_df1.xlsx")
    save_DF_to_Excel(df2, "TEST_df2.xlsx")
    save_DF_to_Excel(df3, "TEST_df3.xlsx")
    save_DF_to_Excel(df4, "TEST_df4.xlsx")

    #df3.plot()
    #plt.show()
    
#==========================================
def example_Create_Empty_Dataframe():
#==========================================
    rows    = np.arange(0,1000)
    columns = ['A', 'B', 'C']

    # VERSION 1: All NaNs (standard)
    df1 = pd.DataFrame(index=rows, columns=columns)
    
    # VERSION 2: All zeros
    df2 = df1.fillna(0)
    
    # VERSION 3: Random values
    df3 = pd.DataFrame(np.random.randn(1000, 4), index=rows, columns=columns )

#==========================================
def example_Create_Multiindex_Dataframe():
#==========================================    
        columns = pd.MultiIndex.from_tuples([
        ('A', 'cat', 'long'), ('B', 'cat', 'long'),
        ('A', 'dog', 'short'), ('B', 'dog', 'short')
        ],
        names=['exp', 'animal', 'hair_length']
        )
       
        df = pd.DataFrame(np.random.randn(4, 4), columns=columns)
        save_DF_to_Excel(df, TmpDir + "/Test.xlsx") 
        save_DF_to_Latex(df, TmpDir + "/Test.tex")
        print df

#==========================================================================================================================
#=========================================== ACCESS TO OTHER INFORMATION ==================================================
#==========================================================================================================================

#==========================================  
def get_Indices(df): #get list of "row labels" (=indices)
#========================================== 
    return df.index.tolist()
    
#==========================================================================================================================
#=============================================== APPEND / DELETE ENTRIES ==================================================
#==========================================================================================================================

#==========================================
def rename_Indices(df):  
#========================================== 
    df.rename(index={"SIFT" : "ABC"})
    
#==========================================
def rename_Cols(df, tr_dict):  
#========================================== 
    df.rename(columns=tr_dict) #e.g: {"GT-BGS" : "GT Mask", "No-BGS" : "No Mask" }
    
#==========================================
def example_append_Col(df):  
#========================================== 
    #Trick: create column as series, then set column with this series
    sLength = len(df['PSNR'])
    df['AvgMBitsPerFrame'] = pd.Series(np.random.randn(sLength), index=df.index)

#==========================================
def example_append_Row():  #append a row to a DF
#==========================================    
    df = pd.DataFrame(np.random.randn(10, 4), index=np.arange(0,10), columns=list('ABCD'))
    
    # create one-row dataframe (with according index and correct column lables)
    data = np.random.randn(1,4) #create a row vector (randn(4) would be a column vector!)
    print data
    print data.shape
    onerow = pd.DataFrame(data, index=(11,), columns=list('ABCD'))
    
    df = df.append(onerow) # append the row
    print df   
    save_DF_to_Excel(df, "TEST_append_Row.xlsx")


#==========================================    
def example_append_Series_to_DF():
#==========================================
    # NOTE: Dataseries are complicated fellas
    # Problem is that they cannot be appended to a dataframe which has its column names already set. SRSLY?

    #this will work:
    df = pd.DataFrame()
    #while this will NOT:
    #df = pd.DataFrame(columns = ['A','B','C','D']  )
    AbsSecs = get_Seconds_since_Midnight()
    s1 = pd.Series([4,2,1,1])
    s1.name = AbsSecs
    df = df.append(s1)
    s2 = pd.Series([4,2,1,5])
    s2.name = AbsSecs + 2
    df = df.append(s2)
    s3 = pd.Series([4,2,1,7])
    s3.name = AbsSecs + 5
    df = df.append(s3)
    s4 = pd.Series([4,2,1,0])
    s4.name = AbsSecs + 100
    df = df.append(s4)
    
    save_DF_to_Excel(df, "TEST_append_Series_to_DF.xlsx")
    df.plot()
    plt.show()
    
#==========================================    
def delete_columns(df, ColumnNames):
#==========================================
    for ColName in ColumnNames:
        df = df.drop(ColName, 1) #STD column gives no information     
    return df
    
    
#==========================================================================================================================
#==================================================== PLOTTING  ===========================================================
#==========================================================================================================================

#==========================================  
def example_provide_axes_for_Pandas():
#==========================================  
    f = plt.figure(figsize=(10, 10))
    df = pd.DataFrame(np.random.randn(6,4), index=list('abcdef'), columns=list('ABCD'))    
    df.plot(ax=f.gca()) # figure.gca means "get current axis"
    plt.title('Title here!', color='black')
    #ALTERNATIVE: ax = df.plot(kind='bar', figsize=(10, 10))  
  
#==========================================    
def plot_two_columns(ColName1, ColName2):
#==========================================
    df.plot(x=ColName1, y=ColName2)

#==========================================
def set_pandas_printoptions():
#==========================================
    pd.set_option('display.height', 1000)
    pd.set_option('display.max_rows', 500)
    pd.set_option('display.max_columns', 1000)
    pd.set_option('display.width', 1000)

#==========================================================================================================================
#==================================================== SAVE / LOAD =========================================================
#==========================================================================================================================    
    
#==========================================
def save_Excel_and_BarPlotJPG(df, DIRBN):
#==========================================
    save_DF_to_Excel(df, DIRBN + ".xlsx")
    save_DF_to_Barplot(df, DIRBN + ".jpg")
    
#==========================================
def save_DF_to_Excel(df, FN, TabName="Data", float_format="%0.4f"):
#==========================================
    print "save_Dataframe_to_Excel(): ", FN
    writer = pd.ExcelWriter(FN)
    df.to_excel(writer, TabName, float_format=float_format)
    writer.save() #closes the writer object
       
#==========================================
def save_DF_to_Barplot(df, FN,  figsize=(A4_x,A4_y/4), Title="", yLabel="", colors=None, ArgList={}):
#==========================================
    #from itertools import cycle, islice
    print "save_DF_to_Barplot(): ", FN
      
    #ax.axis["bottom"].major_ticklabels.set_rotation(30) 
    fig, axarr = plt.subplots(1, 1, figsize=figsize) 
    ax_Quick_Barplot(axarr, df, Title="", yLabel="", colors=colors, ArgList = ArgList)
    plt.tight_layout(pad=0.8, w_pad=2.0, h_pad=2.0)
    plt.savefig(FN)
    close_All_Figures()
    
#==========================================
def load_DF_from_Excel(FN, TabName = "First"):
#==========================================    
    print "load_Dataframe_from_Excel(): ", FN
    import pandas as pd
    excelobject = pd.ExcelFile(FN)
    if TabName == "First":
        df = excelobject.parse(excelobject.sheet_names[0])
    else:
        df = excelobject.parse(TabName)
    return df

#==========================================
def save_DF_to_JPG(df, FN, float_format='%0.1f', Rulers = []):
#==========================================
    tmptmpDir = TmpDir + "/tmptmp"
    createDir(tmptmpDir)
    # func'y lamda func FTW (remove trailing zeros, "%0.3g" will switch to scientific notation)
    lamdafunc = lambda x: (float_format % x).rstrip('0').rstrip('.')
    RawTexcode = df.to_latex(float_format=lamdafunc)
    Texcode = beautify_Table(RawTexcode, Rulers)
    
    #writeToFile(tmptmpDir + "/rawtable.tex", RawTexcode)
    writeToFile(tmptmpDir + "/table.tex", Texcode)
    
    OldDir = changeDir(tmptmpDir)
    runCall("pdflatex table.tex")
    runCall("pdfcrop table.pdf table.pdf")
    runCall('convert -density 300 table.pdf -quality 90 "%s"' % FN)
    print "Written latex table as JPG to: ", FN
    changeDir(OldDir) # and back we go                       
                
#==========================================
def save_DF_to_Latex(df, FN, float_format='%0.1f', Rulers = []):
#==========================================
    # func'y lamda func FTW (remove trailing zeros, "%0.3g" will switch to scientific notation)
    lamdafunc = lambda x: (float_format % x).rstrip('0').rstrip('.')
    RawTexcode = df.to_latex(float_format=lamdafunc)
    RawTexcode.replace("tabular", "tabulary")
    Texcode = beautify_Table(RawTexcode, Rulers)
    writeToFile(FN, Texcode)

#     assert(0) #do NOT use, only after backmerge of above beauty code
#     Texcode = df.to_latex(float_format=lambda x: float_format % x) # interesting! formatter function via lambda-expression!
#     Texcode.replace("tabular", "tabulary") # pandas does not allow this explicitely
#     writeToFile(FN, Texcode)  
    
#==========================================
def save_DF_to_CSV(df, FN, float_format='%0.1f'):
#==========================================
    csv = df.to_csv(FN, sep='\t', encoding='utf-8', float_format=lambda x: float_format % x) # interesting! formatter function via lambda-expression!
    #writeToFile(FN, csv)  

#==========================================
def save_DF_to_HTML(df, FN):
#==========================================
    Header = r'''
<!DOCTYPE html>
<html lang="en">
<head>
<meta charset="utf-8"/>
<style>.dataframe td { text-align: right; }</style>
    '''

    print "save_DF_to_HTML(): ", FN
    HTML_Code = Header + df.to_html(float_format='{0:.10f}'.format, classes='right_aligned_df')
    #print HTML_Code.encode("utf-8")
    with open(FN, "w") as myfile:
        myfile.write(HTML_Code.encode("utf-8"))

