#!/usr/bin/env python
# coding: utf8
# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)

from Imports_Basic import *
set_HARP_Core_Environment(__name__, __file__)

import matplotlib
import pandas as pd
#from Pandas import *  #DONT DO THIS! ECLIPSE CRASH!

from matplotlib.figure import Figure
import matplotlib.mlab as mlab
import matplotlib.pyplot as plt
import pylab

from General import *
from OpenCV import *

print "Used Matplolib: ", matplotlib.__version__

#==========================================
# DEFAULT SETUP (Latex + RC PARAMS)
#==========================================

# EFFECT IS UNCLEAR
# HOWEVER: above set_Font only produces Latex computermodern style if usetex=True
plt.rc('text', usetex=True)


plt.rcParams['lines.linewidth'] = 2  
plt.rcParams['legend.fontsize'] = 8  
plt.rcParams['legend.columnspacing'] = 0.6
plt.rcParams['axes.titlesize'] = 11 
plt.rcParams['figure.dpi']     = 200 
plt.rcParams['savefig.dpi']    = 300
plt.rcParams['savefig.jpeg_quality'] = 95
#plt.rcParams['figure.titlesize'] = 18 # not available in current Matplotlib

#     if Choice == "Palatino":  # latex-CM-like
#         plt.rc('font',**{'family':'serif','serif':['Palatino']})
#     elif Choice == "Helvetica": # std font (assumed)
#        
#OLD:
#plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'serif','serif':['Palatino']})

#NEW:
#plt.rc('font',**{'family':'serif','serif':["Nimbus Roman No9 L"]}) #closest to Times New Roman / Computer Modern
plt.rcParams.update({'font.size': 10})

#deprecated, remove
DPI = 200

# plt.rc('font', family='serif')
# plt.rcParams.update({'font.size': 12})

if 0: # Good for plotting SMALL ARRAYS
    #plt.rcParams['lines.linewidth'] = 2 
    marker_G_O = { 'color': 'lawngreen', 'marker': 'o', 'markersize' : 5, }
    marker_R_O = { 'color': 'red', 'marker': 'o', 'markersize' : 2} 
    marker_B_O = { 'color': 'blue', 'marker': 'o', 'markersize' : 2}    
if 0: # Good for plotting LARGE ARRAYS
    #plt.rcParams['lines.linewidth'] = 1 
    marker_G_O = { 'color': 'lawngreen', 'marker': 'o', 'markersize' : 2, }
    marker_R_O = { 'color': 'red', 'marker': 'o', 'markersize' : 1} 
    marker_B_O = { 'color': 'blue', 'marker': 'o', 'markersize' : 1}   


LINE_A = dict( linewidth=2, linestyle="-" )
LINE_B = dict( linewidth=6, linestyle="-" )


#==========================================
# IMPORTANT: use copy_to_PlotExamples() whenever possible
#==========================================




#==========================================
# QT (DISABLED)
#==========================================
# FOR NOW: DO NOT USE MATPLOTLIB WITH QT-BACKEND FROM INSIDE HARP
# QT BACKEND WILL CRASH ON APPLICATION EXIT DUE TO CONCURRENT C++ CALLS, THREAD SAFETY IS HARMED 
# QT GARBAGE COLLECTION MAY ALSO CRASH


#matplotlib.use('Qt4Agg')
#matplotlib.rcParams['backend.qt4']='PyQt4'
#from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas

# matplotlib is rather complicated when it comes to interpreting images. It goes roughly as follows:
#     if the image is a NxM array of any type, it is interpreted through the colormap (autoscale, if not indicated otherwise). (In principle, if the array is a float array scaled to 0..1, it should be interpreted as a grayscale image. This is what the documentation says, but in practice this does not happen.)
#     if the image is a NxMx3 float array, the RGB components are interpreted as RGB components between 0..1. If the values are outside of this range, they are taken with positive modulo 1, i.e. 1.2 -> 0.2, -1.7 -> 0.3, etc.
#     if the image is a NxMx3 uint8 array, it is interpreted as a standard image (0..255 components)
#     if the image is NxMx4, the interpretation is as above, but the fourth component is the opacity (alpha)
#     if you give matplotlib a NxMx3 array of integers other than uint8 or float, the results are not defined.


#==========================================
# TIGHT LAYOUT
#==========================================
#pad: surounding borders
#w/h: internal distances
#rect: (left, bottom, right, top)  # usefull for making space at top
TIGHT_PADS = dict(pad=1.0, w_pad=2.0, h_pad=2.0, rect= (0.0, 0.0, 1.0, 0.97) ) #for 1/2th of DINA4 page
TIGHT_PADS2 = dict(pad=1.0, w_pad=2.0, h_pad=2.0, rect= (0.0, 0.0, 1.0, 0.95) ) #for 1/4th of DINA4 page
if 0:
    plt.tight_layout(**TIGHT_PADS) 

#==========================================
# LEGEND
#==========================================
# 'best'         : 0, (only implemented for axes legends)
# 'upper right'  : 1,
# 'upper left'   : 2,
# 'lower left'   : 3,
# 'lower right'  : 4,
# 'right'        : 5,
# 'center left'  : 6,
# 'center right' : 7,
# 'lower center' : 8,
# 'upper center' : 9,
# 'center'       : 10,

# BBOX_TO_ANCHOR: http://stackoverflow.com/questions/25068384/bbox-to-anchor-and-loc-in-matplotlib

# LEGEND FONT SIZE: ax.legend(LegendLabels, loc='upper right', fontsize=10)
# LEGEND TITLE: ax.legend(LegendLabels, title=r"Sequ. Spincalendar")
# LEGEND COLUMNS: ax.legend(LegendLabels, ncol=2)

# LGEND TITLE FONT SIZE:
#     leg = axarr[1].legend( ("Map-MSE", "Corner-Deltas"), title="Indicated by:")
#     leg.get_title().set_fontsize('8')

LEGEND_BBOX = dict(bbox_to_anchor=[1.02, 1.2], loc='upper right')
LEGEND_BBOX2 = dict(bbox_to_anchor=[1.1, 1.2], loc='upper right')
SUPTITLE_STYLE = dict( fontsize=14, weight="heavy" )

#==========================================
# MARKERS & LINESTYLES
#==========================================
    #col = ['b','r','g','c','y','m','k','cyan','violet','peru','darkcyan']
    #mar = ['+','o','x','D','d','H','>','<','*','s','^','|'] 

    #ax.plot( ... , linewidth=2, linestyle="--", color=ORANGE)


#==========================================
# DA AXIS HORROR FIGHT (unraveling)
#==========================================
if 0:
    o = self.cols # MOST ELEGANT
    return self.fig.axes[r*o+c] # ROC'Z!

    get_XY_From_Idx() #may help (check!)
    
    axarr_list = axarr.reshape(-1) # MOST NERDY
    axarr_list = list(itertools.chain(*axarr)) 
    axarr_list = axarr[0,0], axarr[0,1], axarr[1,0], axarr[1,1], axarr[2,0], axarr[2,1] # MOST UGLY

#==========================================
# TWIN AXES | Split axes for different scales
#==========================================
if 0:
    ax2 = ax1.twinx()
    for tks in ax2.get_yticklabels():
        tks.set_color(color)  

#======================
def get_XY_From_Idx(idx, DimX, DimY):
#======================
    Y = np.floor(idx / DimX)
    X = (idx % DimX)
    return X,Y
    



#==========================================
# LATEX
#==========================================
if 0:
    set_Font(0)
    set_Font_Size(12)
    set_Latex()

#==========================================
# GENERAL SETUP
#==========================================
# DELETE LATER
# # we need to provide inch values for plt.subplots(..)
# # note that DPI must also be provided to plt.savefig to keep HD resolution
# DPI = 100 #TBD! BUG! decreasing this will increase the resolution!
# #deprecated!
# A4q_x, A4q_y =1920/DPI, 1080/DPI
# #use this!!
# LandscapeX, LandscapeY = 1920/DPI, 1080/DPI
# PortraitX, PortraitY = 1080/DPI, 1920/DPI
#



def mm2inch(value):
    return value/10.0/2.54

#print "DINA4 in inch: ", mm2inch(210), mm2inch(297)
#returns: 8.26771653543 11.6929133858

A3_x,  A3_y  =  mm2inch(297), mm2inch(420)
A3q_x, A3q_y =  mm2inch(420), mm2inch(297)
A4_x,  A4_y  =  mm2inch(210), mm2inch(297)
A4q_x, A4q_y =  mm2inch(297), mm2inch(210)
A5_x,  A5_y   =  mm2inch(148), mm2inch(210)
A5q_x, A5q_y =  mm2inch(210), mm2inch(148)
A6_x,  A6_y  =  mm2inch(105), mm2inch(148)
A6q_x, A6q_y =  mm2inch(148), mm2inch(105)

# A0    841 x 1189
# A1    594 x 841
# A2    420 x 594
# A3    297 x 420
# A4    210 x 297
# A5    148 x 210
# A6    105 x 148
# A7    74 x 105
# A8    52 x 74
# A9    37 x 52
# A10   26 x 37

Plot_JPG_Quality = 95 #1=worst 100=best

#==========================================
# COMMON MARKERS / COLORS (unroll with: **GREEN_MARKER)
#==========================================
RED_MARKER   = { 'color': 'r',    'marker': '+', 'linewidth' : 2 }
BLUE_MARKER  = { 'color': 'b',    'marker': 'x', 'markersize' : 4, 'linewidth' : 2 }
GREEN_MARKER = { 'color': 'g',    'marker': '.', 'markersize' : 10, 'linewidth' : 2 }

ORANGE_MARKER = { 'color': 'darkorange',    'marker': '^', 'markersize' : 6, 'linewidth' : 2 }
#CYAN_MARKER = { 'color': 'cyan',    'marker': 'v', 'markersize' : 6, 'linewidth' : 2 }
LIGHTGREEN_MARKER = { 'color': 'lawngreen',    'marker': 'v', 'markersize' : 6, 'linewidth' : 2 }

marker_G_O = { 'color': 'lawngreen', 'marker': 'o', 'markersize' : 6}
marker_R_O = { 'color': 'red', 'marker': '+', 'markersize' : 6}

#==========================================
# SEQUENTIAL CMAPS
#==========================================
my_cmaps = (['Blues', 'BuGn', 'BuPu',
    'GnBu', 'Greens', 'Greys', 'Oranges', 'OrRd',
    'PuBu', 'PuBuGn', 'PuRd', 'Purples', 'RdPu',
    'Reds', 'YlGn', 'YlGnBu', 'YlOrBr', 'YlOrRd']),


#==========================================
# SUBPLOTS SWISS KNIFE
#==========================================
#==========================================
def subplots_swiss_knife_CURVE():
#==========================================
    fig, axarr = plt.subplots(2, 2, figsize=(A4_x,A4_y/2))
    FigFN = CoreTmpDir + "/subplots_swiss_knife_CURVE.jpg"
    fig.suptitle("Supertitle")
    
    ax = fig.axes[0]
    ax.plot(range(100))
    ax.set_title("Ax Title")
    plt.tight_layout(**TIGHT_PADS) 
    savePlot(FigFN) 
    plt.show()
    close_All_Figures()
    
#==========================================
def subplots_swiss_knife_IMG():
#==========================================
    fig, axarr = plt.subplots(2, 2, figsize=(A4_x,A4_y/2))
    FigFN = CoreTmpDir + "/subplots_swiss_knife_IMG.jpg"
    fig.suptitle(FigFN)
    
    Img = np.zeros((100,100,3), np.uint8)
    ax = fig.axes[0]
    ax.get_xaxis().set_major_locator(plt.NullLocator()) #for images
    ax.get_yaxis().set_major_locator(plt.NullLocator()) #for images
    ax.imshow(Img)
    ax.set_title("Ax Title")
    plt.tight_layout(**TIGHT_PADS) 
    savePlot(FigFN) 
    plt.show()
    close_All_Figures()

#==========================================
def plotImg(Img):
#==========================================
    plt.imshow(Img)
    plt.show()

# alias
def SHOWIMG(Img):
    plotImg(Img)
    
def SHOWIM(Img):
    plotImg(Img)

#==========================================================================================================================
#================================================== LATEX + FONT ==========================================================
#==========================================================================================================================

# OLD
# from matplotlib import rc
#rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
## for Palatino and other serif fonts use:
#rc('font',**{'family':'serif','serif':['Palatino']})
#rc('text', usetex=True)

#deprectated
#==========================================
def set_Font_Size(FontSize = 12):  # 12 is default
#==========================================
    assert(0) #remove, agreement on default (see Plot.py)
    plt.rcParams.update({'font.size': FontSize})

#==========================================
def set_Font(Choice = "Palatino"):  
#==========================================  
    assert(0) #remove, agreement on default (see Plot.py)
    
    if Choice == "Palatino":  # latex-CM-like
        plt.rc('font',**{'family':'serif','serif':['Palatino']})
    elif Choice == "Helvetica": # std font (assumed)
        plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
    else:
        plt.rc('font',**{'family':'serif','serif':[Choice]})
                
#==========================================
def set_Latex():  
#==========================================      
    assert(0) #remove, agreement on default (see Plot.py)
    # EFFECT IS UNCLEAR
    # HOWEVER: above set_Font only produces Latex computermodern style if usetex=True
    plt.rc('text', usetex=True)

#==========================================================================================================================
#========================================== OPERATIONS ON WHOLE FIGURES ===================================================
#==========================================================================================================================

# ======================================
def saveFig(FN):
# ======================================
    DPI=300
    print "Saving figure to: ", FN
    plt.savefig(FN,dpi=DPI)
    crop_Img_Borders_FN(FN)
    close_All_Figures()


# set __builtins__.ExportFigures = True for CPKL export (huge!)
# ======================================
def save_And_Close(fig, FN, Category="", Args=None, DPI=300, Crop=True):
# ======================================
    print "Saving figure to: ", FN
    plt.savefig(FN,dpi=DPI)
    if Crop:
        crop_Img_Borders_FN(FN)
    if Category != "":
        copy_to_SubDir(FN, Category) # DirName = Category
    #if Args != None: # for later replotting
    #DEPRECATED, REMOVE:
    if __builtins__.has_key("ExportFigures"):
        if ExportFigures == True:
            CPKL_FN = removeSuffix(FN) + ".cpkl"
            print "Saving CPKL of fig to: ", CPKL_FN
            save_CPickle_FN(Args, CPKL_FN)
    close_All_Figures()

#==========================================
def create_Legend(Labels, loc='center right'):  
#==========================================
    plt.legend(Labels, loc=loc)
# ‘best’     0
# ‘upper right’     1
# ‘upper left’     2
# ‘lower left’     3
# ‘lower right’     4
# ‘right’     5
# ‘center left’     6
# ‘center right’     7
# ‘lower center’     8
# ‘upper center’     9
# ‘center’     10


#==========================================
def set_fig_suptitle(fig, supertitle): 
#==========================================
    fig.suptitle(supertitle)
    
#==========================================
def getCurrentAxis():  
#==========================================
    return plt.gca() #one axis == subplot!!

#==========================================
def getCurrentFigure():  
#==========================================
    return plt.gcf()

#==========================================
def makeSpaceForSubtitle():  # make space for multiline suptitle
#==========================================
    plt.subplots_adjust(top=0.90)
    
# delete / clear all figures
#==========================================
def close_All_Figures():  # make space for multiline suptitle
#==========================================    
    Figures=[manager.canvas.figure for manager in matplotlib._pylab_helpers.Gcf.get_all_fig_managers()]
    #print "Open Figures", Figures
    
    print "Closing %d Figures" % len(Figures)
    for Fig in Figures:
        #print "Closing " + str(Fig)
        Fig.clf()
    plt.close()   
    

    
#==========================================    
def getCustomCMap_RedToGreen():
#==========================================

    cdict = {'red':   ((0.0, 0.0, 0.0),
                       (0.5, 0.0, 0.0),
                       (1.0, 1.0, 1.0)),
             'blue':  ((0.0, 0.0, 0.0),
                       (1.0, 0.0, 0.0)),
             'green': ((0.0, 0.0, 1.0),
                       (0.5, 0.0, 0.0),
                       (1.0, 0.0, 0.0))}
    
    import matplotlib.colors as mcolors
    custom_cmap = mcolors.LinearSegmentedColormap('custom_cmap', cdict, 100)
    return custom_cmap

#==========================================================================================================================
#============================================ OPERATIONS ON SUBPLOTS ======================================================
#==========================================================================================================================

#run:
#     fig, axarr = plt.subplots(1, 1, figsize=figsize) 
#     ax_Quick_Barplot(axarr, df)
#     plt.tight_layout(pad=0.8, w_pad=2.0, h_pad=2.0)
#     plt.savefig(FN)
#     close_All_Figures()
#==========================================
def ax_Quick_Barplot(ax, df, Title="", yLabel="", colors=None, ArgList = {}):
#==========================================
    from itertools import cycle, islice
    if colors == None:
        colors = list(islice(cycle(['b', 'r', 'g', 'y', 'k', 'darkorange']), None, df.shape[1]))
    
    ax = df.plot(kind='bar', ax=ax, rot=0, color=colors, **ArgList)
    #plt.setp(plt.xticks()[1], rotation=0) #required
    ax.set_title(Title)
    ax.set_ylabel(yLabel)
    #ax.set_xticklabels(ax.xaxis.get_majorticklabels(), rotation=45)

#==========================================
def ax_annotate(ax, Label):  
#==========================================       
    ax.axhline(y=15, color="lawngreen", linewidth=1)
    #xytext is best chosen in "axes fractions"
    # 0.1, 0.1 means lower left (origin!)
    ax.annotate('Chosen Threshold', xy=(100, 15), xytext=(0.5, 0.7), textcoords='axes fraction', \
                horizontalalignment='center', arrowprops=dict(facecolor='black', shrink=0.05),)
    


#==========================================
def set_ax_xlabel(ax, Label):  
#==========================================    
    ax.set_xlabel(Label)
        
#==========================================
def set_ax_ylabel(ax, Label):  
#==========================================    
    ax.set_ylabel(Label)

#==========================================
def set_ax_title(ax, Title): #title above subplot 
#==========================================   
    ax.set_title(Title)
    
#==========================================
def set_ax_limits_X(ax, Min, Max):  
#==========================================       
    ax.set_xlim([Min, Max])
 
#==========================================
def set_ax_limits_Y(ax, Min, Max):  
#==========================================       
    ax.set_ylim([Min, Max])
    
#==========================================
def set_ax_single_limit(ax):  
#==========================================    
    # only effective after plotting   
    ax.set_ylim(-1000.0, ax.get_ylim()[1])
    
#==========================================
def disable_ax_drawing(ax):  
#========================================== 
    ax.axis('off') #no ticks, no labels, no frame 
   
#==========================================
def get_ax_ImageObject(ax):#retrieve a possible ImageObject of this axis
#==========================================
    for obj in ax.get_children():
        if isinstance(obj, matplotlib.image.AxesImage):
            return obj
    return None

#==========================================
def set_ax_BooleanFormat(ax):
#==========================================
    u, l = 1.05, -0.05
    # =============================
    def formatter(x, pos): 
    # =============================
        LUT = ("False", "True")
        #f x not in (0,1):
        if l-0.1 <= x <=l+0.1 or u-0.1 <= x <=u+0.1:
            return LUT[np.int(x)]
        else: return " "
    ax.yaxis.set_major_formatter(matplotlib.ticker.FuncFormatter(formatter))
    ax.set_ylim(l, u) #plus headroom
    
#use: axarr[1].yaxis.set_major_formatter(matplotlib.ticker.FuncFormatter(my_Axis_Formatter))
# =============================
def my_Axis_Formatter(x, pos): # sort of "filter" on axis labels (woa!)
# =============================
    #LUT = ("Not detected", "Detected")
    LUT = ("False", "True")
    if x not in (0,1):
        return " "#str(x)
    else: return LUT[np.int(x)]
    #return "$2^{{{0}}}$".format(x)

#==========================================
def create_Epic_Legend(ax, Colors, Labels, bbox=(1.0, 1.0) ):
#==========================================
#     bbox = (1.2, 1.1)  -> upper right
#     bbox = (0.1, 1.1)   -> upper left

    Patches = []
    for Color in Colors:
        # "edgecolor": frame of box, "facecolor": entry of box, "color": overwrites both!!
        Patch = matplotlib.patches.Patch(edgecolor="k", facecolor=scaleColor(Color))
        Patches.append(Patch)
    ax.legend(Patches, Labels, ncol=1, fontsize=8, labelspacing=None, bbox_to_anchor=bbox) 
   
# ==========================================
def ax_add_Headroom(ax, f=1.3):
# ==========================================
    cur_min, cur_max = ax.get_ylim()
    ax.set_ylim(cur_min, (cur_max) * f)
   
#==========================================
def set_ax_OnlyIntegerValues(ax):
#==========================================
    xaxis = ax.get_xaxis()
    xaxis.set_major_locator(matplotlib.ticker.MaxNLocator(integer=True))

#==========================================
def set_ax_DisableTicks(ax): #pretty plotting of images
#==========================================
    plt.axes(ax) #set focus
    
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())

#still used. function unknown.
#==========================================
def set_ax_DisableTicks_2(ax):  
#========================================== 
    # just getting rid of the frame is complicated
    # Ticks will still remain visible
    ax.axis('on') #safety, it may be disabled
    ax.set_frame_on(False)
    ax.set_xticklabels([])
    ax.set_yticklabels([])
    ax.get_xaxis().set_ticks([])
    ax.get_yaxis().set_ticks([])

#==========================================
def reselectAxis(AxesList, AxIdx): # reselect an axis (=subplot!!) 
#==========================================
    #see compareMapBits_FinalCUs() (->bar plots)
    plt.sca(AxesList[AxIdx])   
    
#==========================================
def set_Clim_of_AxesImageChild_of_Axis(ax, vmin, vmax):  
#==========================================
    AxesImage = getAxesImageFromAxis(ax)
    AxesImage.set_clim([vmin,vmax])
    getCurrentFigure().canvas.draw()

#==========================================================================================================================
#====================================================  SWISS KNIFE ========================================================
#==========================================================================================================================

# EXAMPLE:
# axarr = plt.subplots(3, 3, figsize=(A4q_x*0.9,A4q_y*0.75/6)) 
# plot_Img_SUBPLOT(Img, axarr[0][0])
# plt.show()  
# real examples: search for plot_Img_SUBPLOT(Imgs[idx],  ax, Imgs_Labels[idx])  
#==========================================
def plot_Img_SUBPLOT(Img, ax,     IDStr="", cmap='gray', scale=True):
#==========================================
    plt.axes(ax) #set focus
    
    #FIXED, before: prepare_Axis_for_Img(ax)
    ax.get_xaxis().set_major_locator(plt.NullLocator())
    ax.get_yaxis().set_major_locator(plt.NullLocator())
    
    if Img != None:
        #STRANGE BEHAVIOR. CLARIFY. DISABLED FOR NOW
#         if len(Img.shape) == 3:
#             Img = switchRGB(Img) #matplotlib disrespects BGR
        plt.imshow(Img)
        ax.set_title(IDStr)
    else:
        ax.axis('off') 
        #print "HELLO WORLD\nHELLO WORLD\nHELLO WORLD\nHELLO WORLD\n"
        #ax.set_aspect('auto')
        #ax.set_autoscale_on(True)
        pass
        #plt.axis('off')
        #plt.imshow(np.zeros((10,10,3), np.uint8)) #dummy
#     if scale is True: #let matplotlib decide the dynamic range
#         plt.imshow(Img, cmap=pylab.gray())
#     else: 
#         plt.imshow(Img, cmap=pylab.gray(), vmin=0, vmax=255)
    #plt.axis('off')


# EXAMPLE:
# fig = plt.figure(figsize=(22,11.9))
# fig.canvas.set_window_title(Sequ.FN)
# plt.subplot(5, 1, 1) 
# plt.subplots_adjust(hspace = 0.5)
# subplot_Bar(TS_POCs, BGS_Ducati.TS_SkipPercentage, "BGS_Ducati.TS_SkipPercentage", (1, Sequ.Fr))
#==========================================
def plot_Bar_SUBPLOT(x, y, title, xlimits):
#==========================================
    # we need xlimits, otherwise leading/trailing zero elements are not shown
    plt.bar(x, y)
    plt.xlim(xlimits[0], xlimits[1])
    plt.title(title)

#==========================================================================================================================
#======================================= WIDGETS / INTERACTION / CLICKS ===================================================
#==========================================================================================================================

# SEE: http://matplotlib.org/1.3.1/examples/widgets/index.html




#==========================================
def get_2_Clicks(FN): #helpful for manual cropping without Gimp
#==========================================
    Img = readImg(FN)
    plt.imshow(Img)
    print("Please click")
    x = plt.ginput(2)
    print "Clicked: ", x
    print "Copy-paste this line. Easy!"
    sys.exit()

# GOOD! No inlining necessary. Use event.inaxes to get active axis. Metadata realized as new variables to ax object ("ax.Img") 
# Information is passed outside by declaring global variable 
#==========================================        
def CLICK_FUNCTION__2(event):
#========================================== 
    print "Clicked! POS: %0.2f, %0.2f, " % (event.xdata, event.ydata)   
    print event.inaxes
    fig2, ax2 = plt.subplots(1, 1, figsize=(A4_x,A4_y))
    ax2.imshow(event.inaxes.Img)
    pts = plt.ginput(2)
    print "Clicked: ", pts
    up, bottom = pts[0][1], pts[1][1]
    __builtins__.update(up = up)
    __builtins__.update(bottom = bottom)
    print "up, bottom = ", up, bottom
    plt.close()
    close_All_Figures()
    #plt.show()  


# BAD. We need to place this inline, so that axarr is known 
#==========================================        
def CLICK_FUNCTION(event):
#==========================================
   for ax in axarr.ravel():
       if event.inaxes==ax:
           print "Clicked! POS: %0.2f, %0.2f, " % (event.xdata, event.ydata),            

          


#==========================================================================================================================
#==================================================== HELPER ==============================================================
#==========================================================================================================================

# ============================
def multiplot_AllImages__FN(Img_FNs):
# ============================
    Imgs = []
    for FN in Img_FNs:
        Imgs.append(readImg(FN))
    return multiplot_AllImages(Imgs) 

# ============================
def multiplot_AllImages(Imgs):
# ============================

    NumPlots = len(Imgs)
    cols = int(np.floor(np.sqrt(NumPlots)))
    rows = int(np.ceil(NumPlots / cols))      
    fig, axarr = plt.subplots(rows, cols, figsize=(A4_x,A4_y))

    for idx, Img in enumerate(Imgs):
        ax = fig.axes[idx]
        ax.get_xaxis().set_major_locator(plt.NullLocator())
        ax.get_yaxis().set_major_locator(plt.NullLocator())
        ax.axis('off')
        ImgRGB = cv2.cvtColor(Imgs[idx], cv2.COLOR_BGR2RGB)
        ax.imshow(ImgRGB)
        ax.Img = Img

    plt.tight_layout(**TIGHT_PADS) 
    
    return fig

# ============================
def stack_beauty(Imgs, Descs, FN, cols=2, figsize=(A4_x,A4_y)):
# ============================
    
    #EXAMPLE:
    #Imgs  = VisWarping, vis_prefi, vis_postfi, vis_DiffY,  #vis_Outline
    #Descs =  "Ref (orange) and Cur (green)", "Y Difference", "Matching, After Prefilter", "Matching, After Postfilter", #"Vis Outline"
    #stack_beauty(Imgs, Descs, VisFN, figsize=(A4_x,A4_y/2))
    
    Imgs = copy.deepcopy(Imgs)
    NumSubplots = int(np.ceil(len(Imgs) / float(cols)) * cols)
    fig, axarr = plt.subplots(NumSubplots/cols, cols, figsize=figsize) #subplots-pkg
    axarr_list = fig.axes

    #for images
    for idx in range(len(Imgs)):
        ax = axarr_list[idx]
        ax.get_xaxis().set_major_locator(plt.NullLocator())
        ax.get_yaxis().set_major_locator(plt.NullLocator())
        ax.axis('off')
        ImgRGB = cv2.cvtColor(Imgs[idx], cv2.COLOR_BGR2RGB)
        ax.imshow(ImgRGB)
        ax.set_title(Descs[idx], size='large', weight="bold")

    plt.tight_layout(**TIGHT_PADS) 
    plt.savefig(FN, dpi=DPI)
    crop_Img_Borders_FN(FN)
    close_All_Figures()

#==========================================================================================================================
#====================================================  SIMPLIFIER =========================================================
#==========================================================================================================================

# ============================
def escapeUnderscores(StrList):
# ============================
    NewList = []
    for Str in StrList:
        Str = Str.replace('_', '\_')
        NewList.append(Str)
    return NewList

# ============================
def escape(Str):  # for latex plots
# ============================
    return Str.replace('_', '\_')

#==========================================
def minimal_Plot_Example(DiffMatrix, cmap='gray'): #any types
#==========================================
    import matplotlib.pyplot as plt
    plt.imshow(DiffMatrix, cmap=cmap)
    plt.colorbar()
    plt.show(block=True)
    plt.close()
    
#==========================================
def minimal_Quadplot_Example(Tuple, cmap='gray', block = True): #any types
#==========================================
    cmap = getCustomCMap_RedToGreen()

    import matplotlib.pyplot as plt
    fig = plt.figure(figsize=(A4q_x,A4q_y))
    plt.subplot(2, 2, 1)
    plt.imshow(Tuple[0], cmap=cmap)
    plt.colorbar()
    plt.subplot(2, 2, 2)
    plt.imshow(Tuple[1], cmap=cmap)
    plt.colorbar()
    plt.subplot(2, 2, 3)
    plt.imshow(Tuple[2], cmap=cmap)
    plt.colorbar()
    plt.subplot(2, 2, 4)
    plt.imshow(Tuple[3], cmap=cmap)
    plt.colorbar()
    plt.show(block=block)
    
#==========================================
def minimal_Multiplot_Example(Dims, MatTuple, cmap='custom', block = True, LabelTuple=None): #any types
#==========================================
    #Dims[0] == rows

    if  cmap == 'custom':
        cmap = getCustomCMap_RedToGreen()
        cmap = getCustomCMap_RedToGreen()
        
    assert Dims[0]*Dims[1] >= len(MatTuple), "Check subplot dims"
        
    def mysubplot(fig, Dims, Pos, Matrix, Label, cmap='gray'):
        
        #creates an matplotlib.axes.AxesSubplot object
        ax = plt.subplot(Dims[0], Dims[1], Pos)
        ax.set_title(Label, size='large', weight="bold")
        
        if len(Matrix.shape) == 3: #probably an OpenCV BGR image!
            Matrix = cv2.cvtColor(Matrix, cv2.COLOR_BGR2RGB)
            ax.get_xaxis().set_visible(False)
            ax.get_yaxis().set_visible(False)
            axax = ax.imshow(Matrix, cmap=cmap)
            test = 1
        else:
            #AbsMaximum = get_absolute_Maximum(Matrix)
            #plt.imshow(Matrix, cmap=cmap, vmin=-AbsMaximum, vmax=AbsMaximum)
            axax = ax.imshow(Matrix, cmap=cmap)
            #colorbar_ax = fig.add_axes([0.7, 0.1, 0.05, 0.8]) #does not work
            #fig.colorbar(axax, cax=colorbar_ax)
            from mpl_toolkits.axes_grid1 import make_axes_locatable
            divider = make_axes_locatable(ax)
            cax = divider.append_axes("right", size="5%", pad=0.05)
            plt.colorbar(axax, cax=cax)
            #fig.colorbar(axax)
            
        return ax
        
    import matplotlib.pyplot as plt
    fig = plt.figure(figsize=(A4q_x*2,A4q_y*2)) 
    axes = []
    for Idx, Matrix in enumerate(MatTuple):
        Label = LabelTuple[Idx] if LabelTuple !=None else " "
        if Matrix != None:
            ax = mysubplot(fig, Dims, Idx+1, Matrix, Label, cmap)
            axes.append(ax)
    plt.tight_layout(pad=0.8, w_pad=2.0, h_pad=2.0)
    if block == True:
        plt.show(block = True)
        
    return axes #we may want to further adjust selected subplots (limits, colormap etc.)

#==========================================
def plot_ImgBGR(Img, title="", cmap='gray', Block=False):
#==========================================
    RGB = cv2.cvtColor(Img, cv2.COLOR_BGR2RGB)
    plot_Img(RGB, title=title, cmap=cmap, Block=Block)

#==========================================
def plot_Img(Img, title="", cmap='gray', Block=False):
#==========================================

    Size = 1.0/1.2
    fig = plt.figure(1, figsize=(22*Size,11.9*Size))
    fig.clf() # fix memory leak of figure already open

    plt.imshow(Img) #,vmin=0,vmax=255)
    plt.title(title)
    plt.axis('off')

    if len(Img.shape) == 2: # luminance channel
        CMap = pylab.gray()
        plt.colorbar(CMap)
        
    plt.tight_layout()
    plt.show(block=Block)
    plt.pause(0.001) 
  
  

#==========================================================================================================================
#=================================================  SAVE PLOT =============================================================
#==========================================================================================================================

#==========================================
def savePlot(FN, dpi=DPI, quality=Plot_JPG_Quality):
#==========================================
    #give JPG or PNG ending!
    plt.savefig(FN) #, dpi=dpi, quality=quality)
    print "Saved plot: ", FN

#==========================================
def savePlot_SVG(FN): #any types
#==========================================
    assert getSuffix(FN) == ".svg"
    plt.savefig(FN)
    print "savePlot_SVG: saved ", FN
    
#==========================================
def savePlot_PNG(FN): #any types
#==========================================

    assert getSuffix(FN) == ".png"
#     plt.savefig(FN) # no longer used, we want a chance to invert!
    
    import PIL
    from cStringIO import StringIO
    buffer_ = StringIO()
    plt.savefig(buffer_, format = "png")
    buffer_.seek(0)
    image = PIL.Image.open(buffer_)
    ar = np.asarray(image)
    buffer_.close()

    print "savePlot_PNG: saved ", FN
    saveRGB(ar, FN) 

#========================================== 
def save_Side_By_Side(List0, List1, Descs):
#========================================== 
    for idx, Left in enumerate(List0):
        Right = List1[idx]
        hs = getSpacer("Vertical", Left)
        hs.fill(0)
        drawString(Left, Descs[0], Color=RED)
        drawString(Right,Descs[1], Color=RED)
        
        #OutFN = removeSuffix(FN) + "_SIDEBYSIDE.jpg"
        OutFN = KitTmpDir + "/SIDEBYSIDE_%d__%s__%s.jpg" % (idx, Descs[0], Descs[1])
        print "Saving side by side: " , OutFN
        Both = np.hstack((Left, hs, Right))
        saveBGR(Both, OutFN)

#wraps function above
#========================================== 
def save_Side_By_Side_FromFiles(FNList0, FNList1, Descs):
#========================================== 
    List0 = []
    List1 = []
    for idx, FN in enumerate(FNList0):
        List0.append(readImg(FN))
        List1.append(readImg(FNList1[idx]))
    save_Side_By_Side(List0, List1, Descs)


#==========================================    
def convert_Figure_To_Numpy ( fig, dpi=300 ):
#==========================================

    # DOES NOT WORK
    # convert a Matplotlib figure to a 4D RGBA numpy array
#     print "convert_Figure_To_Numpy"
#     fig.canvas.draw ( )
#     w,h = fig.canvas.get_width_height()
#     buf = np.fromstring ( fig.canvas.tostring_argb(), dtype=np.uint8 )
#     buf.shape = ( w, h,4 )
#     buf = np.roll ( buf, 3, axis = 2 )
#     return buf

    import PIL
    from cStringIO import StringIO
    buffer_ = StringIO()
    plt.savefig(buffer_, format = "png", dpi = dpi)
    buffer_.seek(0)
    image = PIL.Image.open(buffer_)
    RGB = np.asarray(image)
    buffer_.close()
    return RGB

#-
#==========================================================================================================================
#=================================================  COLOR MAPS ============================================================
#==========================================================================================================================

#===============
def shiftedColorMap(cmap, start=0, midpoint=0.5, stop=1.0, name='shiftedcmap'):
#===============
    '''
    Function to offset the "center" of a colormap. Useful for
    data with a negative min and positive max and you want the
    middle of the colormap's dynamic range to be at zero

    Input
    -----
      cmap : The matplotlib colormap to be altered
      start : Offset from lowest point in the colormap's range.
          Defaults to 0.0 (no lower ofset). Should be between
          0.0 and `midpoint`.
      midpoint : The new center of the colormap. Defaults to 
          0.5 (no shift). Should be between 0.0 and 1.0. In
          general, this should be  1 - vmax/(vmax + abs(vmin))
          For example if your data range from -15.0 to +5.0 and
          you want the center of the colormap at 0.0, `midpoint`
          should be set to  1 - 5/(5 + 15)) or 0.75
      stop : Offset from highets point in the colormap's range.
          Defaults to 1.0 (no upper ofset). Should be between
          `midpoint` and 1.0.
    '''
    cdict = {
        'red': [],
        'green': [],
        'blue': [],
        'alpha': []
    }

    # regular index to compute the colors
    reg_index = np.linspace(start, stop, 257)

    # shifted index to match the data
    shift_index = np.hstack([
        np.linspace(0.0, midpoint, 128, endpoint=False), 
        np.linspace(midpoint, 1.0, 129, endpoint=True)
    ])

    for ri, si in zip(reg_index, shift_index):
        r, g, b, a = cmap(ri)

        cdict['red'].append((si, r, r))
        cdict['green'].append((si, g, g))
        cdict['blue'].append((si, b, b))
        cdict['alpha'].append((si, a, a))

    newcmap = matplotlib.colors.LinearSegmentedColormap(name, cdict)
    plt.register_cmap(cmap=newcmap)

    return newcmap

# use for 0-centered color maps
#===============
def get_colormap_midpoint(Matrix):
#===============
    Min, Max = float(np.nanmin(Matrix)), float(np.nanmax(Matrix))
    if not np.isfinite(Min) or not np.isfinite(Max): #sometimes, all values may be NaNs
        #print "NaNs"
        Midpoint, isNegative = 0.5, False 
    elif Min >= 0 or Max < 0:
        #print "Not biased"
        Midpoint, isNegative = 0.5, False
    else:
        Midpoint, isNegative = np.abs(Min) / np.abs(Max-Min), True
    #print "Midpoint: ", Midpoint
    return Midpoint, isNegative

#==========================================
def plot_ImgBGR(Img, title="", cmap='gray', Block=False):
#==========================================
    RGB = cv2.cvtColor(Img, cv2.COLOR_BGR2RGB)
    plot_Img(RGB, title=title, cmap=cmap, Block=Block)

#==========================================
def plot_Img(Img, title="", cmap='gray', Block=False):
#==========================================

    Size = 1.0/1.2
    fig = plt.figure(1, figsize=(22*Size,11.9*Size))
    fig.clf() # fix memory leak of figure already open

    plt.imshow(Img) #,vmin=0,vmax=255)
    plt.title(title)
    plt.axis('off')

    if len(Img.shape) == 2: # luminance channel
        CMap = pylab.gray()
        plt.colorbar(CMap)
        
    plt.tight_layout()
    plt.show(block=Block)
    plt.pause(0.001) 
  
#==========================================================================================================================
#=================================================  INTERNAL ==== =========================================================
#==========================================================================================================================
  
# EXAMPLE:
# fig = plt.figure(figsize=(22,11.9))
# plt.subplot(3, 3, 1) 
# subplot_Img(Img, "Img")
# plt.show()    
#==========================================
def subplot_Img(Img, cmap='gray', scale=False):
#==========================================

    if scale is True: #let matplotlib decide the dynamic range
        plt.imshow(Img, cmap=pylab.gray())
    else: 
        plt.imshow(Img, cmap=pylab.gray(), vmin=0, vmax=255)
    plt.axis('off')

    
# EXAMPLE:
# fig = plt.figure(figsize=(22,11.9))
# fig.canvas.set_window_title(Sequ.FN)
# plt.subplot(5, 1, 1) 
# plt.subplots_adjust(hspace = 0.5)
# subplot_Bar(TS_POCs, BGS_Ducati.TS_SkipPercentage, "BGS_Ducati.TS_SkipPercentage", (1, Sequ.Fr))
#==========================================
def subplot_Bar(x, y, title, xlimits):
#==========================================
    # we need xlimits, otherwise leading/trailing zero elements are not shown
    plt.bar(x, y)
    plt.xlim(xlimits[0], xlimits[1])
    plt.title(title)

#==========================================
def plot_Warping(Ref, Cur, H, Title=""):  #FigNum
#==========================================

    Fig = plt.figure(figsize=(22,11.9))
    
    Fig.suptitle(Title, fontsize=20)
    
    plt.subplot(1, 4, 1)
    subplot_Img(Ref[:,:,0], "Ref")

    plt.subplot(1, 4, 2)
    subplot_Img(Cur[:,:,0], "Cur")
    
    plt.subplot(1, 4, 3)
    subplot_Img(getLumDiff(Ref, Cur), "LumDiff Ref - Cur", scale=True)
    
    plt.subplot(1, 4, 4)
    subplot_Img(getWrpdLumDiff(Ref, Cur, H), "LumDiff WRPD_Ref - Cur", scale=True)
    
    return Fig

#==========================================
def plot_ImageRow(ImgRow, Title="", SubTitles=[]): #FigNum, 
#==========================================

    Fig = plt.figure()#plt.figure(figsize=(22,11.9))
    
    Fig.suptitle(Title, fontsize=20)
    
    NumImages = len(ImgRow)
    
    for Idx, Img in enumerate(ImgRow):
    
        ax = plt.subplot(1, NumImages, Idx+1) #prepare subplot
        subplot_Img(Img, scale=False)
        SubTitle = SubTitles[Idx] if len(SubTitles) > Idx else ""
        ax.set_title(SubTitle)


    return Fig

#==========================================
# 2D Viewer Widget
#==========================================    
from matplotlib.widgets import Cursor, Button
from matplotlib import cbook
#plt.rcParams['font.size']=8

class ViewerWidget(object):
    def __init__(self,z,x=None, y=None):
        """
        Shows a given array in a 2d-viewer.
        Input: z, an 2d array.
        x,y coordinters are optional.
        """
        if x==None:
            self.x=np.arange(z.shape[0])
        else:
            self.x=x
        if y==None:
            self.y=np.arange(z.shape[1])
        else:
            self.y=y
        self.z=z
        self.fig=plt.figure()
        #Doing some layout with subplots:
        self.fig.subplots_adjust(0.05,0.05,0.98,0.98,0.1)
        self.overview=plt.subplot2grid((8,4),(0,0),rowspan=7,colspan=2)
        self.overview.contourf(x, y, z, 15, cmap=plt.get_cmap('rainbow'))
        #self.overview.pcolormesh(self.x,self.y,self.z)
        #self.overview.autoscale(1,'both',1)
        self.x_subplot=plt.subplot2grid((8,4),(0,2),rowspan=4,colspan=2)
        self.y_subplot=plt.subplot2grid((8,4),(4,2),rowspan=4,colspan=2)

        #Adding widgets, to not be gc'ed, they are put in a list:
        
        cursor=Cursor(self.overview, useblit=True, color='black', linewidth=2 )
        but_ax=plt.subplot2grid((8,4),(7,0),colspan=1)
        reset_button=Button(but_ax,'Reset')
        but_ax2=plt.subplot2grid((8,4),(7,1),colspan=1)
        legend_button=Button(but_ax2,'Legend')
        self._widgets=[cursor,reset_button,legend_button]
        #connect events
        reset_button.on_clicked(self.clear_xy_subplots)
        legend_button.on_clicked(self.show_legend)
        self.fig.canvas.mpl_connect('button_press_event',self.click)
    
    def show_legend(self, event):        
        """Shows legend for the plots"""
        for pl in [self.x_subplot,self.y_subplot]:
            if len(pl.lines)>0:
                pl.legend()
        plt.draw()

    def clear_xy_subplots(self,event):
        """Clears the subplots."""
        for j in [self.overview,self.x_subplot,self.y_subplot]:
            j.lines=[]
            j.legend_ = None
        plt.draw()


    def click(self,event):
        """
        What to do, if a click on the figure happens:
            1. Check which axis
            2. Get data coord's.
            3. Plot resulting data.
            4. Update Figure
        """
        if event.inaxes==self.overview:
            #Get nearest data
            xpos=np.argmin(np.abs(event.xdata-self.x))
            ypos=np.argmin(np.abs(event.ydata-self.y))
            
            #Check which mouse button:
            if event.button==1:
                #Plot it                
                c,=self.y_subplot.plot(self.y, self.z[:,xpos],label=str(self.x[xpos]))
                self.overview.axvline(self.x[xpos],color=c.get_color(),lw=2)

            elif event.button==3:
                #Plot it                
                c,=self.x_subplot.plot(self.x, self.z[ypos,:],label=str(self.y[ypos]))
                self.overview.axhline(self.y[ypos],color=c.get_color(),lw=2)

        if event.inaxes==self.y_subplot:
            ypos=np.argmin(np.abs(event.xdata-self.y))
            c=self.x_subplot.plot(self.x, self.z[ypos,:],label=str(self.y[ypos]))
            self.overview.axhline(self.y[ypos],color=c.get_color(),lw=2)

        if event.inaxes==self.x_subplot:
            xpos=np.argmin(np.abs(event.xdata-self.x))
            c,=self.y_subplot.plot(self.y, self.z[:,xpos],label=str(self.x[xpos]))
            self.overview.axvline(self.x[xpos],color=c.get_color(),lw=2)
        #Show it
        plt.draw()
        
#==========================================================================================================================
#===============================================  PLOT STAND ALONE EXAMPLES ===============================================
#==========================================================================================================================

#==========================================
def example_Pie():
#==========================================
    # plotting: counter-clockwise
    labels = 'Intra', 'Inter', 'Merge', 'Skip'
    sizes = [5, 15, 30, 50]
    #colors = (IntraColor, InterColor, MergeColor, SkipColor)
    #colors = [np.asarray(IntraColor) / 255.0, 'gold', 'lightskyblue', 'lightcoral']
    colors = scaleColors( (IntraColor, InterColor, MergeColor, SkipColor) )
    explode = (0, 0.1, 0, 0) # only "explode" the 2nd slice 
    
    plt.pie(sizes, explode=explode, labels=labels, colors=colors,
            autopct='%1.1f%%', shadow=True, startangle=0) 
    plt.axis('equal') # prevent ellipse
    plt.tight_layout()
    FN = CoreTmpDir + "/example_Pie_1.jpg"
    plt.savefig(FN)
    
    copy_to_PlotExamples(FN)  
    plt.show()

#==========================================
def example_Twin_Axis():
#==========================================
    #EXAMPLE_STANDALONE

    x = np.arange(0.01, 10.0, 0.01)
    y1 = np.exp(x)
    y2 = np.sin(2*np.pi*x)
    
    #BASIC
    fig, ax1 = plt.subplots()
    ax1.set_xlabel("xlabel")
    ax1.set_title("title")
    
    #LEFT AXIS
    color = 'b'
    ax1.plot(x, y1, '-o', color=color)
    ax1.set_ylabel('ylabel_1', color=color)
    for tks in ax1.get_yticklabels():
        tks.set_color(color)
       
    #RIGHT AXIS 
    ax2 = ax1.twinx()
    color = 'g'
    ax2.set_ylabel('ylabel_2', color=color)
    ax2.plot(x, y2, '-o', color=color)  
    for tks in ax2.get_yticklabels():
        tks.set_color(color)    

    plt.tight_layout(pad=1, w_pad=1, h_pad=1)
    
    FN = TmpDir + "/example_Twin_Axis.jpg"
    plt.savefig(FN)
    copy_to_PlotExamples(FN)  
    #plt.show()

#==========================================
def example_StackPlot(): #StackPlot
#==========================================   
    def fnx():
        return np.random.randint(5, 50, 10)

    x = np.arange(10)
    print "x: ", x

    y1, y2, y3 = fnx(), fnx(), fnx()
    print "y1: ", y1

    y = np.row_stack((y1, y2, y3))
    y = y.astype(np.float) #just testing
    print "y: ", y
    
    
    
    fig, axarr = plt.subplots(1, 2, figsize=(A4q_x*1,A4q_y*1)) #subplots-pkg
    axarr[0].stackplot(x, y)    
    axarr[1].stackplot(x, y1, y2, y3)
    
    FN = TmpDir + "/example_StackPlot.jpg"
    plt.savefig(FN)
    copy_to_PlotExamples(FN)  
    #plt.show()
    
# DO NOT USE!! SEE PANDAS.PY
#==========================================
def example_Bar_Plot_withMatplotlib(): #BarPlot
#==========================================
    
    # Traditional bar cannot ne recommended because:
    # - only one tuple can be passed to the bar call
    # - the results need to be resorted, otherwise one needs to resort the loop statements (bad in case of CPU load)
    # - the width calculation is just plain horror 
    # - color cycling adds to unreadable code
    
    # each tuple represent bars of the same color (-> Values per POC)
    # each ax.bar call adds one tuple (-> POCs)
    
#     bar(self, left, height, width=0.8, bottom=None, **kwargs):
#     Make a bar plot with rectangles bounded by:
# 
#       `left`, `left` + `width`, `bottom`, `bottom` + `height`
#             (left, right, bottom and top edges)
# 
#     
#     left : sequence of scalars
#     the x coordinates of the left sides of the bars
# 
#     height : sequence of scalars
#     the heights of the bars
# 
#     width : scalar or array-like, optional, default: 0.8
#     the width(s) of the bars
    
    ind = np.arange(5) # number of groups
    width = 0.35       # the width of the bars
    
    menMeans = (20, 35, 30, 35, 27)
    womenMeans = (25, 32, 34, 20, 25)
    
    fig, ax = plt.subplots()
    rects1 = ax.bar(ind, menMeans, width, color='r')
    rects2 = ax.bar(ind + width, womenMeans, width, color='y')
    
    plt.show()

    #OLD EXAMPLE
    if 0:
        
        # Note: not intuitive, use Pandas instead!
    
        fig = plt.figure()
        # Example data
        Performer = ('Performer0', 'Performer1', 'Performer2', 'Performer3', 'Performer4')
        x_pos = np.arange(len(Performer))
        #how each performer performs in catgory X, one entry for each performer
        Cat0_Performance = 3 + 10 * np.random.rand(len(Performer))  
        Cat1_Performance = 3 + 10 * np.random.rand(len(Performer))
        Cat2_Performance = 3 + 10 * np.random.rand(len(Performer))
        
        Cat0_Label, Cat0_Color = "Translations", 'r'
        Cat1_Label, Cat1_Color = "Rotations", 'g'
        Cat2_Label, Cat2_Color = "Zooms", 'b'
        
        width = 0.15
        #think of only the leftmost bar-collection (first "Group")
        #all other groups 
        plt.bar(x_pos+0*width, Cat0_Performance, width, color=Cat0_Color)
        plt.bar(x_pos+1*width, Cat1_Performance, width, color=Cat1_Color)  
        plt.bar(x_pos+2*width, Cat2_Performance, width, color=Cat2_Color,)
        plt.xticks(x_pos+1.5*width, Performer)
        #plt.title('How fast do you want to go today?')
        
        # DRAWING A HORIZONTAL LINE
        plt.axhline(y=Cat0_Performance[0], linewidth=2, xmin=0.0, linestyle="--", color=Cat0_Color)
        plt.axhline(y=Cat1_Performance[0], linewidth=2, xmin=0.01*3.5, linestyle="--", color=Cat1_Color)
        plt.axhline(y=Cat2_Performance[0], linewidth=2, xmin=0.02*3.5, linestyle="--", color=Cat2_Color)
        
        #pyplot.hlines(Cat0_Performance[0], 0, x_arr, color='red')
        plt.legend((Cat0_Label, Cat1_Label, Cat2_Label))
        plt.show()

#==========================================
def fast_Plot_EXAMPLE():
#==========================================
    aimage = cv2.imread(res("~/Desktop/Jetson_TK1/HARP/Various/Resources/HarrisonOnTheMoon2.jpg"))
    aimage = aimage[...,0] * 34
    aimage = aimage.astype(np.float32)
    Size = 1.0/1.2
    fig = plt.figure(1, figsize=(22*Size,11.9*Size))
    fig.clf() # fix memory leak of figure already open
    plt.imshow(aimage) #,vmin=0,vmax=255)    
    plt.colorbar()
    plt.show(block=True)

#==========================================
def plot_Sankey_EXAMPLE():
#==========================================
    from matplotlib.sankey import Sankey
    s = 1
    Sankey(flows=[100/s, -15/s, -5/s, -80/s],
           labels=["", "","",""],
           #labels=['Input', 'Rotation', 'Zoom','Translational/\nStill'],
           orientations=[ 0, 1, 1, 0],
           scale=0.01,
           format='%.0f', unit='%', 
           #offset=20,
           pathlengths = [2, 1-0.5, 1.2-0.5, 1],
           head_angle=90,
           lw=2.0
            ).finish()
    plt.title("The default settings produce a diagram like this.")
    plt.savefig("tmp/SankeyArrow.svg" )
    plt.show()
    
#==========================================
def plot_Curve_EXAMPLE(): #multiple curves
#==========================================
    x1 = np.random.rand(100)
    x2 = np.random.rand(100)

    fig = plt.figure(1, figsize=(3.5, 6), dpi=80)
    ax = fig.gca()
    ax.plot(x1, label="Random value x1", c='g')
    ax.plot(x2, label="Random value x2", c='r')
    ax.set_ylabel('Amplitude')
    plt.show(block=True)
    

    
#==========================================
def example_Scatter_1():
#==========================================
    pass

#                     FailIdx = Tmp["FailIdx"]
#                     SuccIdx = Tmp["SuccIdx"]
#                     ScatPlot.scatter(Tmp["MAD_WarpedCorners"][FailIdx],Tmp["ABSDIFF_MSE_WarpedMap"][FailIdx], "Failing", axidx=0, color="red", LogScale=True)   
#                     ScatPlot.scatter(Tmp["MAD_WarpedCorners"][SuccIdx],Tmp["ABSDIFF_MSE_WarpedMap"][SuccIdx], "Success", axidx=0, color="green", LogScale=True) 
#                     ScatPlot.scatter(Tmp["MAD_WarpedCorners"][FailIdx],Tmp["ABSDIFF_MSE_WarpedMap"][FailIdx], "Failing", axidx=1, color="red", LogScale=False)   
#                     ScatPlot.scatter(Tmp["MAD_WarpedCorners"][SuccIdx],Tmp["ABSDIFF_MSE_WarpedMap"][SuccIdx], "Success", axidx=1, color="green", LogScale=False) 

#==========================================
def example_Scatter_2(x = None, y = None, ):
#==========================================
    #import matplotlib.pyplot as plt
    #import numpy as np
    
    # HOWTO:
    # A scatter call always uses ONE COLOR ONLY
    # So: each set of points has its own color
    # Call scatter multiple times, with presorted lists of individual colors
    # One set of points: provide x and y coords separately
    
    
    ## the data
    if x == None:
        N=1000
        x = np.random.randn(N)
        y = np.random.randn(N)
        z = np.random.randint(50,200)
    
    fig = plt.figure()
    ax1 = fig.add_subplot(121)
    
    ## left panel
    ax1.scatter(x,y,color='blue',s=5,edgecolor='none')
    ax1.set_aspect(1./ax1.get_data_ratio()) # make axes square
    
    ## right panel
    ax2 = fig.add_subplot(122)
    props = dict(alpha=0.5, edgecolors='none' )
    
    handles = []
    colors = ['blue', 'green', 'magenta', 'cyan']
    for color in colors:
        x = np.random.randn(N)
        y = np.random.randn(N)
        s = np.random.randint(50,200)
        handles.append(ax2.scatter(x, y, c=color, s=s, **props))
    
    ax2.set_ylim([-5,11])
    ax2.set_xlim([-5,11])
    
    ax2.legend(handles, colors)
    ax2.grid(True)
    ax2.set_aspect(1./ax2.get_data_ratio())
    plt.show()

#==========================================================================================================================
#================================================== COLLECTED PLOTS =======================================================
#==========================================================================================================================


# NOTE: also look in HDT dir for HDT plotting examples 

#==========================================
def plotFigure_MotionClassifierPerf(ClassifierNames, CategoryNames, CategoryPerformance, EpsFN):
#==========================================
    
    global FigNum
    fig = plt.figure(FigNum, figsize=(6.2, 5.5), dpi=80)
    FigNum = FigNum + 1
    
#     plt.rc('text', usetex=True)
#     plt.rc('font', family='serif')
#     plt.rcParams.update({'font.size': 12})
    plt.subplots_adjust(left=0.11, right=0.97, top=0.94, bottom=0.11)
    
    width = 0.25
    #offs = len(CategoryNames)/3*width
    
    # Example data
    Performer = ClassifierNames
    x_pos = np.arange(len(Performer))
    #how each performer performs in catgory X, one entry for each performer
    Cat0_Performance = CategoryPerformance[0]  
    Cat1_Performance = CategoryPerformance[1] 
    Cat2_Performance = CategoryPerformance[2] 
    
    Cat0_Label, Cat0_Color = CategoryNames[0], 'r'
    Cat1_Label, Cat1_Color = CategoryNames[1], 'g'
    Cat2_Label, Cat2_Color = CategoryNames[2], 'b'
    
    #think of only the leftmost bar-collection (first "Group")
    #all other groups 
    plt.bar(x_pos+0*width, Cat0_Performance, width, color=Cat0_Color)
    plt.bar(x_pos+1*width, Cat1_Performance, width, color=Cat1_Color)
    plt.bar(x_pos+2*width, Cat2_Performance, width, color=Cat2_Color)
    plt.xticks(x_pos+1.5*width, Performer)
    #plt.title('How fast do you want to go today?')
    ax = fig.gca() 
    ax.set_ylabel('Number of frames')

    plt.legend((Cat0_Label, Cat1_Label, Cat2_Label), loc='center left')
    
    plt.axhline(y=Cat0_Performance[0], linewidth=2, xmin=0.0, linestyle="--", color=Cat0_Color)
    plt.axhline(y=Cat1_Performance[0], linewidth=2, xmin=0.01*4.0, linestyle="--", color=Cat1_Color)

    plt.axhline(y=Cat2_Performance[0], linewidth=2, xmin=0.02*4.3, linestyle="--", color=Cat2_Color)
    
    #-------------------------------------
    # EXPORT FIGURE 1
    #-------------------------------------    
    (FN, BN, Dir, DIRBN, AbsPath) = splitPathFN(EpsFN)
    #plt.savefig("%s1.png" % DIRBN) #not used
    plt.savefig("%s.svg" % DIRBN)
    #os.system("inkscape --export-eps=%s1.eps %s.svg" % (DIRBN, DIRBN))
    
    #plt.ylim(0, 600)
    #ax.set_ylim(-20, 20)
    
#==========================================
def plotFigure_NaturalSequences_ZoomRotValues(FE):
#==========================================
    #FS = 12 #font size
    fig = plt.figure(1, figsize=(3.5, 6), dpi=80)
    ax = fig.gca()
    
    ax = fig.gca() 
#     plt.rc('text', usetex=True)
#     plt.rc('font', family='serif')
#     plt.rcParams.update({'font.size': FS})
    plt.subplots_adjust(left=0.18, right=0.78, top=0.95, bottom=0.10)

    
    x = np.arange(1,FE.shape[0]/2)
    ax.plot(x, FE[200:400,ROTVAL_GT], label="GT angle", c='g')
    ax.plot(x, FE[200:400,ROTVAL], label="Estim. angle", c='r')
    ax.set_xlabel('Frame index')
    ax.set_ylim(-0.2, 0.30)
    ax.set_ylabel('Rotation angle (in deg)', color='g', fontsize=FS)
    for tl in ax.get_yticklabels():
        tl.set_color('g')
    ax.legend(loc='upper left', title=r"Sequ. Spincalendar", fontsize=12)
    #legend = ax.legend( shadow=False, numpoints=1, loc='upper left') #loc='upper center',
    #legend.get_frame().set_facecolor('0.90')    
    
    #fig = plt.figure(2)
    ax2 = ax.twinx()
    #ax2 = fig.gca()
    ax2.plot(x, FE[:198,RATIOVAL_GT], label="GT zoom", c='b')
    ax2.plot(x, FE[:198,RATIOVAL], label="Estim. zoom", c='y')
    ax2.set_ylim(0.99, 1.01)
    ax2.set_ylabel('Zoom (scale factor)', color='b', fontsize=FS)
    for tl in ax2.get_yticklabels():
        tl.set_color('b')
    ax2.legend(loc='lower right', title=r"Sequ. Bigships", fontsize=FS)
    #plt.tight_layout()
    (FN, BN, Dir, DIRBN, AbsPath) = splitPathFN('tmp/Figure_NaturalSequences_RotZoomValues.eps')
    plt.savefig("%s.svg" % DIRBN)   
    
    
#==========================================
def plotFigure_NaturalSequences_MoClaPerformance(ClassifierNames, CategoryNames, CategoryPerformance, EpsFN):
#==========================================
    
    global FigNum
    fig = plt.figure(FigNum, figsize=(3.5, 6), dpi=80)
    FigNum = FigNum + 1
    
    #FS = 12 #font size
    
#     plt.rc('text', usetex=True)
#     plt.rc('font', family='serif')
#     plt.rcParams.update({'font.size': 12})
    plt.subplots_adjust(left=0.18, right=0.97, top=0.94, bottom=0.11)
    
    width = 0.25
    #offs = len(CategoryNames)/3*width
    
    # Example data
    Performer = ClassifierNames
    x_pos = np.arange(len(Performer))
    ax = fig.gca() 
    ax.set_ylabel('Number of frames')
    #how each performer performs in catgory X, one entry for each performer
    Cat0_Performance = CategoryPerformance[0]  
    Cat1_Performance = CategoryPerformance[1] 
    Cat2_Performance = CategoryPerformance[2] 
    
    Cat0_Label, Cat0_Color = CategoryNames[0], 'r'
    Cat1_Label, Cat1_Color = CategoryNames[1], 'g'
    Cat2_Label, Cat2_Color = CategoryNames[2], 'b'
    
    #think of only the leftmost bar-collection (first "Group")
    #all other groups 
    plt.bar(x_pos+0*width, Cat0_Performance, width, color=Cat0_Color)
    plt.bar(x_pos+1*width, Cat1_Performance, width, color=Cat1_Color)
    plt.bar(x_pos+2*width, Cat2_Performance, width, color=Cat2_Color)
    plt.xticks(x_pos+2*width, Performer)
    #plt.title('How fast do you want to go today?')
    

    plt.legend((Cat0_Label, Cat1_Label, Cat2_Label), loc='upper left', fontsize=12)
    
    plt.axhline(y=Cat0_Performance[0], linewidth=2, xmin=0.0, linestyle="--", color=Cat0_Color)
    plt.axhline(y=Cat1_Performance[0], linewidth=2, xmin=0.02*4.7, linestyle="--", color=Cat1_Color)

    plt.axhline(y=Cat2_Performance[0], linewidth=2, xmin=0.04*4.5, linestyle="--", color=Cat2_Color)
    plt.ylim(0, 300)
    #-------------------------------------
    # EXPORT FIGURE 1
    #-------------------------------------    
    (FN, BN, Dir, DIRBN, AbsPath) = splitPathFN(EpsFN)
    #plt.savefig("%s1.png" % DIRBN) #not used
    plt.savefig("%s.svg" % DIRBN)
    #os.system("inkscape --export-eps=%s1.eps %s.svg" % (DIRBN, DIRBN))
    
    #plt.ylim(0, 600)
    #ax.set_ylim(-20, 20)

#==========================================
def plotFigure_ClassifierSIFTvsSADvsSSD(ClassifierNames, CategoryNames, CategoryPerformance, EpsFN):
#==========================================
    
    global FigNum
    fig = plt.figure(FigNum, figsize=(7, 4), dpi=80)
    FigNum = FigNum + 1
    
#     plt.rc('text', usetex=True)
#     plt.rc('font', family='serif')
#     plt.rcParams.update({'font.size': 12})
    plt.subplots_adjust(left=0.13, right=0.97, top=1.0, bottom=0.13)
    
    width = 0.35
    offs = len(CategoryNames)/2*width
    
    # Example data
    Performer = ClassifierNames
    x_pos = np.arange(len(Performer))
    #how each performer performs in catgory X, one entry for each performer
    Cat0_Performance = CategoryPerformance[0]  
    Cat1_Performance = CategoryPerformance[1] 

    Cat0_Label, Cat0_Color = CategoryNames[0], 'g'
    Cat1_Label, Cat1_Color = CategoryNames[1], 'b'

    #think of only the leftmost bar-collection (first "Group")
    plt.bar(x_pos+0*width, Cat0_Performance, width, color=Cat0_Color)
    plt.bar(x_pos+1*width, Cat1_Performance, width, color=Cat1_Color)

    plt.xticks(x_pos+offs, Performer)


    plt.legend((Cat0_Label, Cat1_Label), loc='upper left', title="Detected motion:")

    plt.axhline(y=Cat0_Performance[0], linewidth=2, xmin=0.0, linestyle="--", color=Cat0_Color)
    plt.axhline(y=Cat1_Performance[0], linewidth=2, xmin=0.01*9, linestyle="--", color=Cat1_Color)

    #-------------------------------------
    # EXPORT FIGURE 1
    #-------------------------------------    
    (FN, BN, Dir, DIRBN, AbsPath) = splitPathFN(EpsFN)
    #plt.savefig("%s1.png" % DIRBN) #not used
    plt.savefig("%s.svg" % DIRBN)
    #os.system("inkscape --export-eps=%s1.eps %s1.svg" % (DIRBN, DIRBN))
    
    #plt.ylim(0, 600)
    #ax.set_ylim(-20, 20)
    
#==========================================
def plot3D_Figure_SKIPvsINTERvsXX(Version, AxStr, Data, LA, EpsFN):
#==========================================

    global FigNum
    fig = plt.figure(FigNum, figsize=(6, 4), dpi=80)
    FigNum = FigNum + 1

    #-------------------------------------
    # PLOT BEAUTY
    #-------------------------------------
#     plt.rc('text', usetex=True)
#     plt.rc('font', family='serif')
#     plt.rcParams.update({'font.size': 12})
    
    FigTitle = ""    #%s VS. %s VS. %s" % (AxStr[0], AxStr[1], AxStr[2])
    ax = fig.add_subplot(111, projection='3d', title=FigTitle)
        
    #-------------------------------------
    # DATA
    #-------------------------------------
    MRK_SIZE = 9
    ROT_LEGEND = "Rotation"
    ROT_COL = 'g'
    ROT_MRK = '^'
    ax.plot(Data[0][np.where(LA == ROT)], 
               Data[1][np.where(LA == ROT)], 
               Data[2][np.where(LA == ROT)], 
               c=ROT_COL, marker=ROT_MRK, label=ROT_LEGEND, markersize=MRK_SIZE, linestyle='None') 
    
    ZOOM_LEGEND = "Zoom"
    ZOOM_COL = 'b'
    ZOOM_MRK = '^'
    ax.plot(Data[0][np.where(LA == ZOOM)], 
               Data[1][np.where(LA == ZOOM)], 
               Data[2][np.where(LA == ZOOM)], 
               c=ZOOM_COL, marker=ZOOM_MRK, label=ZOOM_LEGEND, markersize=MRK_SIZE, linestyle='None') 
    
    TRST_LEGEND = "Translation"
    TRST_COL = 'r'
    TRST_MRK = '^'
    ax.plot(Data[0][np.where(LA == TRST)], 
               Data[1][np.where(LA == TRST)], 
               Data[2][np.where(LA == TRST)], 
               c=TRST_COL, marker=TRST_MRK, label=TRST_LEGEND, markersize=MRK_SIZE, linestyle='None') 
    
    ax.set_xlabel(AxStr[0])
    ax.set_ylabel(AxStr[1])
    ax.set_zlabel(AxStr[2])
    
    #-------------------------------------
    # PLOT BEAUTY
    #-------------------------------------
#     legend = ax.legend( shadow=False, numpoints=1) #loc='upper center',
#     legend.get_frame().set_facecolor('0.90')
    def my_on_click(event):
        print(ax.elev, ax.azim)
    
    #plt.zticks([0, 1, 2])
    if Version=="Version1":
        plt.subplots_adjust(left=0.00, right=0.96, top=0.99, bottom=0.04)
        LegendLoc = 'upper right'
    
    if Version=="Version2":
        ax.set_zticklabels(["0", "", "1", "", "2"]) 
        plt.subplots_adjust(left=0.03, right=0.96, top=0.99, bottom=0.04)
        LegendLoc = 'upper right'
        
    #-------------------------------------
    # EXPORT FIGURE 1
    #-------------------------------------
    legend = ax.legend( shadow=False, numpoints=1, loc=LegendLoc) #loc='upper center',
    legend.get_frame().set_facecolor('0.90')
    fig.canvas.mpl_connect('button_release_event', my_on_click)
    #ax.view_init(elev=30.4, azim=41.1)
    ax.view_init(elev=28.1, azim=-15.5)
    
    (FN, BN, Dir, DIRBN, AbsPath) = splitPathFN(EpsFN)
    #plt.savefig("%s1.png" % DIRBN) #not used
    plt.savefig("%s_left.svg" % DIRBN)
    #os.system("inkscape --export-eps=%s1.eps %s1.svg" % (DIRBN, DIRBN))

#     #-------------------------------------
#     # EXPORT FIGURE 2
#     #-------------------------------------    
#     #legend = ax.legend( shadow=False, numpoints=1, loc='upper right') #loc='upper center',
#     #legend.get_frame().set_facecolor('0.90')
#     fig.canvas.mpl_connect('button_release_event', my_on_click)
#     ax.view_init(elev=30.4, azim=-36.0)
#     
#     (FN, BN, Dir, DIRBN, AbsPath) = splitPathFN(EpsFN)
#     #plt.savefig("%s2.png" % DIRBN) #not used
#     plt.savefig("%s_right.svg" % DIRBN)
#     #os.system("inkscape --export-eps=%s2.eps %s2.svg" % (DIRBN, DIRBN))
    
#==========================================
def plot2D_Figure_SADvsSSD(AxStr, Data, LA, EpsFN):
#==========================================

    global FigNum
    fig = plt.figure(FigNum, figsize=(6, 4), dpi=80)
    FigNum = FigNum + 1

    #-------------------------------------
    # PLOT BEAUTY
    #-------------------------------------
#     plt.rc('text', usetex=True)
#     plt.rc('font', family='serif')
#     plt.rcParams.update({'font.size': 12})
    
    FigTitle = ""    #%s VS. %s VS. %s" % (AxStr[0], AxStr[1], AxStr[2])
    ax = fig.add_subplot(111, title=FigTitle)
    #plt.subplots_adjust(left=0.01, right=0.99, top=0.99, bottom=0.01)
    plt.subplots_adjust(left=0.13, right=0.97, top=0.9, bottom=0.1)
       
        
    #-------------------------------------
    # DATA
    #-------------------------------------
    MRK_SIZE = 8
    

    
    ROT_LEGEND = "Rotation"
    ROT_COL = 'g'
    ROT_MRK = '^'
    ax.plot(Data[0][np.where(LA == ROT)], 
               Data[1][np.where(LA == ROT)], 
               c=ROT_COL, marker=ROT_MRK, label=ROT_LEGEND, markersize=MRK_SIZE, linestyle='None') 
    
    ZOOM_LEGEND = "Zoom"
    ZOOM_COL = 'b'
    ZOOM_MRK = '^'
    ax.plot(Data[0][np.where(LA == ZOOM)], 
               Data[1][np.where(LA == ZOOM)], 
               c=ZOOM_COL, marker=ZOOM_MRK, label=ZOOM_LEGEND, markersize=MRK_SIZE, linestyle='None') 
    
    TRST_LEGEND = "Translation"
    TRST_COL = 'r'
    TRST_MRK = '^'
    ax.plot(Data[0][np.where(LA == TRST)], 
               Data[1][np.where(LA == TRST)], 
               c=TRST_COL, marker=TRST_MRK, label=TRST_LEGEND, markersize=MRK_SIZE, linestyle='None') 

    #plt.subplots_adjust(left=0.13, right=0.97, top=0.9, bottom=0.1)

    
    #-------------------------------------
    # PLOT BEAUTY
    #-------------------------------------
    legend = ax.legend( shadow=False, numpoints=1, loc='upper left') #loc='upper center',
    #legend.get_frame().set_facecolor('0.90')

    ax.annotate("Distinction of motion\n difficult in this\n value range", xy=(19+1,1230+500+50),
            horizontalalignment='center', verticalalignment='center')

    from matplotlib.patches import Ellipse
    el = Ellipse((20,1230), 22, 2100, facecolor='r', alpha=0.5)
    ax.add_artist(el)
    el.set_clip_box(ax.bbox)
    
    ax.set_xlabel(AxStr[0])
    ax.set_ylabel(AxStr[1])
    
#     ax.set_xlim(-20, 20)
#     ax.set_ylim(-20, 20)

    
    #-------------------------------------
    # EXPORT FIGURE 1
    #-------------------------------------    
    (FN, BN, Dir, DIRBN, AbsPath) = splitPathFN(EpsFN)
    #plt.savefig("%s1.png" % DIRBN) #not used
    plt.savefig("%s.svg" % DIRBN)
    #os.system("inkscape --export-eps=%s.eps %s.svg" % (DIRBN, DIRBN))
#-

if __name__ == '__main__':
    pass
    
    #example_Pie()
    #example_Bar_Plot_withPandas()
    #example_Twin_Axis()