#!/usr/bin/env python
# coding: utf8
# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)

from Plot import *
from OpenCV import *

#==========================================
# SWISS KNIFE: GALLERY
#==========================================
if 0:
    Name = "10x2_Overview"  
    rows = 5
    Gallery  = PlotGallery(rows, 2, A4_x, A4_y / (5.0/rows))
    #BGS = "Diff-BGS"
    for POCIdx in range(4,9):
        row = POCIdx - 4
        
        Gallery.subplot(row, 0, readImg(RES[BGS]["FN_Cur"][POCIdx]),  "POC%d" % POCIdx )
        Gallery.subplot(row, 1, readImg(RES[BGS]["FN_Est_Mask"][POCIdx]),  "%s: Estimated Mask" % BGS )
        
        Colors = BRBA_GREEN, BLACK
        Labels = "Map Area", "Static Areas"
        create_Epic_Legend(Gallery.axarr[row, 1], Colors, Labels, bbox=(1.0, 1.0))             
    
    FN = combine(PostDir + "/", Name, SequFN, BGS, ".jpg")
    save_And_Close(Gallery.fig, FN, Category = Name)                         
    del Gallery # IF NOT DONE, FOLLOWING GALLERIES WILL BE EMPTY. WH00T? BUGFIX REQUIRED!

#==========================================
# INCHES INFORMATION
#==========================================

#A4q_y = 8.2  #letter size in inches
#A4q_x = A4q_y * 1.41

CMap_RedGreen = getCustomCMap_RedToGreen()
CMap_Gray     = matplotlib.cm.get_cmap("Greys_r") #inverted
CMap_RedBlue  = matplotlib.cm.get_cmap("seismic")


# USE:
#     Gallery  = PlotGallery(2, 3, A4_x, A4_y)
#     Gallery.subplot(0, 0, VisWarping,  "Ref & Cur Frame")
#     plt.tight_layout()
#     save_And_Close(Gallery.fig, FN, Category = "REF Idcs", Args = (POC,))
#     del Gallery.fig, Gallery.axarr, Gallery
#|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
class PlotGallery(object):
#|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
    #==========================================
    def __init__(self, NumRows, NumCols, DimX=A3q_x, DimY=A3q_y, DPI=300):
    #==========================================
        super(PlotGallery, self).__init__()
        # CHANGE HERE -------------------------------------------------
        # -------------------------------------------------------------
        #DPI = 200
        zoom = 1
        self.rows, self.cols = NumRows, NumCols
        #creates an matplotlib.axes.AxesSubplot objects
        self.fig, self.axarr = plt.subplots(self.rows, self.cols, figsize=(DimX*zoom,DimY*zoom), dpi=DPI)
        self.fig.canvas.mpl_connect('button_press_event',self.onClick)
        
        #make sure empty plots are not shown
        for r in range(self.rows):
            for c in range(self.cols):
                #self.axarr[r, c].axis('off') 
                self.fig.axes[r * self.cols + c].axis('off') 
        
        self.Data = [[0] * self.cols for i in range(self.rows)] #keeping the data for lookup

#         make_INI_Global()
#         self.SelPos  = INI["Visualization"]["SelPos"]
#         self.SelSize = INI["Visualization"]["SelSize"]
#         if self.SelPos != "":
#             self.SelPos  =  [int(i) for i in self.SelPos  ] 
#             self.SelSize =  [int(i) for i in self.SelSize ] 
#         else:
#             self.SelPos = None
#             self.SelSize = None

    #==========================================
    def getax(self, r, c):
    #==========================================
        o = self.cols
        return self.fig.axes[r*o+c] # ROC'Z!

    #==========================================
    def subplot(self, r, c, Matrix, Label, cmap=CMap_Gray, isImage=False):
    #==========================================
        #-------------------------------------
        # COMMENT
        #-------------------------------------
        #self.axarr[0, 0].set_title('Axis [0,0]')
        #plt.show()        

#         if self.SelPos != None:
#             idcs = self.selectArea(self.SelPos, self.SelSize)
#             Matrix = Matrix[idcs]

        if "_" in Label:
            Label = Label.replace("_", r'\_')
        
#         if  self.rows == 1
#         
#         else:
        
        o = self.cols
        ax = self.fig.axes[r*o+c]
        #ax = self.axarr[r, c] # getting required AxesSubplot object
        ax.axis('off') #"activate" empty plot; PROBLEM: will put a border around each image!!
        ax.set_title(Label, size='large', weight="bold")
        
        #disable pixel position labels
        ax.set_xticklabels([])
        ax.set_yticklabels([])
        

        
        
        if len(Matrix.shape) == 3 or isImage == True: 
            if len(Matrix.shape) == 3: #probably an OpenCV BGR image!
                Matrix = cv2.cvtColor(Matrix, cv2.COLOR_BGR2RGB)
            #ax.get_xaxis().set_visible(False)
            #ax.get_yaxis().set_visible(False)
            axax = ax.imshow(Matrix, cmap=cmap)
            
            self.Data[r][c] = Matrix #keep data for lookup
        else:
            #Min, Max = Matrix.min(), Matrix.max()
            #AbsMaximum = get_absolute_Maximum(Matrix)
            #plt.imshow(Matrix, cmap=cmap, vmin=-AbsMaximum, vmax=AbsMaximum)
                        
            self.set_Grid(r,c,64)
            axax = ax.imshow(Matrix, cmap=cmap)
            #colorbar_ax = fig.add_axes([0.7, 0.1, 0.05, 0.8]) #does not work
            #fig.colorbar(axax, cax=colorbar_ax)
            from mpl_toolkits.axes_grid1 import make_axes_locatable
            divider = make_axes_locatable(ax)
            cax = divider.append_axes("right", size="5%", pad=0.05)
            plt.colorbar(axax, cax=cax)
            
            
            self.Data[r][c] = Matrix #keep data for lookup
                        
            #ax.set_xlim([440,640])
            #ax.set_ylim([80, 0])
            #fig.colorbar(axax)
 
    #==========================================        
    def onClick(self,event):
    #==========================================
        #for ax in self.axarr.ravel(): #find out which axis we just clicked
    
        for r in range(0, self.rows):
            for c in range(0, self.cols):
                ax = self.axarr[r, c]
                if event.inaxes==ax:
                    xpos, ypos = int(event.xdata), int(event.ydata)
                    print "Plot %0.2f,%0.2f, " % (r, c)
                    print "Data Shape: ", self.Data[r][c].shape
                    print "Position x/y: %0.2f/%0.2f, " % (xpos, ypos)
                    print "Value: ", self.Data[r][c][ypos, xpos]
                    print ""

    #==========================================        
    def selectArea(self, Pos, Size):
    #==========================================
        x1, x2 = Pos[0], Pos[0] + Size[0]
        y1, y2 = Pos[1], Pos[1] + Size[1]  
        return np.index_exp[y1:y2, x1:x2]
            
    #==========================================
    def set_Labels_atRow(self, Row, Labels): 
    #==========================================       
        for ax, Label in zip(self.axarr[Row,:], Labels):
            ax.set_title(Label, size='large', weight="bold") #, rotation='horizontal'
    
    #==========================================
    def set_Labels_atCol(self, Col, Labels): 
    #==========================================       
        for ax, Label in zip(self.axarr[:,Col], Labels):
            set_ax_DisableTicks_2(ax)
            ax.set_ylabel(Label, size='large', weight="bold", rotation='vertical')
            #DO NOT CALL THIS HERE self.fig.canvas.draw()

        #do not call this guy with short pauses in between! Seems to have reentrancy problems
        self.fig.canvas.draw()

    #Plot_FINAL_Buffers.draw_Border(1,0, 32)
    #==========================================
    def draw_Border(self, r, c, Margin): 
    #==========================================   
        ax =  self.axarr[r, c]
        autoAxis = ax.axis()
        rec = pylab.Rectangle((autoAxis[0]-Margin,autoAxis[2]-Margin),(autoAxis[1]-autoAxis[0])+Margin,(autoAxis[3]-autoAxis[2])+Margin,fill=False,lw=2)
        rec = ax.add_patch(rec)
        rec.set_clip_on(False)
        
    #==========================================
    def set_Colormap(self, r, c, CMap): 
    #==========================================       
        ax = self.axarr[r, c]
        axax = get_ImageObject_FromAxis(ax)
        axax.set_cmap(CMap)
        
    #==========================================
    def set_Aspect_Ratio(self, r, c, Ratio="equal"): 
    #==========================================  
        ax = self.axarr[r, c]
        ax.set_aspect(Ratio)
     
    #==========================================        
    def set_Limits(self, r, c, Min, Max):
    #==========================================
        ax = self.axarr[r, c]
        
        ImageObject = get_ax_ImageObject(ax)
        ImageObject.set_clim([Min,Max])
        
    #==========================================
    def set_Grid(self, r, c, GridSize):
    #==========================================
        ax = self.axarr[r, c]
        ax.xaxis.set_ticks(np.arange(0, 1920, GridSize))
        ax.yaxis.set_ticks(np.arange(0, 1920, GridSize))
        ax.grid(True, which='major', linestyle='-', color='green')
         
    # dont use this, will disable the labels as well:
#         self.axarr[r, c].get_xaxis().set_visible(isVisible) 
#         self.axarr[r, c].get_yaxis().set_visible(isVisible)     
            
    #==========================================
    def set_suptitle(self, Suptitle, fontsize=20):
    #==========================================
        self.fig.suptitle(Suptitle, fontsize=fontsize, weight="heavy" )
        # note: use this to make space: plt.subplots_adjust(top=0.90)

    #==========================================
    def show(self):
    #==========================================
        #plt.tight_layout(pad=0.8, w_pad=2.0, h_pad=2.0)
        #plt.subplots_adjust(top=0.90) # make space for multiline suptitle
        self.fig.canvas.draw()
        plt.show(block=True)
    
    #==========================================
    def save(self, FN, format = "auto", dpi=200):
    #==========================================
        # have the layout adjust one last time:
        self.fig.tight_layout()
        #plt.subplots_adjust(top=0.90) # make space for multiline suptitle
        #self.fig.canvas.draw() # strange. This tends to crash if multiple PlotGalleries are saving one after another. BUT REQUIRED! Otherwise plot remains empty.
        
        print "PlotGallery: Saving to " + FN
        if format == "auto":
            plt.savefig(FN, dpi=dpi)
        else:
            plt.savefig(FN, format=format, dpi=dpi)
        
#         if int(INI["Visualization"]["Invert"]) == True:
#             tmp = readImg(FN)
#             tmp = invertImage(tmp)
#             saveBGR(tmp, FN)

    #==========================================
    def savefig(self, FN):
    #==========================================     
        # NEVER CALL PLT.SAVEFIG SOMEWHERE AFTER THE CLASS INSTANTIATION!
        # ALWAYS CALL IT FROM AS A CLASS MEMBER FUNCTION, OR FORCE DEL MANUALLY
        # GARBAGE COLLECTION DOES NOT GET THE GLOBAL "PLT"-REFERENCE
        # AND MIGHT CLEAR THE CLASS INSTANCE BEFORE THE SAVEFIG CALL   
        # NOT REALLY CLEAR
        plt.savefig(FN, dpi=DPI, quality=Plot_JPG_Quality)
        print "QuickPlot_RES savefig: saved ", FN       

    #==========================================
    def __del__(self):
    #==========================================
        self.fig.clf()
        plt.close() 


# EXAMPLE:

def EXAMPLE_ScatterPlot():
    x,y,z = get_XYZ_Random_Values()
    ScatPlot = ScatterPlot()
    ScatPlot.scatter(x,y,"PointSetA", axidx=0)
    ScatPlot.scatter(x*4,y, "PointSetB", axidx=0, color = 'green')
    ScatPlot.scatter(x,y, "PointSetC", axidx=1, color = 'green')
    ScatPlot.add_legend()
    ScatPlot.add_legend(axidx=1)
    plt.show()    

#|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
class RD_Curves(object):
#|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
    #==========================================
    def __init__(self, rows, cols, DimX = A4q_x, DimY = A4q_y):
    #==========================================
        super(RD_Curves, self).__init__()
        
        # CHANGE HERE -------------------------------------------------
        self.colors     = ['r','g','b','y','c','m','k'] 
        self.markers    = ['+','>','o','D','d','x','H','<']
        self.linestyle  = '-'
        self.linewidth  = 2.0 
        # -------------------------------------------------------------

        self.rows, self.cols = rows, cols
        self.fig, axarr = plt.subplots(self.rows, self.cols, figsize=(DimX,DimY), dpi=DPI)

        self.axes  = self.fig.axes # oh yes!!
        
        for ax in axarr:   
            ax.set_xlabel('Bitrate in Mbit/s')
            ax.set_ylabel('PSNR Y in dB')
        
        #alternative: 
        #self.axarr = np.array(self.axarr)
        #self.axarr = self.axarr.reshape(-1):
                
    #==========================================        
    def add_Curve(self, axidx, crvidx, label, DataX, DataY):
    #==========================================
        #ax.legend(Pretty_Variants, loc='best')
        ax = self.fig.axes[axidx]
        mew=0.5*1.3
        ms=6*1.3
        marker, color, linestyle, linewidth = str(self.markers[crvidx]), str(self.colors[crvidx]), self.linestyle, self.linewidth
        ax.plot(DataX, DataY, label=label, marker=marker, color=color, linestyle=linestyle, linewidth=linewidth, mew=mew, ms=ms)

    #==========================================
    def save(self, FN):
    #==========================================
        self.fig.tight_layout()
        print "Plot RD_Curves: Saving to " + FN
        plt.savefig(FN, dpi=DPI)
        
    #==========================================
    def __del__(self):
    #==========================================
        self.fig.clf()
        plt.close()     
    

#|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
class ScatterPlot(object):
#|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
    #==========================================
    def __init__(self, xLabel, yLabel):
    #==========================================
        self.rows, self.cols = 1,2
        self.fig, self.axarr = plt.subplots(self.rows, self.cols, figsize=(A4q_x*1,A4q_y*1))
        self.xLabel = xLabel
        self.yLabel = yLabel
        
        self.colors = ['blue', 'green', 'magenta', 'cyan']

        self.ScatterHandles = ([],[]) # required for legend
        self.ScatterDescrip = ([],[]) # required for legend
        
    #==========================================
    def scatter(self, x, y, Descr,
                axidx=0, color = 'blue', size=25, LogScale=False):
                #marker='o', edgecolor='black' ): # use markers later, problems here (seems to override color) 
    #==========================================
        
        x = list(x) #empty numpy arrays cause problems, so cast to "list"
        y = list(y)
    
        ax = self.axarr[axidx]
        
        # SCATTER 
        Handle = ax.scatter(x,y, color=color,s=size) #, marker=marker, edgecolor=edgecolor)
        self.ScatterHandles[axidx].append(Handle) # required for legend 
        self.ScatterDescrip[axidx].append(Descr) # required for legend 
        
        ax.set_xlabel(self.xLabel)
        ax.set_ylabel(self.yLabel)
        
        if LogScale:
            ax.set_yscale('log')
            ax.set_xscale('log')

        #ax.set_aspect(1./ax.get_data_ratio()) # make axes square
        
    #==========================================
    def add_legend(self, axidx=0):
    #==========================================
        ax = self.axarr[axidx]
        ax.legend(self.ScatterHandles[axidx],
               self.ScatterDescrip[axidx],
               scatterpoints=1,
               loc='lower left',
               ncol=3,
               fontsize=8)
        
    #==========================================
    def savefig(self, FN):
    #==========================================     
        plt.savefig(FN, dpi=DPI, quality=Plot_JPG_Quality)
        print "ScatterPlot.savefig(): saved to: ", FN  
        

#|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
class QuickPlot_RES(object):
#|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
    #==========================================
    def __init__(self, RES, Title, BlackList=[], WhiteList="All", Pages=1):
    #==========================================
        super(QuickPlot_RES, self).__init__()
        # CHANGE HERE -------------------------------------------------
        # -------------------------------------------------------------
        
        self.RES = RES
        
        from collections import Counter
        SelectedKeys = []
        #print sorted(RES.keys())
        for key in sorted(RES.keys()):
            #print key
            #print type(RES[key])
            #print type(RES[key][:])
            #print RES[key][:]
            
            if type(RES[key]) == list:
                Types = [type(Number) for Number in RES[key][:]] #dont need this: if Number != None]
                MostCommonType = Counter(Types).most_common() 
                #print "MostCommonType: ", MostCommonType
                if  MostCommonType[0][0] != np.ndarray and \
                    MostCommonType[0][0] != list and \
                    MostCommonType[0][0] != str and \
                    MostCommonType[0][0] != tuple:
                    
                    #some curves may not be interesting
                    isBlacklisted = False
                    for entry in BlackList:
                        if entry in key:
                            isBlacklisted = True
                            
                    if WhiteList != "All": #whitelist overrides blacklist
                        isBlacklisted = True 
                        for entry in WhiteList:
                            if entry in key:
                                isBlacklisted = False                            
                         
                    # we got a winner: plot this as a curve   
                    if not isBlacklisted:
                        SelectedKeys.append(key)

        # Note: "+1" makes space for uber title
        self.rows, self.cols = len(SelectedKeys)+1, 1
        self.fig, self.axarr = plt.subplots(self.rows, self.cols, figsize=(A4q_x*2,A4q_y*Pages))
        
        self.axarr[0].axis('off') # space for uber title
        
        #Plotting of each key
        for idx, key in enumerate(SelectedKeys):
            self.subplot_Key(key, idx+1)
            
        self.fig.suptitle(Title, fontsize=16)
        plt.tight_layout(pad=1, w_pad=1, h_pad=1)
        self.SelectedKeys = SelectedKeys
            
    #==========================================
    def subplot_Key(self, Key, r):
    #==========================================
    
        ax = self.axarr[r] # getting required AxesSubplot object
        ax.axis('on') #"activate" empty plot
        ax.set_title(Key, size='large', weight="bold")
        
#         if self.RES[Key][0] == None
#             self.RES[Key][0] == -1 #otherwise plot will start at idx=1
        
        ax.set_xlim([0,len(self.RES[Key])])
        
        ax.plot(self.RES[Key], label=Key, c='g')
        self.fig.canvas.draw()
        #ax.set_xlabel(Key)

    #==========================================
    def set_YLimits(self, Key, Min, Max):
    #==========================================
        idx = self.SelectedKeys.index(Key)
        ax = self.axarr[idx + 1]
        ax.set_ylim([Min, Max])
        
    #==========================================
    def show(self):
    #==========================================
        plt.tight_layout(pad=1, w_pad=1, h_pad=1)
        self.fig.canvas.draw()
        plt.show(block=True)

    #==========================================
    def savefig(self, FN):
    #==========================================     
        # NEVER CALL PLT.SAVEFIG SOMEWHERE AFTER THE CLASS INSTANCE
        # GARBAGE COLLECTION DOES NOT GET THE GLOBAL "PLT"-REFERENCE
        # AND MIGHT CLEAR THE CLASS INSTANCE BEFORE THE SAVEFIG CALL   
        plt.savefig(FN, dpi=DPI, quality=Plot_JPG_Quality)
        print "QuickPlot_RES savefig: saved ", FN       

#     #==========================================
#     def __del__(self):
#     #==========================================
#         print "QuickPlot_RES destructor called"
#         self.fig.clf()
#         plt.close() 
         
#==========================================
if __name__ == '__main__':
#==========================================   

    __builtins__.ProjectDir = os.path.abspath(os.path.dirname(os.path.realpath(__file__)) + "/.." )
    Image_FN = ProjectDir + "/Various/Resources/Special/LMS_Logo.png"

    rows, cols = 3,2 
    Plot_Test = PlotGallery(rows, cols)
    Plot_Test.SelPos = None # plot everything
    Plot_Test.set_suptitle("PLOT GALLERY TEST")
    Plot_Test.set_Left_Labels(("Row0", "Row1", "Row2"))
    
    #plt.set_cmap('hot') 
    
    for row in range(rows):
        for col in range(cols):
            TestMatrix = np.random.rand(480,800)
            
            if col == 0 and row == 0:
                Img = readImg(Image_FN)
                Plot_Test.subplot(row, col, Img, "TestMatrix at %d,%d" % (row, col))
            else:
                Plot_Test.subplot(row, col, TestMatrix, "TestMatrix at %d,%d" % (row, col))
            
            Plot_Test.set_Aspect_Ratio(row, col, "auto")
            #Plot_Test.set_Aspect_Ratio(row, col, "equal")
                        
            if col == 1 and row == 0:
                Plot_Test.set_Colormap(row, col, "hot")
            if col == 1 and row == 1:
                Plot_Test.set_Colormap(row, col, CMap_RedGreen)
            if col == 1 and row == 2:
                Plot_Test.set_Colormap(row, col, CMap_RedBlue)
                
    Plot_Test.save(ProjectDir + "/tmp/Plot_Test.png")
    plt.show()
            
    
    
    
