#!/usr/bin/env python
# coding: utf8
# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)
import sys, os
CoreDir = os.path.realpath(os.path.dirname(os.path.realpath(__file__)) + "/..") # CHANGE HERE
sys.path.extend([CoreDir ,])
assert "HARP_Core" == os.path.split(CoreDir)[1], "Not CoreDir: " + CoreDir
from Imports_Basic import *
set_HARP_Core_Environment(CoreDir, __name__)

from Imports_Basic import *
from General import *
from OpenCV import *
from Plot import *
from Regex import *
from PlotGallery import *
from HDT.HarpDataTree import *
from HDT.POST_CUs import *
from Pandas import *
from HDT.RDO_CUs import *
from HDT.ConsumedBits import *

from External import configobj
from Multithreading import *
from ICIP2016_Visualize import *


from PySide import QtCore, QtGui
from PySide.QtGui import *


# |||||||||||||||||||||||||||||||||||||||||||||||||
class VisTask(object):
# |||||||||||||||||||||||||||||||||||||||||||||||||

    #========================
    def __init__(self, POC, Desc, FuncName):
    #========================
        self.POC = POC
        self.Desc = Desc
        self.FuncName = FuncName
    
    #========================
    def __call__(self):
    #========================
        #time.sleep(0.1) # pretend to take some time to do the work
        #return '%s * %s = %s' % (self.a, self.b, self.a * self.b)
        eval(self.FuncName + "(self.POC, self.Desc)")
        return "Done: " + self.FuncName
    
    #========================
    def __str__(self):
    #========================
        return self.FuncName


# |||||||||||||||||||||||||||||||||||||||||||||||||
class C_TAB(QtGui.QWidget):
# |||||||||||||||||||||||||||||||||||||||||||||||||

    #========================
    def __init__(self, title, parent):
    #========================
        super(C_TAB, self).__init__(parent)

        self.parent = parent
        self.scaleFactor = 0.0
        self.title = title
 
        self.imageLabel = QtGui.QLabel()
        self.imageLabel.setBackgroundRole(QtGui.QPalette.Base)
        self.imageLabel.setSizePolicy(QtGui.QSizePolicy.Ignored, QtGui.QSizePolicy.Ignored)
        self.imageLabel.setScaledContents(True)
 
        self.scrollArea = QtGui.QScrollArea()
        self.scrollArea.setBackgroundRole(QtGui.QPalette.Dark)
        self.scrollArea.setWidget(self.imageLabel)
        
        #never use this! will basically disable resizing code
        #self.scrollArea.setWidgetResizable(True)
 
        self.setWindowTitle("C_ImageViewer")
        self.VBoxLayout = QtGui.QVBoxLayout()
        self.setLayout(self.VBoxLayout)
        self.VBoxLayout.addWidget(self.scrollArea)
        
        self.Pixmap = None
        self.lastupdated = datetime.datetime.fromtimestamp(0) # 1969-12-31 17:00:00

    
    #========================
    def setJPEG(self, JPEG_FN):
    #========================
        #JPEG_FN = "/home/dom/Desktop/Poster PYTHON/SMALL ICIP 2016 -- POSTER PYTHON -- v0.8.jpg"
        self.Pixmap = QtGui.QPixmap.fromImage(QtGui.QImage(JPEG_FN))         
        self.update()
        
        #self.fitToWindowAct.setEnabled(True)
  
    #========================
    def resizeEvent(self, event):
    #========================
        super(C_TAB, self).resizeEvent(event)
        #print "Resize Event!"
        self.redraw()
            
    #========================
    def redraw(self):
    #========================
        if self.Pixmap != None:
            #print "Resizing..."
            w = self.scrollArea.width()
            h = self.scrollArea.height()
            #print w, h
            
            p_w, p_h = self.Pixmap.width(), self.Pixmap.height()
            #if p_w > p_h: # landscape
            ScaledPixmap = self.Pixmap.scaledToHeight(h, QtCore.Qt.SmoothTransformation)
            
            
            self.imageLabel.setPixmap(ScaledPixmap)
            self.imageLabel.adjustSize()         
        else: 
            self.imageLabel.clear()
            
#==========================================
def createYUVSequ(ImgList, YUV_FN):
#==========================================
    #YUV_FN = CoreTmpDir + "/tmp_%dx%d.yuv" % (DimX, DimY)
    FH = open(YUV_FN, "wb")

    for POCIdx, Img in enumerate(ImgList):
        Img = cv2.cvtColor(Img, cv2.COLOR_RGB2YUV)
        y,u,v = cv2.split(Img)
        
        FH.write(np.frombuffer(np.getbuffer(y), dtype=np.uint8));
        u = cv2.resize(u, (0, 0), fx=0.5, fy=0.5, interpolation=cv2.INTER_CUBIC)
        v = cv2.resize(v, (0, 0), fx=0.5, fy=0.5, interpolation=cv2.INTER_CUBIC)
        FH.write(np.frombuffer(np.getbuffer(u), dtype=np.uint8))
        FH.write(np.frombuffer(np.getbuffer(v), dtype=np.uint8))  
        print "YUV created: %s" % (YUV_FN)
    
    FH.close()
             
#==========================================
def main():
#==========================================

    #========================
    def compressGOP_internal():
    #========================
        
        Button1.setEnabled(False)
    
        print ""
        POCIdx = SpinBox2.SpinBox.value()
        NumPOCs = SpinBox11.SpinBox.value()
        MultiPOC_Idcs = range(NumPOCs)
        timer.stop()#stop webcam
        YUV_FN = KitTmpDir + "/tmp_%dx%d.yuv" % (Frames[0].shape[1], Frames[0].shape[0])
        createYUVSequ(Frames, YUV_FN)
        
        if 1:
            OldDir = changeDir(KitTmpDir)
            CfgFN = KitDir + "/Src_HM/cfg/encoder_lowdelay_P_main.cfg"
            EncBinary = KitDir + "/bin/TAppEncoder"
            
            Cmd = "%s -i %s -wdt 640 -hgt 360 " \
            "-f %d -c %s  --QP=%d " \
            "-b str.bin -o rec.yuv -fr 30 --InputBitDepth=8" % (EncBinary, YUV_FN, NumPOCs, CfgFN, SpinBox1.SpinBox.value())
            
            # Some ugly segfault. Does not hinder functionality. We will trace it down ASAP.
            if os.system(Cmd) != 0:
                print "WARNING: returned with non-zero"

            changeDir(OldDir)
        if 1:
            deleteFiles(KitTmpDir, "*.jpg")
            progressBar.setValue(0)
            #clear TABs
            for tabidx, TAB in enumerate(TABs):
                wid_inner.tab.setTabIcon(tabidx, NoIcon)
                #TAB.Pixmap = None
                TAB.redraw() 
                TAB.update()
                TAB.repaint()
                app.processEvents();  
            
            #for POCIdx in range(1, NumPOCs):

            POC = load_POC(KitTmpDir, POCIdx)
            Desc = "Phoenix"
            

            
            tasks, results, num_consumers = get_Queues()
            num_jobs = 7

            Task4 = VisTask(POC, Desc, "plot_Bit_Consumption")
            tasks.put(Task4)
            Task5 = VisTask(POC, Desc, "plot_Overview")
            tasks.put(Task5)
            
            MultiPOCs = NEW_unpickle_POCs(KitTmpDir, MultiPOC_Idcs)
            
            Task1 = VisTask(POC, Desc, "plot_RDODetails_ME")
            tasks.put(Task1)
            Task2 = VisTask(POC, Desc, "plot_RDODetails_RESI")
            tasks.put(Task2)
            Task3 = VisTask(POC, Desc, "plot_RDODetails_PRED")
            tasks.put(Task3)
            
            
            Task6 = VisTask(MultiPOCs, Desc, "plotMULTIPOC_CU_Modes_Lum_Resi")
            tasks.put(Task6)
             
            Task7 = VisTask(MultiPOCs, Desc, "plotMULTIPOC_SideInfoBars")
            tasks.put(Task7)


            
            # Add a poison pill for each consumer
            for i in xrange(num_consumers):
                tasks.put(None)
        
            # Wait for all of the tasks to finish
            
            while 1: 
                app.processEvents();
                doBreak = False
                progressBar.setValue(results.qsize())
                if results.qsize() >= 7:
                    doBreak = True
                #print "QSIZE:", results.qsize()
                time.sleep(0.1)
                for FN in glob.glob(os.path.join(KitTmpDir, "*.jpg")):
                    #print FN
                    for tabidx, TAB in enumerate(TABs):
                        TAB_Text = TABs_Texts[tabidx]
                        if TAB_Text[0] in pureFN(FN):
                            #if TAB.Pixmap == None:
                            if 1:
                                if (datetime.datetime.now() - TAB.lastupdated).seconds > 10:
                                    time.sleep(1) #wait till jpeg is written
                                    TAB.setJPEG(FN)
                                    wid_inner.tab.setTabIcon(tabidx, GreenIcon)
                                    TAB.redraw() 
                                    TAB.update()
                                    TAB.repaint()   
                                    TAB.lastupdated = datetime.datetime.now()
                                    print "Loaded ", pureFN(FN)
                app.processEvents();
                if doBreak:
                    break
            
            # Start printing results
            while num_jobs:
                result = results.get()
                print 'Result:', result
                num_jobs -= 1
                
            tasks.join()
            print "Tasks joined!"  
            
            for tabidx, TAB in enumerate(TABs):
                print "Updating TAB", tabidx
                TAB.redraw() 
                TAB.update()
                TAB.repaint()    

        Button1.setEnabled(True) 
        timer.start() #start webcam    
        #print SpinBox3.value()
        if SpinBox3.SpinBox.value() == 1:
            timer2.start()
        print "END OF compressGOP_internal()"
            
    #sys.exit(0)    
    StartTime = get_Start_Time()

    #main application
    app = QtGui.QApplication(sys.argv)
    wid = QtGui.QWidget()
    grid = QtGui.QGridLayout(wid)
    wid.setLayout(grid)
    
    # setting the inner widget and layout
    grid_inner = QtGui.QGridLayout(wid)
    wid_inner = QtGui.QWidget(wid)
    wid_inner.setLayout(grid_inner)
    
    # |||||||||||||||||||||||||||||||||||||||||||||||||
    class C_SpinBox(QtGui.QWidget):       
    # |||||||||||||||||||||||||||||||||||||||||||||||||
        #========================
        def __init__(self, Text, Range, Value, parent=None):
        #========================
            super(C_SpinBox, self).__init__(parent)
            
            self.Layout = QHBoxLayout()
            self.SpinBox = QSpinBox()
            self.SpinBox.setRange(*Range)
            self.SpinBox.setValue(Value)
            self.Label = QLabel(Text)
            self.Layout.addWidget(self.Label)
            self.Layout.addWidget(self.SpinBox)
            self.setLayout(self.Layout)
            
            
    # |||||||||||||||||||||||||||||||||||||||||||||||||
    class C_LabeledImg(QtGui.QWidget):       
    # |||||||||||||||||||||||||||||||||||||||||||||||||
        #========================
        def __init__(self, Text, parent=None):
        #========================
            super(C_LabeledImg, self).__init__(parent)
            
            self.Layout = QVBoxLayout()
            self.TextLabel = QLabel(Text)
            self.ImgLabel = QLabel()
            self.Layout.addWidget(self.TextLabel)
            self.Layout.addWidget(self.ImgLabel)
            self.setLayout(self.Layout)
            self.ImgLabel.setFixedSize(int(640/2.2), int(360/2.2))
        
        #========================
        def setImg(self, Frame):
        #========================
            #print Frame
            height, width, channel = Frame.shape
            bytesPerLine = 3 * width
            qImg = QImage(Frame.data, width, height, bytesPerLine, QImage.Format_RGB888)
            qImg = qImg.rgbSwapped()
            
            w = self.ImgLabel.width()
            h = self.ImgLabel.height()

            # set a scaled pixmap to a w x h window keeping its aspect ratio 
            #label->setPixmap(p.scaled(w,h,Qt::KeepAspectRatio));
            Pixmap = QtGui.QPixmap.fromImage(qImg)
            Pixmap = Pixmap.scaled(w,h,QtCore.Qt.KeepAspectRatio)
            
            self.ImgLabel.setPixmap(Pixmap)

    
    # add the inner widget to the outer layout
    grid.addWidget(wid_inner)
    
    # add tab frame to widget
    wid_inner.tab = QtGui.QTabWidget(wid_inner)
    grid_inner.addWidget(wid_inner.tab)
    
    splitter = QtGui.QSplitter()
    ConfigBarLayout = QtGui.QVBoxLayout()
    ConfigBar = QtGui.QWidget()
    ConfigBar.setLayout(ConfigBarLayout)
    Button1 = QPushButton("Compress GOP")
    ConfigBarLayout.addWidget(Button1)
    Button1.clicked.connect(compressGOP_internal)
    
#     SpinBox = QtGui.QSpinBox()
#     SpinBox.setRange(1, 51)        
#     SpinBox.setSingleStep(1)
#     SpinBox.setValue(20)
    SpinBox1 = C_SpinBox("QP", (1,51), 20)
    ConfigBarLayout.addWidget(SpinBox1)
    
    SpinBox11 = C_SpinBox("Number of POCs", (0,4), 2)
    ConfigBarLayout.addWidget(SpinBox11)
    SpinBox2 = C_SpinBox("Shown POC", (0,4), 1)
    
    SpinBox3 = C_SpinBox("Autorun", (0,1), 0)
    
    ConfigBarLayout.addWidget(SpinBox2)
    # DISABLED, VERY STRANGE, WILL SEGFAULT ON EXIT. PROBABLY PYSIDE BUG.
    #ConfigBarLayout.addWidget(SpinBox3)
    ConfigBarLayout.addStretch()
    
    SecondLayout = QHBoxLayout()
    progressBar = QProgressBar()
    progressBar.setOrientation(QtCore.Qt.Vertical)
    progressBar.setSizePolicy(QtGui.QSizePolicy.Ignored, QtGui.QSizePolicy.Ignored)
    progressBar.setFixedWidth(20)
    progressBar.setRange(0, 7)
    SecondLayout.addWidget(ConfigBar)
    SecondLayout.addWidget(progressBar)
    SecondWidget = QWidget()
    SecondWidget.setLayout(SecondLayout)
    
    splitter.addWidget(SecondWidget)
    splitter.addWidget(wid)
    splitter.setStretchFactor(0, 0)
    splitter.setStretchFactor(1, 1)
    
    # create tab ( NEW TAB IS A SIMPLE QWIDGET! )
    new_tab = QtGui.QWidget(wid_inner.tab)
    grid_tab = QtGui.QGridLayout(new_tab)
    grid_tab.setSpacing(10)
    new_tab.setLayout(grid_tab)
    new_tab.tab_name_private = "test1"
        
    source = 0
    cap = cv2.VideoCapture(source)
    cap.set(cv2.cv.CV_CAP_PROP_FRAME_WIDTH, 640)
    cap.set(cv2.cv.CV_CAP_PROP_FRAME_HEIGHT, 480)
    if cap is None or not cap.isOpened():
        print 'Warning: unable to open video source: ', source  
        assert 0  
         
    ret, img = cap.read()
    assert ret == 1
# #     cv2.imshow('capture', img)
# #     cv2.waitKey()
         
    Frames = []
    for i in range(4):
        img = cap.read()[1]
        img = img[0:360,:,:]
        Frames.append(img)
        #Frames.append(None)
     
    def updateWebcam():
        #print "Updating webcam images"
        ret, img = cap.read()
        img = img[0:360,:,:]
        #print img.shape
        Frames.insert(0, img)
        del Frames[4]
        assert len(Frames) == 4
        for i in range(4):
            Frame = Frames[i] 
            FrameLabels[i].setImg(Frame)
             
         
    FramesLayout = QVBoxLayout()
    FrameLabels = []
    for i in range(4):
        FrameLabels.append(C_LabeledImg("POC%d" %i))
        FramesLayout.addWidget(FrameLabels[i])
         
    ConfigBarLayout.addLayout(FramesLayout)
 
  
    timer = QtCore.QTimer()
    timer.setInterval(250)
    timer.start()
    timer.setSingleShot(False)
    timer.timeout.connect(updateWebcam)
      
      
#     # start countdown for next GOP

    # DISABLED, VERY STRANGE, WILL SEGFAULT ON EXIT. PROBABLY PYSIDE BUG.
#     timer2 = QtCore.QTimer()
#     timer2.setInterval(10000)
#     timer2.setSingleShot(True)
#     timer2.timeout.connect(compressGOP_internal)  
   
    
    
    TABLUT = {}
    
    class CTAB:
        pass
    
    TABs = []
    #TABs_Descs = 
    
    TABs_Texts = []
    TABs_Texts.append(("Overview Final Decisions", "Final Decisions", ""))
    TABs_Texts.append(("CU Modes and Luminance", "3x POC", ""))
    TABs_Texts.append(("Bit Consumption Overview", "Bits Overview", ""))
    TABs_Texts.append(("Bit Consumption Details - Intra", "Bits Intra", ""))
    TABs_Texts.append(("Bit Consumption Details - Inter", "Bits Inter", ""))
    TABs_Texts.append(("Bit Consumption Details - Skip", "Bits Skip", ""))
    TABs_Texts.append(("Bit Consumption Details - Merge", "Bits Merge", ""))
    
    TABs_Texts.append(("RDO Details - Motion-Estimation", "RDO Mo-Estim.", ""))
    TABs_Texts.append(("RDO Details - PRED Buffer", "RDO Prediction", ""))
    TABs_Texts.append(("RDO Details - RESI Buffer", "RDO Residual", ""))
    
    TABs_Texts.append(("SideInfo Bars", "SideInfo Bars", ""))
    
    
#     TABs_Titles = , "Bit Consumption Details", "Bit Consumption Ref Idcs",\
#                   "Compact Overview", "CU Modes and Luminance", "SideInfo Bars"\
#                   "RDO Details - Motion-Estimation", "RDO Details - PRED Buffer", \
#                   "RDO Details - RESI Buffer"
    NumTabs = len(TABs_Texts)
    
    parent = wid_inner.tab
    for i in range(NumTabs):
        TAB = C_TAB(TABs_Texts[i][1], parent)
        wid_inner.tab.addTab(TAB, TAB.title)
        #TAB.setStyleSheet("background-color: RED");
        TABs.append(TAB)
    
    #TABs[0].setStyleSheet("border-left: 1px solid RED");

    GreenIcon = QIcon(CoreDir + "/RES/CheckMark.svg");
    NoIcon =  QIcon(CoreDir + "/RES/SandClock.svg")
    
    
       
    wid_inner.tab.setCurrentWidget(TABs[0])    
    #splitter.resize(500, 400)
    
    
    

            
            
        
#     args = []
#     MyThread = threading.Thread(target=runOnce, args=args)
#     MyThread.daemon = False
#     MyThread.start()
#     
#     for TAB in TABs:
#         TAB.setJPEG("/home/dom/Desktop/HARP_Kit/tmp/RDO Details - RESI Buffer Phoenix POC00001.jpg")
#     
    splitter.show()
    splitter.resize(1890-600, 1000)
    
    def myExitHandler():
        #timer.stop()
        #timer2.stop()
        print "Exiting"
    app.aboutToQuit.connect(myExitHandler)
    app.exec_()
    


#==========================================
if __name__ == '__main__':
#==========================================
    main()
