#!/usr/bin/env python
# coding: utf8
# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)

from Imports_Basic import *
from General import *
from OpenCV import *
from Plot import *
from Regex import *
from PlotGallery import *
from HDT.HarpDataTree import *
from HDT.POST_CUs import *
from Pandas import *
from HDT.RDO_CUs import *
from HDT.ConsumedBits import *

#from External import configobj
#from a_prepareTestDir import prepareTestDir


if 0: # OVERVIEW
    
    # OVERVIEW_2x2
    Name = "OVERVIEW_2x2"
    plot_OVERVIEW_2x2(POC, Name, Desc)

#==========================================
def plot_Overview(POC, Desc):
#==========================================

    POCIdx = POC["Idx"]
    FN = KitTmpDir+ "/Compact Overview " + Desc + " POC%05d" % POC["Idx"] + ".jpg"
    
    CurYUV = get_POC_YUV_Pixels(POC)
    if len(POC["List0_Refs"]) > 0:
        RefYUV = get_POC_YUV_Pixels(POC["List0_Refs"][0])
    else:
        RefYUV = dc(CurYUV)
        RefYUV[:,:,:] = np.NaN
        
    LumDiff = getLumDiff(CurYUV, RefYUV)

    #saveBGR(Final, KitTmpDir + "/VisRotation_POC%04d.jpg" % POCIdx)
    
    #SHOW(POC)
    if len(POC["List0_Refs"]) == 1: # 
        RefIdcsColors = colornames2BGR(('yellow',))
    if len(POC["List0_Refs"]) == 2: # 
        RefIdcsColors = colornames2BGR(('cyan', 'yellow',))       
    if len(POC["List0_Refs"]) == 3: # 
        RefIdcsColors = colornames2BGR(('cyan', 'violet', 'yellow'))     
    if len(POC["List0_Refs"]) == 4: # 
        RefIdcsColors = colornames2BGR(('cyan', 'violet', 'chocolate', 'yellow'))   
    
    # Visualize Modes and Ref Indices
    VisModes = draw_CU_Modes(POC)
    VisRefIndices = draw_Ref_Indices(POC, RefIdcsColors)
    VisModes = draw_MV_Vectors(POC, VisModes)
    VisRefIndices = draw_MV_Vectors(POC, VisRefIndices)
    VisResi = get_FINAL_Buffer(POC, "Y_Resi")
    
    Gallery  = PlotGallery(2, 2, A4_x, A4_x*0.6)
    Gallery.subplot(0, 0, LumDiff,  "a) Ref / Current Frame" )
    Gallery.subplot(0, 1, VisModes,  "b) CU Modes")
    Gallery.subplot(1, 1, VisRefIndices,  "c) Ref Indices and MVs")
    Gallery.subplot(1, 0, VisResi,  "d) Luminance Residual", cmap='jet')
    Gallery.set_Limits(1, 0, *ResiLimits)  

    # LEGENDS
    Colors = IntraColor, InterColor, MergeColor, SkipColor
    Labels = "Intra", "Inter", "Merge", "Skip"
    create_Epic_Legend(Gallery.axarr[0, 1], Colors, Labels, bbox=(0.2, 1.1) )
    Colors = RefIdcsColors
    Labels = "RefPos1", "RefPos2", "RefPos3", "RefPos4"
    create_Epic_Legend(Gallery.axarr[1, 1], Colors, Labels, bbox=(0.2, 1.1) )
        
    Gallery.fig.tight_layout()
    plt.savefig(FN, dpi=DPI)
    close_All_Figures()
    del Gallery 
 
    
#==========================================
def plot_Bit_Consumption(POC, Desc):
#==========================================

    FN1 = KitTmpDir+ "/Bit Consumption Overview - " + Desc + " POC%05d" % POC["Idx"] + ".jpg"
    FN2 = KitTmpDir+ "/Overview Final Decisions - " + Desc + " POC%05d" % POC["Idx"] + ".jpg"
    

    print "Analyzing POC ", POC["Idx"]
    ConsumerNamesAll  = "Coeffs", "SkipFlag",  "PredMode", "PartSize", "PredInfoAllPUs", "IntraIPCM" , "SkipMergeIndex",
    ConsumerNames  = "Coeffs", "SkipFlag",  "PredMode", "PartSize", "PredInfoAllPUs",                   ###  "IntraIPCM" , "SkipMergeIndex",    
    ConsumedBits = mine_Consumed_Bits(POC, ConsumerNamesAll, Desc)
    
    # OVERVIEW Consumed Bits
    Gallery, fig, axarr = plot_Consumed_Bits_OVERVIEW(POC, ConsumedBits, ConsumerNames)
    Gallery.fig.tight_layout()
    Gallery.set_suptitle("Bit Consumption Overview", 30)
    plt.savefig(FN1, dpi=DPI)
    close_All_Figures()
    del Gallery 
    
    # REF IDCS Consumed Bits
    Gallery, fig, axarr = plot_RefIdcs_MVField_Buffers(POC)
    Gallery.fig.tight_layout()
    Gallery.set_suptitle("Overview Final Decisions", 30)
    plt.savefig(FN2, dpi=DPI)
    close_All_Figures()
    del Gallery 

    # DET    AILS Consumed Bits
    ModeNames = ("Intra", "Inter" , "Merge", "Skip")
    for Mode in ModeNames:
        FN3 = KitTmpDir+ "/Bit Consumption Details - " + Mode + " " + Desc + " POC%05d" % POC["Idx"] + ".jpg" # will get further suffix
        #FN3_new = insert_Into_FN_right(FN3, " %s" % Mode)
        Gallery, fig, axarr = plot_Consumed_Bits_DETAIL(Mode, POC, ConsumedBits, ConsumerNames)
        makeSpaceForSubtitle
        Gallery.set_suptitle("Consumed Bits - " + Mode, 30)
        Gallery.fig.tight_layout()
        plt.savefig(FN3, dpi=DPI)
        close_All_Figures()
        del Gallery 

#==========================================      
def plot_RDODetails_ME(POC, Desc):
#==========================================
    FN1 = KitTmpDir+ "/RDO Details - Motion-Estimation " + Desc + " POC%05d" % POC["Idx"] + ".jpg"    
    # RDO ME
    Gallery, fig, axarr = plot_RDO_Motion_Estimation(POC)
    Gallery.fig.tight_layout()
    Gallery.set_suptitle("RDO Details - Motion-Estimation", 30)
    plt.savefig(FN1, dpi=DPI)
    close_All_Figures()
    del Gallery, fig, axarr  
    
#==========================================      
def plot_RDODetails_RESI(POC, Desc):
#==========================================
    FN2 = KitTmpDir+ "/RDO Details - RESI Buffer " + Desc + " POC%05d" % POC["Idx"] + ".jpg"
    # RDO Buffers RESI
    Gallery, fig, axarr = plot_RDO_Resi_Buffers(POC)
    Gallery.fig.tight_layout()
    Gallery.set_suptitle("RDO Details - RESI Buffer", 30)
    plt.savefig(FN2, dpi=DPI)
    close_All_Figures()
    del Gallery, fig, axarr 
        
#==========================================
def plot_RDODetails_PRED(POC, Desc):
#==========================================

    FN3 = KitTmpDir+ "/RDO Details - PRED Buffer " + Desc + " POC%05d" % POC["Idx"] + ".jpg"
    # RDO Buffers PRED
    Gallery, fig, axarr = plot_RDO_Pred_Buffers(POC)
    Gallery.fig.tight_layout()
    Gallery.set_suptitle("RDO Details - PRED Buffer", 30)
    plt.savefig(FN3, dpi=DPI)
    close_All_Figures()
    del Gallery, fig, axarr  
    
#==========================================
def plotMULTIPOC_SideInfoBars(POCs, Desc):
#==========================================

    POC_Idcs = get_POC_Idcs(POCs)
    Str = "_POCs%d-%d" % (POC_Idcs[0], POC_Idcs[-1])
    FN_JPG   = KitTmpDir + "/SideInfo Bars"  + Desc + " " + Str + ".jpg"
    FN_COEF  = KitTmpDir + "/SideInfo Bars COEF"  + Desc + " " + Str + ".xlsx"
    FN_SIDE  = KitTmpDir + "/SideInfo Bars SIDE"  + Desc + " " + Str + ".xlsx"

    printLARGE("Extracting barplot coeff/sinfo DFs")
    (df_coeff, df_sinfo, POCBits) = get_DF_Coeff_SInfo(POCs)
    
    print "df_coeff DF:\n", df_coeff
    print "df_sinfo DF:\n", df_sinfo
    save_DF_to_Excel(df_coeff, FN_COEF, float_format="%0.4f")
    save_DF_to_Excel(df_sinfo, FN_SIDE, float_format="%0.4f")
    
    fig, axarr = plot_CoeffSinfo_BarPlots(POC_Idcs, df_coeff, df_sinfo)
    
    fig.tight_layout()
    print "Saving figure to: ", FN_JPG
    plt.savefig(FN_JPG, dpi=DPI)
    close_All_Figures()
    
#==========================================
def plotMULTIPOC_CU_Modes_Lum_Resi(POCs, Desc):
#==========================================
    
    POC_Idcs = get_POC_Idcs(POCs)
    Str = "_POCs%d-%d" % (POC_Idcs[0], POC_Idcs[-1])
    FN = KitTmpDir + "/CU Modes and Luminance Residuals - " + Desc + " " + Str + ".jpg"
    
    cols = len(POCs)
    Gallery  = PlotGallery(2, cols, A4_y, A4_x) # / (5.0/len(POCs)))
    for col, POC in enumerate(POCs):
        POCIdx = POC["Idx"]
        
        VisModes = draw_CU_Modes(POC)
        VisModes = draw_MV_Vectors(POC, VisModes)
        
        VisResi = get_FINAL_Buffer(POC, "Y_Resi")
        
        Gallery.subplot(0, col, VisModes,  "POC%d: CU Modes" % POCIdx )
        Gallery.subplot(1, col, VisResi,  "POC%d: Luminance Residual" % POCIdx, cmap="jet")
        Gallery.set_Limits(1, col, *ResiLimits)  
        
        # CONSTRUCT AN EPIC LEGEND
        Colors = IntraColor, InterColor, MergeColor, SkipColor
        Labels = "Intra", "Inter", "Merge", "Skip"
        create_Epic_Legend(Gallery.axarr[0, col], Colors, Labels, bbox=(1.02, 1.1))
            
    print "Saving figure to: ", FN
    Gallery.fig.tight_layout()
    makeSpaceForSubtitle()
    plt.savefig(FN,dpi=DPI)
    close_All_Figures()
    del Gallery  

#==========================================
def main():
#==========================================
    
    Name = "LMS_Test"
    HDT_InputDir =   KitTmpDir    
    createDir(KitTmpDir)
    
    # ===============================
    # FINAL FIGURES   
    # ===============================
    
    #POC_Idcs = 1,2, #3,4
    POC_Idcs = 1,2,3,4,5
    
    for POCIdx in range(1,5):
        POC_Idcs = []
        POC_Idcs.append(POCIdx)
        #POC_Idcs.append(1); POC_Idcs.append(2)
        POCs = NEW_unpickle_POCs(HDT_InputDir, POC_Idcs)
    
        if 0: #RUNS!
            Desc = "Phoenix"
            plotMULTIPOC_CU_Modes_Lum_Resi(POCs, Desc)
    
                
        if 0: #RUNS!
            Desc = "Phoenix"
            plotMULTIPOC_SideInfoBars(POCs, Desc)
            
        if 1: #RUNS!
            Desc = "Phoenix"
            for POC in POCs:
                plot_RDO_Details(POC, Desc)
                #plot_Bit_Consumption(POC, Desc)
                #plot_Overview(POC, Desc)
        
    #compare_All_POCs(RunDirs, Descs, (1,2))
    if 0:
        POC_Idcs = (1,2,3,4,5,6,7,8,9)
        #POC_Idcs = (1,2)
        for POCIdx in POC_Idcs:
            print "Comparing single POC: ", POCIdx
            ep_POC = load_POC(RunDirs[0], POCIdx) 
            analyze_Single_POC(ep_POC, Descs[0])
            
            #anc_POC = load_POC(RunDirs[1], POCIdx)
            #analyze_Single_POC(anc_POC, Descs[1])
            #compare_Single_POCs(ep_POC, anc_POC, ep_Desc, anc_Desc)
    
    #show reference list
    if 0: #TBD, GET TO RUN AGAIN, WILL VISUALIZE REFERENCE FRAME LIST!
        for POCIdx in (1,2,3,4,5,6,7,8,9):
        #for POCIdx in (4,5,6,7,): #DEBUG
            printLARGE("POC%d: Mining reference list frames" % POCIdx)
            TestDir = "/home/dom/Desktop/Jetson_TK1/HARP_Kit/tmp/REFBUF_GWRF/"
            __builtins__.KitTmpDir = "/home/dom/Desktop/Jetson_TK1/HARP_Kit/tmp/REFBUF_GWRF_RESULTS/"
            POC = load_POC(TestDir, POCIdx)
            assert POCIdx == POC["Idx"]
            
            createDir(KitTmpDir)
            
            show_Object(POC)
            Gallery  = PlotGallery(3,4, DimX=A3q_x, DimY=A3q_y) 
            
            Img = POC["YuvOrg"]["Y"]
            Gallery.subplot(0, 0, Img, "Current frame", isImage=True)
            
            # REF FRAMES ORG
            Buffer = "YuvRec"
            for cnt, RefFr in enumerate(POC["MODIFIED_List0_Refs"]):
                Img = Y_2_BGR(RefFr[Buffer]["Y"].astype(np.uint8))
                Str = "POC:%d" % RefFr["Idx"]
                drawString(Img, Str, (20, 40), Color=(0, 255, 0))
                Gallery.subplot(1, cnt, Img, "RefFr at RefPos%d" %  RefFr["RefPos"], isImage=True)
            
            
            # GALLERY SAVE&DEL    
            Gallery.fig.tight_layout()
            FN = combine(KitTmpDir+"/", "POC%05d" % POC["Idx"], "RPS.jpg")
            
            plt.savefig(FN)
            copy_to_SubDir(FN, "RefFrameLists")
            #copy_to_PlotExamples(FN) 
            del Gallery  

    
#==========================================
if __name__ == '__main__':
#==========================================
    #sys.exit(0)    
    StartTime = get_Start_Time()
    main()
    print_Seconds(since(StartTime))

 