#!/usr/bin/env python
# coding: utf8
# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)

from Imports_Basic import *
from OpenCV import *
from Regex import *
from PIL import Image

#==============================================================
# SWISS KNIFE
#==============================================================
if 0:
    
    # LIST SEQUENCES
    #===============================
    SequenceFNs = []
    SequenceFNs.append( "1p5fps_Basel2D_800x480.yuv" )
    SequenceFNs.append( "1p5fps_Wilmersdorf2D_800x480.yuv" )

    # ========================
    # LOAD SEQUENCE
    # ========================
    Sequ = Sequence(SequenceFN, Fr=-1, FPS=30)
    Sequ.load_Settings()
    STAR_Mask = Sequ.GT_Mask if MaskType == "GT-Mask" else Sequ.NO_Mask
    save_Mask(STAR_Mask, CoreTmpDir + "/StarMask.png")
    for POCIdx in np.arange(0, Sequ.Fr):
        CurYUV = Sequ.getPOC(POCIdx)
        RefYUV = Sequ.getPOC(POCIdx-1)
 
    # ========================
    # GET GROUNDTRUTH!
    # ========================       
    GT_H  = Sequ.get_Motion(POCIdx, "GT_H")
    GT_Mask = Sequ.get_Mask(POCIdx, "GT_Mask")
     
#==========================================================================================================================
#====================================================  INTERNAL ===========================================================
#==========================================================================================================================

#========================================== 
def find_Sequence(FN):
#========================================== 
    UsualSuspects = ( #res("/media/Daten/AUDI_SET8_yuv"), 
#                      res("/media/Daten/AUDI_SET10_yuv"),
#                      res("/media/Daten/AUDI_SET10_yuv/LFR"))
                      res("/home/dom/Desktop/Jetson_TK1/AUDI_SET10_yuv"),
                      res("/home/dom/Desktop/Jetson_TK1/AUDI_SET10_yuv/LFR"))
                      #res("/media/Daten/AUDI_SET10_yuv/lowfps")),
                      #res(KitDir + "/DEV/YUVs"),
                      #res(KitDir + "/Various"))
    
    if FN.startswith("/"): #absolute path
        assertFileExists(FN)
        return FN
    
    for Dir in UsualSuspects:
        FoundFiles = globFiles(Dir, FN)
        #print Dir
        #print FoundFiles
        if len(FoundFiles) > 0:
            assert len(FoundFiles) == 1   
            return FoundFiles[0]
        
    assert(0), "No such sequence found: %s" % FN


#use as: Sequ = Sequence(FN_YUV, Fr=-1, FPS=30)
#|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
class Sequence:
#|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
    
    #==========================================
    def __init__(self, SequName, Fr=1, FPS=30):
    #==========================================
    
        #NOTE: fps not used so far
        self.SequPFN = find_Sequence(SequName)
    
        assert os.path.isfile(self.SequPFN), "YUV file not found: " + self.SequPFN     
        (self.FN, self.BN, self.Dir, self.DIRBN, self.AbsFN) = splitPathFN(self.SequPFN )
        self.Name = self.BN
        (self.DimX, self.DimY) = get_DimX_DimY_from_Filename(self.FN)
        (self.Fr, self.FPS, self.FrameSize) = (Fr, FPS, self.DimX*self.DimY*1.5)
        
        
        self.MetainfoDir  = self.Dir + "/" + self.BN
        self.MotionDir = self.Dir + "/" + self.BN + "/Motion"
        self.MaskDir   = self.Dir + "/" + self.BN + "/Mask"
        self.load_Metainfo()
#         self.load_MotionGT()
        
        #self.Alias = get_Alias_From_FN(self.FN)
        if self.Metainfo == None:
            self.Alias = self.BN
        else:
            self.Alias = self.Metainfo.Alias  #CAREFULL, BREAKS CURRENT MEASUREMENT, RERUN!
        
        self.FH = open(self.SequPFN, "rb")
        self.FH.seek(0, os.SEEK_END)
        self.NumBytes = self.FH.tell()
        self.FH.seek(0, os.SEEK_SET)
        self.FrameSize = (self.DimX*self.DimY*1.5)
        self.TotalNumFr = int(self.NumBytes/self.FrameSize)
        if self.Fr > self.TotalNumFr: # if we ask for more than we can get
            self.Fr = self.TotalNumFr
        if self.Fr == -1: # if we want all
            self.Fr = self.TotalNumFr
    
        #self.GT_Mask, self.NO_Mask = self.getMask_GT_and_NO()


    #==========================================
    def load_Metainfo(self): 
    #==========================================     
    
        FN = self.MetainfoDir + "/Metainfo.py"
        if os.path.isfile(FN):
            print "Loaded metainfo from: ", FN
            self.Metainfo = loadModule(FN, ModuleName='Metainfo')
        else:
            print "WARNING: No Metainfo file found: ", FN
            self.Metainfo = None
        return             

    #OLD
#     #==========================================
#     def load_MotionGT(self):
#     #==========================================  
# 
#         FN = self.MotionGT_Dir + "/Motion_GT.pkl"
#         if os.path.isfile(FN):
#             self.MotionGT = load_Pickle_FN(FN, silent=True)
#             print "Sequ: MotionGT loaded"
#         else:
#             #assert 0, "Missing MotionGT: " + FN
#             print "WARNING: No Metainfo file found: ", FN
#             self.MotionGT = None
#         return           

    #OLD
#     #==========================================
#     def get_GT_H(self, POCIdx):
#     #==========================================  
#         GT_H = self.MotionGT["POCs"][POCIdx]["H_gt"]
#         assert POCIdx == self.MotionGT["POCs"][POCIdx]["POCIdx"] #shame on you if this fires
#         print "GT_H loaded for POC ", POCIdx
#         return GT_H
    
    #==========================================
    def get_Motion(self, POCIdx, MotionType):
    #==========================================
        if MotionType == "GT_H":
            FN = self.MotionDir + "/GT_H__POC%03d.cpkl" % POCIdx 
            assertFileExists(FN)
            Motion = LOAD(FN)
            if POCIdx == 0:
                Motion = None # GT-Gen workaround
                
            if self.Alias == "Wilmers2D" and POCIdx == 149:
                Motion = get_Eye_Homography() # ASIFT estimation failure, no chance
            if self.Alias == "LFR-Wilmers2D" and POCIdx == 15:
                Motion = get_Eye_Homography() # ASIFT estimation failure, no chance
                    
        else: assert 0, "MotionType unknown: " + MotionType
            
        #print "Loaded motion for MotionType=", MotionType
        return Motion
 
    #==========================================
    def get_GT_H(self, POCIdx):
    #==========================================   
        GT_H = self.get_Motion(POCIdx, "GT_H")
        return GT_H
    
    #==========================================
    def get_GT_SIGMO(self):
    #==========================================   
        Hand_SigMo_POCIdcs = np.asarray(self.Metainfo.POCS_SigMo)
        tmp = self.get_POCIdcs()
        tmp = tmp.astype(np.float)
        tmp[:] = 0
        tmp[0] = None
        tmp[Hand_SigMo_POCIdcs] = 1
        GT_isSigMo = tmp
        return GT_isSigMo
    
    #==========================================
    def get_GT_Mask(self, POCIdx):
    #==========================================
        FN = (self.DIRBN + "/Mask/GT_Mask.png")
        assertFileExists(FN)
        Mask = readImg(FN) / 255    
        return Mask  

    #==========================================
    def get_BGS_Mask(self, POCIdx, BGS, Flag, Fill):
    #==========================================        
        assert Fill in ("Unfilled", "Filled")
        
        if BGS in ("GT-BGS", "GT_Mask"):
            Mask = self.get_GT_Mask(POCIdx)         
        elif BGS in ("No-BGS", "No-Mask"):
            Mask = np.ones((self.DimY, self.DimX, 3), np.uint8)
        else:               
            Context = "%s_%s_%s" % (Flag, Fill, BGS)
            FN = self.MaskDir + "/" + "Final_Mask_POC%03d.png" % POCIdx
            assertFileExists(FN)
            Mask = readImg(FN) / 255   
#             #Context = "%s_%s_%s" % (Flag, Fill, BGS)
#             #CurDir =  self.MaskDir + "/" + Context
#             FN = self.MaskDir + "Final_Mask_POC%03d.png" % POCIdx
#             BGS_RES_FN = CurDir +"/BGS_RES.cpkl"
#             BGS_RES = LOAD(BGS_RES_FN)
            
        print "Mask %s (POC=%s) loaded" % (BGS, POCIdx)
        return Mask
        
   
    #==========================================
    def getPOC_Y(self, POCIdx):
    #==========================================    
        last_pos = self.FH.tell() 
        self.FH.seek(POCIdx*self.FrameSize, os.SEEK_SET)
        
        Raw = self.FH.read(self.DimX*self.DimY)
        Y = Image.frombuffer('L', (self.DimX,self.DimY), Raw, 'raw', 'L', 0, 1)
        Cb = self.FH.read(self.DimX*self.DimY/4)
        Cr = self.FH.read(self.DimX*self.DimY/4)
        Y = np.array(Y)
        
        self.FH.seek(last_pos, os.SEEK_SET)
        return Y

    #==========================================
    def get_POCIdcs(self): # GET YUV
    #==========================================       
        POCIdcs = np.arange(0, self.Fr)
        return POCIdcs
    
    #==========================================
    def getPOC(self, POCIdx): # GET YUV
    #==========================================   
        last_pos = self.FH.tell() # for savety, we save the current handle (still necessary?)
        
        self.FH.seek(POCIdx*self.FrameSize, os.SEEK_SET)
        Y_Raw = self.FH.read(self.DimX*self.DimY)
        U_Raw = self.FH.read(self.DimX*self.DimY/4)
        V_Raw = self.FH.read(self.DimX*self.DimY/4)
        Y = np.array(Image.frombuffer('L', (self.DimX,self.DimY),     Y_Raw, 'raw', 'L', 0, 1) )
        U = np.array(Image.frombuffer('L', (self.DimX/2,self.DimY/2), U_Raw, 'raw', 'L', 0, 1) )
        V = np.array(Image.frombuffer('L', (self.DimX/2,self.DimY/2), V_Raw, 'raw', 'L', 0, 1) )
        U = cv2.resize(U, (self.DimX, self.DimY), interpolation=cv2.INTER_CUBIC)
        V = cv2.resize(V, (self.DimX, self.DimY), interpolation=cv2.INTER_CUBIC)
        YUV = np.dstack((Y, U, V))
        
        self.FH.seek(last_pos, os.SEEK_SET) # restore the current handle
        return YUV   

    #==========================================
    def getPOC_BGR(self, POCIdx): # GET YUV
    #==========================================
        YUV = self.getPOC(POCIdx)
        
        #BUG! Superstrange! COLOR_YUV2BGR seems to be produce RGB channels!!
        #BGR = cv2.cvtColor(YUV, cv2.COLOR_YUV2BGR) 
        
        # WORKAROUND:
        BGR = cv2.cvtColor(YUV, cv2.COLOR_YUV2RGB) 
        return BGR
    
    #==========================================
    def savePOC(self, POCIdx, FN):  #assuming BGR image
    #==========================================
        POC = self.getPOC_BGR(POCIdx)
        saveBGR(POC, FN)    
    
    #==========================================
    def getSAD(self, POC):
    #==========================================
        RefY  = self.getPOC_Y(POC-1) 
        CurY  = self.getPOC_Y(POC) 
        SAD = np.sum(np.abs(np.int16(CurY) - np.int16(RefY))) / self.DimX / self.DimY 
        return SAD

# OBSOLETE, SEE METAINFO.PY
#     #==========================================
#     def load_Settings(self): 
#     #==========================================     
#     
#         FN = (self.DIRBN + "/" + self.BN + "_Settings.py")
#         if os.path.isfile(FN):
#             import imp
#             TmpModule = imp.load_source('settings', FN)
#             self.settings = TmpModule.settings
#         else:
#             print "WARNING: No Settings file found: ", FN
#             self.settings = None
#         return 

#-    
    #==========================================
    def get_LA_GT(self):  # Groundtruth Labels
    #==========================================    
        assert(0)

    #OLD
#         INI = load_INI(self.DIRBN + ".ini" )
#         Line1 = INI.get("HandAnnotated", "RotatingPOCS").split(",")
#         if Line1 != "":
#             RotatingPOCS_GT = [int(Entry) for Entry in ]
#         ZoomingPOCS_GT  = [int(Entry) for Entry in INI.get("HandAnnotated", "ZoomingPOCS").split(",")]
#         
#         LA_GT = np.zeros(self.Fr, np.uint8)
#         for POC in np.arange(self.Fr):
#             if POC in RotatingPOCS_GT:
#                 LA_GT[POC] = ROT
#             if POC in ZoomingPOCS_GT:
#                 LA_GT[POC] = ZOOM
#         return LA_GT
#-    
    #==========================================
    def get_FE_GT(self):  # Groundtruth Features (e.g. angles, ratios)
    #==========================================     
        INI = loadINI(self.DIRBN + ".ini" )
        RotationAngles_GT = [float(Entry) for Entry in INI.get("RotationAngles")]
        EigenvalRatios_GT = [float(Entry) for Entry in INI.get("EigenvalRatios")]
        
        RotationAngles_GT=np.array(RotationAngles_GT)[:self.Fr]
        EigenvalRatios_GT=np.array(EigenvalRatios_GT)[:self.Fr]
        
#         FE_GT = np.zeros((self.Fr,2), np.float32)
#         FE_GT[:,0] = np.array(RotationAngles_GT)
#         FE_GT[:,1] = np.array(EigenvalRatios_GT)
        return (RotationAngles_GT, EigenvalRatios_GT)
    

            
    #==========================================
    def OLDgetMask(self):
    #==========================================
        FN1 = (self.DIRBN + "/" + self.BN + "_Mask.png")  # GET RID OF THIS DUDE
        FN2 = (self.DIRBN + "/STAR_Mask.png")
        #assertFileExists(FN)
        
        if os.path.isfile(FN1):
            GtMask = readImg(FN1) / 255
        elif os.path.isfile(FN2):
            GtMask = readImg(FN2) / 255
        else:
            print "WARNING: MASK NOT FOUND, SETTING TO NONE!!"
            assert(0), "MASK NOT FOUND"
            GtMask = None
        
        return GtMask  
        
        #GtMask = np.zeros((self.DimY, self.DimX), np.uint8)

    #==========================================
    def OLDgetMask_GT_and_NO(self):
    #==========================================
        GT_Mask = self.getMask()
        NO_Mask = GT_Mask.copy()
        NO_Mask.fill(1)
        return GT_Mask, NO_Mask

    #==========================================
    def OLDgetMaskFN(self):
    #==========================================
        FN1 = (self.DIRBN + "/" + self.BN + "_Mask.png")  # GET RID OF THIS DUDE
        FN2 = (self.DIRBN + "/STAR_Mask.png")
        
        if os.path.isfile(FN1):
            FoundFN = FN1
        elif os.path.isfile(FN2):
            FoundFN = FN2
        else:
            assert(0)
        return FoundFN

#==========================================
def write_YUV(FH, Cur):
#==========================================
#     for cnt in range(0,6):
#         self.writeText(Cur, (0,10+64*cnt), "%d" % POC)
    
    y,u,v = cv2.split(Cur)
    FH.write(np.frombuffer(np.getbuffer(y), dtype=np.uint8));
    u = cv2.resize(u, (0, 0), fx=0.5, fy=0.5, interpolation=cv2.INTER_CUBIC)
    v = cv2.resize(v, (0, 0), fx=0.5, fy=0.5, interpolation=cv2.INTER_CUBIC)
    FH.write(np.frombuffer(np.getbuffer(u), dtype=np.uint8))
    FH.write(np.frombuffer(np.getbuffer(v), dtype=np.uint8))  
    #print "YUV file written to file"
    
# EXAMPLE:
# resizeToHD(Sequ, "tmp/" + Sequ.BN + "_HDresized.yuv")
#==========================================
def resizeToHD(Sequ, Target_FN):
#==========================================

    YuvReader = CYuvReader(Sequ.AbsPath, Sequ.DimX, Sequ.DimY)

    # FOR ALL FRAMES --------------------------------------
    for POC in range(0,Sequ.Fr):
    #------------------------------------------------------  
        print "Resizing POC %d to HD, sharpening..." % POC
        (Y, U, V) = YuvReader.read_YUV()
        
        YUV = np.dstack((Y, U, V))
        RGB = cv2.cvtColor(YUV, cv2.COLOR_YUV2RGB)
        
        RGB = cv2.resize(RGB, (1920, 1080), interpolation=cv2.INTER_CUBIC)
        
        Sharpened = cv2.GaussianBlur(RGB, (0, 0), 3)
        Sharpened = addWeighted(RGB, 1.5, Sharpened, -0.5, 0, Sharpened);
        #cv2.imwrite("tmp/TestHD.png", Sharpened)
        
        YUV = cv2.cvtColor(Sharpened, cv2.COLOR_RGB2YUV)
        YuvReader.write_YUV(Target_FN, YUV)
        
        a = 1
    
    YuvReader.close()
 

 
# EXAMPLE   
# createSequFromPNGs("/home/dom/Desktop/Version Michael/Denver3DHD_MZ", "Denver3DHD_MZ", "tmp/test", 200)
#==========================================
def createSequFromPNGs(PNGDir, PNG_BN, Target_DIRBN, NumFrames):
#==========================================
    #WARNING! INVERTED ORDER!

    FN = PNGDir + "/%s%04d.png" % (PNG_BN, 0)
    Img = cv2.imread(FN)
    DimY, DimX = Img.shape[:2]

    Target_FN = Target_DIRBN + "_%dx%d.yuv" % (DimX, DimY)
    FH = open(Target_FN, "wb")

    # FOR ALL FRAMES --------------------------------------
    #for POC in range(0,NumFrames):
    for POC in range(NumFrames-1,-1,-1):
    #------------------------------------------------------  
        FN = PNGDir + "/%s%04d.png" % (PNG_BN, POC)
        assert os.path.isfile(FN), "File not found: " + FN

        Img = cv2.imread(FN)
        Img = cv2.cvtColor(Img, cv2.COLOR_RGB2YUV)
        y,u,v = cv2.split(Img)
        
        FH.write(np.frombuffer(np.getbuffer(y), dtype=np.uint8));
        u = cv2.resize(u, (0, 0), fx=0.5, fy=0.5, interpolation=INTER_CUBIC)
        v = cv2.resize(v, (0, 0), fx=0.5, fy=0.5, interpolation=INTER_CUBIC)
        FH.write(np.frombuffer(np.getbuffer(u), dtype=np.uint8))
        FH.write(np.frombuffer(np.getbuffer(v), dtype=np.uint8))  
        print "%s written to %s" % (FN, Target_FN)
    
    FH.close()


#====================================================================================
class CYuvReader():
#====================================================================================

    #==========================================
    def __init__(self, Filename, DimX, DimY):
    #==========================================
        self.Filename = Filename
        self.DimX = DimX
        self.DimY = DimY
        
        self.FH = open(Filename, "rb")

        self.scaleX = 1 
        self.scaleY = self.scaleX
        
        self.FH.seek(0, os.SEEK_END)
        self.NumBytes = self.FH.tell()
        self.FH.seek(0, os.SEEK_SET)
        
        self.FrameSize = (self.DimX*self.DimY*1.5)

    #==========================================
    def read_Y(self):
    #==========================================
        #print "FTell: %d of %d" % (self.FH.tell(), self.NumBytes)
        Raw = self.FH.read(self.DimX*self.DimY)
        Y = Image.frombuffer('L', (self.DimX,self.DimY), Raw, 'raw', 'L', 0, 1)
        #print "Size raw:", len(Raw)
        #image.save("test.jpg")
        
        Cb = self.FH.read(self.DimX*self.DimY/4)
        Cr = self.FH.read(self.DimX*self.DimY/4)
        Y = np.array(Y)
        Y = cv2.resize(Y, (0, 0), fx=self.scaleX , fy=self.scaleY, interpolation=cv2.INTER_LINEAR)
        
        #cv2.imwrite("Img.jpg", Img)
        return  Y
    
    #==========================================
    def read_YUV(self):
    #==========================================
        #print "FTell: %d of %d" % (self.FH.tell(), self.NumBytes)
        Y_Raw = self.FH.read(self.DimX*self.DimY)
        U_Raw = self.FH.read(self.DimX*self.DimY/4)
        V_Raw = self.FH.read(self.DimX*self.DimY/4)
        
        Y = np.array(Image.frombuffer('L', (self.DimX,self.DimY),     Y_Raw, 'raw', 'L', 0, 1) )
        U = np.array(Image.frombuffer('L', (self.DimX/2,self.DimY/2), U_Raw, 'raw', 'L', 0, 1) )
        V = np.array(Image.frombuffer('L', (self.DimX/2,self.DimY/2), V_Raw, 'raw', 'L', 0, 1) )
        
        U = cv2.resize(U, (self.DimX, self.DimY), interpolation=cv2.INTER_CUBIC)
        V = cv2.resize(V, (self.DimX, self.DimY), interpolation=cv2.INTER_CUBIC)
        
        return  (Y, U, V)
    
    #==========================================
    def readPOC_Y(self, POC):
    #==========================================    
        last_pos = self.FH.tell() 
        self.FH.seek(POC*self.FrameSize, os.SEEK_SET)
        Y = self.read_Y()
        self.FH.seek(last_pos, os.SEEK_SET)
        return Y
        
    #==========================================
    def write_YUV(self, FN, YUV):
    #==========================================        
        FH = open(FN, "ab")
        y,u,v = cv2.split(YUV)
        FH.write(np.frombuffer(np.getbuffer(y), dtype=np.uint8));
        u = cv2.resize(u, (0, 0), fx=0.5, fy=0.5, interpolation=INTER_CUBIC)
        v = cv2.resize(v, (0, 0), fx=0.5, fy=0.5, interpolation=INTER_CUBIC)
        FH.write(np.frombuffer(np.getbuffer(u), dtype=np.uint8))
        FH.write(np.frombuffer(np.getbuffer(v), dtype=np.uint8))  
        print "YUV file written to file"
        FH.close()
    
    #==========================================
    def getNumFrames(self):
    #==========================================   
        return int(self.NumBytes/self.FrameSize)
    
    #==========================================
    def getResolution(self):
    #==========================================
        return (self.DimX*self.scaleX, self.DimY*self.scaleX)
    
    #==========================================
    def close(self):
    #==========================================
        self.FH.close()

#==========================================================================================================================
#==================================================== VARIOUS   ===========================================================
#==========================================================================================================================

# OLD, NO LONGER USED!

#========================================== 
def get_Sequ_GT(InputYUV):
#========================================== 
    assert 0, "DEPRECATED, REPLACE BY Sequ.get_GT_H()"
    
    #(FN, BN, Dir, DIRBN, PathFN) = splitPathFN( INI_load_STR("Tmp", "FN_InputYUV"))
    
    (FN, BN, Dir, DIRBN, PathFN) = splitPathFN(InputYUV)
    PKL_FN = Dir +  "/" + BN + "/" + BN + ".pkl"
    #print "FN_InputYUV:" , INI_load_STR("Tmp", "FN_InputYUV")
    #print PKL_FN
    print "Trying to load Sequ_GT from", PKL_FN
    if os.path.isfile(PKL_FN):
        Sequ_GT = load_Pickle_FN(PKL_FN, silent=True)
        print "Success. GT found"
    else:
        Sequ_GT = None
        #print "Failed... GT not found."
        assert 0, "Failed... GT not found:" + PKL_FN
    return Sequ_GT

#========================================== 
def get_Sequ_ESTH(EncoderLUT_Dir, InputYUV, ID_Suffix):
#========================================== 

    assert 0, "DEPRECATED, REPLACE BY Sequ.get_GT_H()"

    #(FN, BN, Dir, DIRBN, PathFN) = splitPathFN( INI_load_STR("Tmp", "FN_InputYUV"))
    
    (FN, BN, Dir, DIRBN, PathFN) = splitPathFN(InputYUV)
    PKL_FN = Dir +  "/" + BN + "/" + BN + "_" + ID_Suffix + ".pkl"
    #print "FN_InputYUV:" , INI_load_STR("Tmp", "FN_InputYUV")
    #print PKL_FN
    print "Trying to load Sequ_ESTH from", PKL_FN
    if os.path.isfile(PKL_FN):
        ESTH_GT = load_Pickle_FN(PKL_FN, silent=True)
        print "Success. ESTH found"
    else:
        Sequ_GT = None
        #print "Failed... GT not found."
        assert 0, "Failed... GT not found:" + PKL_FN
    return ESTH_GT

# call: reduce_Framerates_ALL(AllYUVsinDir = "/media/Daten/AUDI_SET10_yuv")
#========================================== 
def reduce_Framerates_ALL(AllYUVsinDir = "/media/Daten/AUDI_SET10_yuv"):
#========================================== 
    FNs = glob.glob(AllYUVsinDir + "/*.yuv")
    
    OutDir = AllYUVsinDir + "/LFR"
    createDir(OutDir)

    Fr=-1
    FPS = 30
    EveryXthFrame = 10
    for FN in FNs:
        if "Basel2D" not in FN \
        and "Basel3D" not in FN \
        and "Wilmersdorf2D" not in FN \
        and "Versailles3D" not in FN:
            continue
        printLARGE("REDUCING FRAMERATE OF " + getPureFN(FN))
        OutYUV_FN = OutDir + "/LFR-" + getPureFN(FN)
        YUV_FileH = open(OutYUV_FN, "wb")
        
        # GENERATING YUV
        Sequ = Sequence(FN, Fr, FPS)
        for idx in range(0, Sequ.TotalNumFr):
            if idx % EveryXthFrame == 0:
                CurYUV = Sequ.getPOC(idx)
                write_YUV(YUV_FileH, CurYUV)
                print "Written YUV frame %d to file %s" % (idx, OutYUV_FN)
                
        # COPYING MASK
        
        Mask_FN = Sequ.getMaskFN()
        MetainfoFolder = getPureDIR(OutYUV_FN) + "/" + getPureFN_NoSuffix(OutYUV_FN)
        createDir(MetainfoFolder)
        copyFile2Dir(Mask_FN, MetainfoFolder + "/STAR_Mask.png")

# call: create_Example_Screenshots_ALL(AllYUVsinDir = "/media/Daten/AUDI_SET10_yuv")  
#========================================== 
def create_Example_Screenshots_ALL(AllYUVsinDir = "/media/Daten/AUDI_SET10_yuv"):
#========================================== 
    FNs = glob.glob(AllYUVsinDir + "/*.yuv")
    
    OutDir = AllYUVsinDir
    createDir(OutDir)

    Fr=-1
    FPS = 30
    EveryXthFrame = 20
    for YUV_FN in FNs:
        printLARGE("Creating screenshots for " + getPureFN(YUV_FN))
        
        # GENERATING YUV
        Sequ = Sequence(YUV_FN, Fr, FPS)
        for idx in range(0, Sequ.TotalNumFr):
            if idx % EveryXthFrame == 0:
                PNG_FN = getPureDIR(YUV_FN) + "/" + getPureFN_NoSuffix(YUV_FN) + "/" + getPureFN_NoSuffix(YUV_FN) + "_POC%04d.png" % idx
                CurYUV = Sequ.getPOC(idx)
                saveYUV(CurYUV, PNG_FN)
                print "Written PNG: ", PNG_FN

#========================================== 
def get_PNGs(Variant_SEQU): #used for first version of mask estimation
#========================================== 
        PNGs = glob.glob(Variant_SEQU + "/*.png")
        PNGs = [PNG for PNG in PNGs if "_002_" not in PNG]
        PNGs.sort()
        if Debug:
            PNGs = PNGs[0:NumFrames]
            #PNGs = PNGs[465:475+90]
            #PNGS = [ PNGs[index] for index in [26,27,467,468,469] ] #manual selection   
        return PNGs