// Copyright (c) 2010-2014, ITU/ISO/IEC
// License: see Src_HM/source/Lib/TLibEncoder/TEncSearch.cpp

// next_predInterSearch is taken from TEncSearch.cpp
// It is only active if "ACTIVATE_NEXT_RDO" is defined

#include "PyFactory.h"
#include "HARP_Defines.h"
#include "CCallgrind.h"
#include "CLibGlobal.h"
#include "CHelper.h"

#if AMP_MRG
Void TEncSearch::predInterSearch( TComDataCU* pcCU, TComYuv* pcOrgYuv, TComYuv*& rpcPredYuv, TComYuv*& rpcResiYuv, TComYuv*& rpcRecoYuv, Bool bUseRes, Bool bUseMRG )
#else
Void TEncSearch::predInterSearch( TComDataCU* pcCU, TComYuv* pcOrgYuv, TComYuv*& rpcPredYuv, TComYuv*& rpcResiYuv, TComYuv*& rpcRecoYuv, Bool bUseRes )
#endif
{
#ifdef ACTIVATE_HARP
  if(loadINI("Visualization", "VisRDO"))
	LibGlobal.Show_RDO.start_predInterSearch(pcCU, pcOrgYuv, rpcPredYuv, rpcResiYuv, rpcRecoYuv);
#endif

  m_acYuvPred[0].clear();
  m_acYuvPred[1].clear();
  m_cYuvPredTemp.clear();
  rpcPredYuv->clear();
  
  if ( !bUseRes )
  {
    rpcResiYuv->clear();
  }
  
  rpcRecoYuv->clear();
  
  TComMv        cMvSrchRngLT;
  TComMv        cMvSrchRngRB;
  
  TComMv        cMvZero;
  TComMv        TempMv; //kolya
  
  TComMv        cMv[2];
  TComMv        cMvBi[2];
  TComMv        cMvTemp[2][33];
  
  Int           iNumPart    = pcCU->getNumPartitions();
  Int           iNumPredDir = pcCU->getSlice()->isInterP() ? 1 : 2;
  
  TComMv        cMvPred[2][33];
  
  TComMv        cMvPredBi[2][33];
  Int           aaiMvpIdxBi[2][33];
  
  Int           aaiMvpIdx[2][33];
  Int           aaiMvpNum[2][33];
  
  AMVPInfo aacAMVPInfo[2][33];
  
  Int           iRefIdx[2]={0,0}; //If un-initialized, may cause SEGV in bi-directional prediction iterative stage.
  Int           iRefIdxBi[2];
  
  UInt          uiPartAddr;
  Int           iRoiWidth, iRoiHeight;
  
  UInt          uiMbBits[3] = {1, 1, 0};
  
  UInt          uiLastMode = 0;
  Int           iRefStart, iRefEnd;
  
  PartSize      ePartSize = pcCU->getPartitionSize( 0 );

  Int           bestBiPRefIdxL1 = 0;
  Int           bestBiPMvpL1 = 0;
  UInt          biPDistTemp = MAX_INT;

#if ZERO_MVD_EST
  Int           aiZeroMvdMvpIdx[2] = {-1, -1};
  Int           aiZeroMvdRefIdx[2] = {0, 0};
  Int           iZeroMvdDir = -1;
#endif

  TComMvField cMvFieldNeighbours[MRG_MAX_NUM_CANDS << 1]; // double length for mv of both lists
  UChar uhInterDirNeighbours[MRG_MAX_NUM_CANDS];
  Int numValidMergeCand = 0 ;

  //----------------------------------------------------
  //  HARP_SETTINGS.INI
  //----------------------------------------------------
  bool disableMerge = pyf_INI_load_INT("Experimental", "disableMerge");
  bool Only_2Nx2N = pyf_INI_load_INT("Experimental", "Only_2Nx2N");

  //----------------------------------------------------
  //  FOR EACH PU IN THIS CU
  //----------------------------------------------------
  for ( Int iPartIdx = 0; iPartIdx < iNumPart; iPartIdx++ )
  {
	#ifdef ACTIVATE_HARP
	  if(loadINI("Visualization", "VisRDO"))
		LibGlobal.Show_RDO.append1_predInterSearch(pcCU, pcOrgYuv, rpcPredYuv, rpcResiYuv, rpcRecoYuv, bUseRes, bUseMRG, iPartIdx);
	#endif

	PyObject *CurPU = LibGlobal.DataTree.create_PU("CUs_RDO", pcCU, iPartIdx, 0, 0);  //0,0: arguments not required for RDO CUs

    UInt          uiCost[2] = { MAX_UINT, MAX_UINT };  //holds the best PU costs for each ref list
    UInt          uiCostBi  =   MAX_UINT;
    UInt          uiCostTemp;
    
    UInt          uiBits[3];
    UInt          uiBitsTemp;
#if ZERO_MVD_EST
    UInt          uiZeroMvdCost = MAX_UINT;
    UInt          uiZeroMvdCostTemp;
    UInt          uiZeroMvdBitsTemp;
    UInt          uiZeroMvdDistTemp = MAX_UINT;
    UInt          auiZeroMvdBits[3];
#endif
    UInt          bestBiPDist = MAX_INT;

    //----------------------------------------------------
    //  STORAGE REQUIRED FOR BI-PREDICTION
    //----------------------------------------------------
    UInt          uiCostTempL0[MAX_NUM_REF];
    for (Int iNumRef=0; iNumRef < MAX_NUM_REF; iNumRef++)
    {
      uiCostTempL0[iNumRef] = MAX_UINT;
    }
    UInt          uiBitsTempL0[MAX_NUM_REF];

    TComMv        mvValidList1;
    Int           refIdxValidList1 = 0;
    UInt          bitsValidList1 = MAX_UINT;
    UInt          costValidList1 = MAX_UINT;

    xGetBlkBits( ePartSize, pcCU->getSlice()->isInterP(), iPartIdx, uiLastMode, uiMbBits);
    
    pcCU->getPartIndexAndSize( iPartIdx, uiPartAddr, iRoiWidth, iRoiHeight );
    
    //----------------------------------------------------
    //  FORCE MERGE CHECK FOR AMP PARTITION (NO MOTION SEARCH FOR AMP)
    //----------------------------------------------------
#if AMP_MRG
    Bool bTestNormalMC = true;
    
    if ( bUseMRG && pcCU->getWidth( 0 ) > 8 && iNumPart == 2 )
    {
      bTestNormalMC = false;
    }
    
    //----------------------------------------------------
    //  GREAT FORTUNE! WE MAY JUST DO NORMAL MOTION COMPENSATION
    //----------------------------------------------------
    if (bTestNormalMC)
    {
#endif

    //----------------------------------------------------
    //  DO MOTION ESTIMATION FOR UNI AND BI PREDICTION
	//  FOR REF LIST 0 (AND, IF B-FRAME, FOR REF LIST 1)
    //----------------------------------------------------
    //  P-Frame: iNumPredDir=1, B-Frame: iNumPredDir=2
    for ( Int iRefList = 0; iRefList < iNumPredDir; iRefList++ )
    {
      assert(iRefList == 0);
      RefPicList  eRefPicList = ( iRefList ? REF_PIC_LIST_1 : REF_PIC_LIST_0 );
      
      //----------------------------------------------------
      //  FOR EACH REF FRAME
      //----------------------------------------------------
      for ( Int iRefIdxTemp = 0; iRefIdxTemp < pcCU->getSlice()->getNumRefIdx(eRefPicList); iRefIdxTemp++ )
      {
        uiBitsTemp = uiMbBits[iRefList];
        //if more than one reference: add 1 to the bits costs (clarify!)
        if ( pcCU->getSlice()->getNumRefIdx(eRefPicList) > 1 )
        {
          uiBitsTemp += iRefIdxTemp+1;
          if ( iRefIdxTemp == pcCU->getSlice()->getNumRefIdx(eRefPicList)-1 ) uiBitsTemp--;
        }
#if ZERO_MVD_EST
        xEstimateMvPredAMVP( pcCU, pcOrgYuv, iPartIdx, eRefPicList, iRefIdxTemp, cMvPred[iRefList][iRefIdxTemp], false, &biPDistTemp, &uiZeroMvdDistTemp);
#else
        xEstimateMvPredAMVP( pcCU, pcOrgYuv, iPartIdx, eRefPicList, iRefIdxTemp, cMvPred[iRefList][iRefIdxTemp], false, &biPDistTemp);
#endif
        aaiMvpIdx[iRefList][iRefIdxTemp] = pcCU->getMVPIdx(eRefPicList, uiPartAddr);
        aaiMvpNum[iRefList][iRefIdxTemp] = pcCU->getMVPNum(eRefPicList, uiPartAddr);
        
        if(pcCU->getSlice()->getMvdL1ZeroFlag() && iRefList==1 && biPDistTemp < bestBiPDist)
        {
          bestBiPDist = biPDistTemp;
          bestBiPMvpL1 = aaiMvpIdx[iRefList][iRefIdxTemp];
          bestBiPRefIdxL1 = iRefIdxTemp;
        }

        uiBitsTemp += m_auiMVPIdxCost[aaiMvpIdx[iRefList][iRefIdxTemp]][AMVP_MAX_NUM_CANDS];
#if ZERO_MVD_EST
        if ( iRefList == 0 || pcCU->getSlice()->getList1IdxToList0Idx( iRefIdxTemp ) < 0 )
        {
          uiZeroMvdBitsTemp = uiBitsTemp;
          uiZeroMvdBitsTemp += 2; //zero mvd bits

          m_pcRdCost->getMotionCost( 1, 0 );
          uiZeroMvdCostTemp = uiZeroMvdDistTemp + m_pcRdCost->getCost(uiZeroMvdBitsTemp);

          if (uiZeroMvdCostTemp < uiZeroMvdCost)
          {
            uiZeroMvdCost = uiZeroMvdCostTemp;
            iZeroMvdDir = iRefList + 1;
            aiZeroMvdRefIdx[iRefList] = iRefIdxTemp;
            aiZeroMvdMvpIdx[iRefList] = aaiMvpIdx[iRefList][iRefIdxTemp];
            auiZeroMvdBits[iRefList] = uiZeroMvdBitsTemp;
          }          
        }
#endif
        
        assert (iRefList == REF_PIC_LIST_0);
        assert (GPB_SIMPLE_UNI);

        //----------------------------------------------------
        //  FOR TRANSLATIONAL ME
        //----------------------------------------------------
        UInt ME_BestCost = MAX_UINT;  //OWN temp container to hold the best costs so far

        //we arrive here with an initial guess of MVPred, CandIdx, and BitsTemp
        //these values get changed by the following, so we need to restore them for every new ME test
        Int    Fresh_CandIdx    = aaiMvpIdx[iRefList][iRefIdxTemp];
        UInt   Fresh_uiBitsTemp = uiBitsTemp;
        TComMv Fresh_cMvPred    = cMvPred[iRefList][iRefIdxTemp];

        {
          UInt _uiMvCost, Costs_HadamardSAD_V1;

          Int     _CandIdx = Fresh_CandIdx;    //holds the MV predictor candidate idx
          TComMv  _cMvPred = Fresh_cMvPred;    //holds the chosen MV predictor (init by result from xEstimateMvPredAMVP())
          UInt _uiBitsTemp = Fresh_uiBitsTemp; //holds the incremented bits (init by bits so far)
          UInt _uiCostTemp = MAX_UINT; //found total cost (no need to init);
          TComMv _cMVTemp;             //found MV


          {

            PyObject *Cur_CHECK_MotionType = LibGlobal.DataTree.create_CHECK_MotionType("CUs_RDO", ME_TRANSL, pcCU, iPartIdx);

            {
              //====================================================
              // TRANSLATIONAL ME
              //====================================================
              //will change: cMvTemp, uiCostTemp
              //will NOT change: uiBitsTemp
              //will internally set the predictor in order to remember the MvPred inside the TComRDCost structure
              xMotionEstimation ( pcCU, pcOrgYuv, iPartIdx, eRefPicList, &_cMvPred, iRefIdxTemp,
                  _cMVTemp, _uiBitsTemp, _uiCostTemp ); //"return values",_uiCostTemp represents: Hadamard costs + getCost(_uiMvBits)

            }

            // copy the candidates (=AMVPInfo) of this CU (LIST0) to the TEncSearch member aacAMVPInfo
            xCheckBestMVP(pcCU, eRefPicList, _cMVTemp,
                _cMvPred, _CandIdx, _uiBitsTemp, _uiCostTemp); //"return values"
            xCopyAMVPInfo(pcCU->getCUMvField(eRefPicList)->getAMVPInfo(), &aacAMVPInfo[iRefList][iRefIdxTemp]); // must always be done ( also when AMVP_MODE = AM_NONE )

            //-----------------------------------
            // FINAL COSTS CALCULATION
            // Inputs: _uiCostTemp, _uiBitsTemp, _cMVTemp
            //-----------------------------------
            // FINAL _uiBitsTemp: MV bits plus bits for refidx and candidate signaling
            // FINAL _uiCostTemp: remove MV prederror costs added by xPatternRefinement, then add the total costs for all side information

            //uses MvPred inside the TComRDCost structure to get the bits for the !!MV DIFF!!

            UInt _uiMvBits 			  = m_pcRdCost->getBits( _cMVTemp.getHor(), _cMVTemp.getVer() );
            assert(m_pcRdCost->m_iCostScale == 0);

            //_uiBitsTemp      			 += _uiMvBits;

            Double fWeight = 1.0; //Bi: 0.5
            Double Costs_MvBits         = (Double) m_pcRdCost->getCost(_uiMvBits);
            Double Costs_Sideinfo       = (Double) m_pcRdCost->getCost( _uiBitsTemp );
            Double Costs_HadSAD = (Double) _uiCostTemp - Costs_MvBits; //this is NOT the same as Costs_HadamardSAD_V1
            Double Costs_Final =  floor( fWeight * ( Costs_HadSAD ) ) + Costs_Sideinfo;

            if (Costs_Final > MAX_UINT)
              Costs_Final = MAX_UINT;

            {
              //---------------------------
              // WE TAKE A DECISION: THE CURRENT MV PRED GUY IS FINAL
              //---------------------------
              ME_BestCost = _uiCostTemp;
              aaiMvpIdx[iRefList][iRefIdxTemp] = _CandIdx;
              cMvTemp[iRefList][iRefIdxTemp] = _cMVTemp;

              //save MVPred for later
              cMvPred[iRefList][iRefIdxTemp] = _cMvPred;
              uiBitsTemp = _uiBitsTemp; //this is best so far -> remember the estimated bits
              uiCostTemp = _uiCostTemp; //this is best so far -> remember the estimated costs
            }


//            // NO LONGER USED!!
//            inline void pyf_appendto_ME(PyObject* poME,  TComMv _cMVTemp, TComMv _cMvPred, Int _CandIdx,
//                UInt _uiMvBits, UInt _uiMvCost, UInt _ruiCost_SADonly, UInt _uiBitsTemp, UInt _uiCostTemp)
//            {
//              assert(0);
//
//              //MV (found)
//              PyObject *PoTuple = pyf_getTuple(_cMVTemp.getHor(), _cMVTemp.getVer());
//              PyDict_SetItem(poME, PyString_FromString("MV_Found"), PoTuple);
//              Py_DECREF(PoTuple);
//
//
//              //MV Predictor
//              PyObject *PoTuple2 = pyf_getTuple(_cMvPred.getHor(), _cMvPred.getVer());
//              PyDict_SetItem(poME, PyString_FromString("MV_Pred"), PoTuple2);
//              Py_DECREF(PoTuple2);
            //Dict_addObject(PoDict,  "MV", pyf_getTuple(Mv.getHor(), Mv.getVer()));

            // add RDO PU Candidate Check MV Info



            LibGlobal.DataTree.appendTo_CHECK_MotionType("CUs_RDO", Cur_CHECK_MotionType, iRefIdxTemp, _CandIdx, _cMVTemp, _cMvPred,
                Costs_MvBits, Costs_Sideinfo, Costs_HadSAD, Costs_Final, uiCostTemp, uiBitsTemp);
          }

        } //END FOR

        if ( iRefList == 0 )
        {
          uiCostTempL0[iRefIdxTemp] = uiCostTemp;
          uiBitsTempL0[iRefIdxTemp] = uiBitsTemp;
        }

        //if found cost for this ref img is cheapest so far
        if ( uiCostTemp < uiCost[iRefList] )
        {
          uiCost[iRefList] = uiCostTemp;
          uiBits[iRefList] = uiBitsTemp; // storing for bi-prediction

          // set motion
          cMv[iRefList]     = cMvTemp[iRefList][iRefIdxTemp];
          iRefIdx[iRefList] = iRefIdxTemp;
        }

        // LIST1 stuff
        if ( iRefList == 1 && uiCostTemp < costValidList1 && pcCU->getSlice()->getList1IdxToList0Idx( iRefIdxTemp ) < 0 )
        {
          assert(0);
          costValidList1 = uiCostTemp;
          bitsValidList1 = uiBitsTemp;

          // set motion
          mvValidList1     = cMvTemp[iRefList][iRefIdxTemp];
          refIdxValidList1 = iRefIdxTemp;
        }
      } //END FOR: each ref frame
    } //END FOR: each ref list

    //  Bi-directional prediction
    if ( (pcCU->getSlice()->isInterB()) && (pcCU->isBipredRestriction(iPartIdx) == false) )
    	assert(0);

#if AMP_MRG
    } //end if bTestNormalMC
#endif

    //----------------------------------------------------
    // WE NOW HAVE TWO OPTIMAL INTER SOLUTIONS FOR THIS CU, ONE FOR LIST 0 AND ONE FOR LIST 1
    // WE PREPARE BY CLEANING THE MV FIELD AND THE MV PREDICTORS
    //----------------------------------------------------

    //  Clear MV Field of this CU (using uiPartAddr for keying into it)
    pcCU->getCUMvField(REF_PIC_LIST_0)->setAllMvField( TComMvField(), ePartSize, uiPartAddr, 0, iPartIdx ); //set to zero: Mv and RefIdx
    pcCU->getCUMvField(REF_PIC_LIST_1)->setAllMvField( TComMvField(), ePartSize, uiPartAddr, 0, iPartIdx );
    pcCU->getCUMvField(REF_PIC_LIST_0)->setAllMvd    ( cMvZero,       ePartSize, uiPartAddr, 0, iPartIdx ); //set to zero: Mvd
    pcCU->getCUMvField(REF_PIC_LIST_1)->setAllMvd    ( cMvZero,       ePartSize, uiPartAddr, 0, iPartIdx );

    // Set MVPIdx's and MVPNum's of all subparts to -1 (for both list 0 and list 1)
    pcCU->setMVPIdxSubParts( -1, REF_PIC_LIST_0, uiPartAddr, iPartIdx, pcCU->getDepth(uiPartAddr));
    pcCU->setMVPNumSubParts( -1, REF_PIC_LIST_0, uiPartAddr, iPartIdx, pcCU->getDepth(uiPartAddr));
    pcCU->setMVPIdxSubParts( -1, REF_PIC_LIST_1, uiPartAddr, iPartIdx, pcCU->getDepth(uiPartAddr));
    pcCU->setMVPNumSubParts( -1, REF_PIC_LIST_1, uiPartAddr, iPartIdx, pcCU->getDepth(uiPartAddr));
    
    UInt uiMEBits = 0;
    // Set Motion Field_
    cMv[1] = mvValidList1;
    iRefIdx[1] = refIdxValidList1;
    uiBits[1] = bitsValidList1;
    uiCost[1] = costValidList1;
#if AMP_MRG
    if (bTestNormalMC)
    {
#endif
    if ( uiCostBi <= uiCost[0] && uiCostBi <= uiCost[1])
      assert(0);

    //----------------------------------------------------
    //  IF LIST0-PREDICTION ROCX: INITIALIZE WITH cMv[0] and iRefIdx[0]
    //----------------------------------------------------
    else if ( uiCost[0] <= uiCost[1] )
    {
      uiLastMode = 0;
      //-------------------------------------
      // MAGIC MOMENT 1: PU CHECK GETS ITS FINAL MV AND REFIDX
      //-------------------------------------
      pcCU->getCUMvField(REF_PIC_LIST_0)->setAllMv( cMv[0], ePartSize, uiPartAddr, 0, iPartIdx );
      pcCU->getCUMvField(REF_PIC_LIST_0)->setAllRefIdx( iRefIdx[0], ePartSize, uiPartAddr, 0, iPartIdx );

      TComMv Test = pcCU->getCUMvField(REF_PIC_LIST_0)->getMv( uiPartAddr );
      //-------------------------------------
      // MAGIC MOMENT 2: PU CHECK GETS ITS FINAL MVD
      //-------------------------------------

      TempMv = cMv[0] - cMvPred[0][iRefIdx[0]];

      pcCU->getCUMvField(REF_PIC_LIST_0)->setAllMvd    ( TempMv,                 ePartSize, uiPartAddr, 0, iPartIdx );
      pcCU->setInterDirSubParts( 1, uiPartAddr, iPartIdx, pcCU->getDepth(0) );
      
      pcCU->setMVPIdxSubParts( aaiMvpIdx[0][iRefIdx[0]], REF_PIC_LIST_0, uiPartAddr, iPartIdx, pcCU->getDepth(uiPartAddr));
      pcCU->setMVPNumSubParts( aaiMvpNum[0][iRefIdx[0]], REF_PIC_LIST_0, uiPartAddr, iPartIdx, pcCU->getDepth(uiPartAddr));

      uiMEBits = uiBits[0];
    }
    else
        assert(0);
#if AMP_MRG
    } // end if bTestNormalMC
#endif

    //if PartSize is anything small: check merging with other partitions
    if ( pcCU->getPartitionSize( uiPartAddr ) != SIZE_2Nx2N )
    {
      UInt uiMRGInterDir = 0;     
      TComMvField cMRGMvField[2];
      UInt uiMRGIndex = 0;

      UInt uiMEInterDir = 0;
      TComMvField cMEMvField[2];

      m_pcRdCost->getMotionCost( 1, 0 );
#if AMP_MRG
      // calculate ME cost
      UInt uiMEError = MAX_UINT;
      UInt uiMECost = MAX_UINT;

      //do a normal motion compensation with the found MVs: is HAD costs actually smaller than merging?
      if (bTestNormalMC)
      {
        xGetInterPredictionError( pcCU, pcOrgYuv, iPartIdx, uiMEError, m_pcEncCfg->getUseHADME() );
        uiMECost = uiMEError + m_pcRdCost->getCost( uiMEBits );
      }
#else
      // calculate ME cost
      UInt uiMEError = MAX_UINT;
      xGetInterPredictionError( pcCU, pcOrgYuv, iPartIdx, uiMEError, m_pcEncCfg->getUseHADME() );
      UInt uiMECost = uiMEError + m_pcRdCost->getCost( uiMEBits );
#endif 
      // save ME result.
      uiMEInterDir = pcCU->getInterDir( uiPartAddr );
      pcCU->getMvField( pcCU, uiPartAddr, REF_PIC_LIST_0, cMEMvField[0] );
      pcCU->getMvField( pcCU, uiPartAddr, REF_PIC_LIST_1, cMEMvField[1] );

      // estimate merge costs
      UInt uiMRGCost = MAX_UINT;
      xMergeEstimation( pcCU, pcOrgYuv, iPartIdx, uiMRGInterDir, cMRGMvField, uiMRGIndex, uiMRGCost, cMvFieldNeighbours, uhInterDirNeighbours, numValidMergeCand);

      // ========================================
      // CHECK: IS MERGE BETTER?
      // ========================================
      if ( uiMRGCost < uiMECost and not disableMerge) //HARP: allow to disable merge
      {
        // set Merge result
        pcCU->setMergeFlagSubParts ( true,          uiPartAddr, iPartIdx, pcCU->getDepth( uiPartAddr ) );
        pcCU->setMergeIndexSubParts( uiMRGIndex,    uiPartAddr, iPartIdx, pcCU->getDepth( uiPartAddr ) );
        pcCU->setInterDirSubParts  ( uiMRGInterDir, uiPartAddr, iPartIdx, pcCU->getDepth( uiPartAddr ) );
        pcCU->getCUMvField( REF_PIC_LIST_0 )->setAllMvField( cMRGMvField[0], ePartSize, uiPartAddr, 0, iPartIdx );
        pcCU->getCUMvField( REF_PIC_LIST_1 )->setAllMvField( cMRGMvField[1], ePartSize, uiPartAddr, 0, iPartIdx );

        pcCU->getCUMvField(REF_PIC_LIST_0)->setAllMvd    ( cMvZero,            ePartSize, uiPartAddr, 0, iPartIdx );
        pcCU->getCUMvField(REF_PIC_LIST_1)->setAllMvd    ( cMvZero,            ePartSize, uiPartAddr, 0, iPartIdx );

        pcCU->setMVPIdxSubParts( -1, REF_PIC_LIST_0, uiPartAddr, iPartIdx, pcCU->getDepth(uiPartAddr));
        pcCU->setMVPNumSubParts( -1, REF_PIC_LIST_0, uiPartAddr, iPartIdx, pcCU->getDepth(uiPartAddr));
        pcCU->setMVPIdxSubParts( -1, REF_PIC_LIST_1, uiPartAddr, iPartIdx, pcCU->getDepth(uiPartAddr));
        pcCU->setMVPNumSubParts( -1, REF_PIC_LIST_1, uiPartAddr, iPartIdx, pcCU->getDepth(uiPartAddr));
      }
      else
      {
        // set ME result
        //setting merge flag of this subpartition to false
        pcCU->setMergeFlagSubParts( false,        uiPartAddr, iPartIdx, pcCU->getDepth( uiPartAddr ) );
        pcCU->setInterDirSubParts ( uiMEInterDir, uiPartAddr, iPartIdx, pcCU->getDepth( uiPartAddr ) );
        pcCU->getCUMvField( REF_PIC_LIST_0 )->setAllMvField( cMEMvField[0], ePartSize, uiPartAddr, 0, iPartIdx );
        pcCU->getCUMvField( REF_PIC_LIST_1 )->setAllMvField( cMEMvField[1], ePartSize, uiPartAddr, 0, iPartIdx );
      }
    }

#ifdef ACTIVATE_HARP
  if(loadINI("Visualization", "VisRDO"))
	  if (Global.isObsCTU())
	  {

	  //QString PUInfoStr = QString(" | PU%1: Af=%2").arg(iPartIdx).arg(cMv[0].isAffine);
		QString PUInfoStr = QString(" | PU%1").arg(iPartIdx);
	  LibGlobal.Show_RDO.ImgMatrix.appendToLastHeadline(PUInfoStr);
	  }
#endif

    LibGlobal.DataTree.append_PU("CUs_RDO", CurPU, pcCU, -1); //-1: Bits_PUPredInfo needs some work here, skipping

    //----------------------------------------------------
    //  IN FACT, THIS DOES ONLY FILL THE PREDYUV-BUFFER... NO MC 
    //----------------------------------------------------
    LibGlobal.isNotGetPredError = true; //avoid logging the getPredError-Function above!
    motionCompensation ( pcCU, rpcPredYuv, REF_PIC_LIST_X, iPartIdx );
    LibGlobal.isNotGetPredError = false;
  } // end for all PUs in CU loop

#ifdef ACTIVATE_HARP
  if(loadINI("Visualization", "VisRDO"))
	  if (Global.isObsCTU())
		  LibGlobal.Show_RDO.append2_predInterSearch(pcCU, pcOrgYuv, rpcPredYuv);
#endif

  setWpScalingDistParam( pcCU, -1, REF_PIC_LIST_X );

  return;
}
