#!/bin/sh

# (c) 2014 Dominic Springer
# This file is licensed under the 2-Clause BSD License (see HARP_License.txt)

set -x #echo on
set -e #exit on failing command

# CHANGE HERE ================================================================
ProjectDir=${PWD##*/}
ProjectBuildDir="${ProjectDir}_Build" #sibling to project dir

EclipseProjectName=${ProjectDir}        #you may choose your own project name        
EclipseBin="$HOME/bin/eclipse/eclipse" #specify eclipse path!
EclipseVersion="4.3"                   #specify eclipse version!
#=============================================================================
echo $ProjectDir
echo $ProjectBuildDir

if [ ! -f $EclipseBin ]; then
    echo "Please install Eclipse 4.3 and set its path accordingly in this script.\n"
    exit 0
fi

cd ..
mkdir -p "$ProjectBuildDir";
cd "$ProjectBuildDir"
SleepSeconds=0

echo "\n"
echo "======================================"
echo "HARP CMake Build System"
echo "======================================"
echo "ProjectDir = $ProjectDir"
echo "ProjectBuildDir = $ProjectBuildDir"
echo "Current working dir = ${PWD##*/}"


# we avoid wildcard rm, since this guy is dangerous
# remove just makes sure that CMake won't cache
#rm -rf ThirdParty
rm -rf CMakeFiles
rm -rf CMakefiles
rm -f Makefile
rm -f CMakeCache.txt
rm -f cmake_install.cmake

if [ ! -f ./.cproject ]; then
    echo "creating and preparing Eclipse project...\n"
    sleep $SleepSeconds
    cmake -G"Eclipse CDT4 - Unix Makefiles" -DPROJECT_NAME=$EclipseProjectName -DCMAKE_ECLIPSE_VERSION=$EclipseVersion -DCMAKE_ECLIPSE_EXECUTABLE=$EclipseBin ../"$ProjectDir"
    #sed -i "s|<name>$EclipseProjectName.*/name>|<name>$EclipseProjectName</name>|g" .project # patching the project name 
else
    echo "existing Eclipse project found, skipping Eclipse project creation...\n"
    sleep $SleepSeconds
    cmake -DPROJECT_NAME=$EclipseProjectName ../"$ProjectDir"
fi

# Create the pydev-project 
# We need it to conveniently set the PythonLib path for source lookups
set +x #echo off
VAR1=$(cat <<EOF
<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<?eclipse-pydev version="1.0"?><pydev_project>
<pydev_property name="org.python.pydev.PYTHON_PROJECT_INTERPRETER">Default</pydev_property>
<pydev_property name="org.python.pydev.PYTHON_PROJECT_VERSION">python 2.7</pydev_property>

<pydev_pathproperty name="org.python.pydev.PROJECT_SOURCE_PATH">
<path>/${PROJECT_DIR_NAME}/[Source directory]/PythonLib</path>
</pydev_pathproperty>
</pydev_project>
EOF
)

echo "${VAR1}" > .pydevproject
set -x #echo on

make -j 4 


